@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -24,7 +24,6 @@ model abstraction for administration clients supporting different views
 """
 
 import sys
-
 import gobject
 
 from twisted.spread import pb
@@ -37,44 +36,32 @@ from flumotion.common import keycards, worker, planet, medium, package, messages
 # serializable worker and component state
 from flumotion.twisted import flavors
 from flumotion.twisted.defer import defer_generator_method
-from flumotion.twisted.compat import implements
 
 from flumotion.configure import configure
-from flumotion.common import reload
+from flumotion.common import reload, connection
 from flumotion.twisted import credentials
 from flumotion.twisted import pb as fpb
+from flumotion.twisted.compat import implements
 
 from flumotion.common.pygobject import gsignal, gproperty
 
 from flumotion.common.messages import N_
 T_ = messages.gettexter('flumotion')
 
-# FIXME: keycard should be created and handed to factory instead
 class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
-    def __init__(self, medium, user, passwd):
+    perspectiveInterface = interfaces.IAdminMedium
+
+    def __init__(self, medium, extraTenacious=False):
         """
-        @type medium:  AdminModel
-        @param user:   username to log in with
-        @param passwd: password to log in with
+        @type medium:   AdminModel
         """
         fpb.ReconnectingFPBClientFactory.__init__(self)
-
-        self.user = user
-        self.passwd = passwd
         self.medium = medium
         self.maxDelay = 20
 
+        self.extraTenacious = extraTenacious
         self.hasBeenConnected = 0
 
-        # FIXME: try more than one auth method
-        #keycard = keycards.KeycardUACPP(user, passwd, 'localhost')
-        keycard = keycards.KeycardUACPCC(user, 'localhost')
-        # FIXME: decide on an admin name ?
-        keycard.avatarId = "admin"
-
-        # start logging in
-        self.startLogin(keycard, medium, interfaces.IAdminMedium)
-
     def clientConnectionMade(self, broker):
       self.hasBeenConnected = 1
 
@@ -86,17 +73,28 @@ class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
         """
         if reason.check(error.DNSLookupError):
             self.debug('DNS lookup error')
-            self.medium.connectionFailed(reason)
+            if not self.extraTenacious:
+                self.medium.connectionFailed(reason)
             return
-        elif reason.check(error.ConnectionRefusedError):
+        elif (reason.check(error.ConnectionRefusedError)
+              or reason.check(error.ConnectError)):
             # If we're logging in for the first time, we want to make this a
             # real error; we present a dialog, etc. 
             # However, if we fail later on (e.g. manager shut down, and 
             # hasn't yet been restarted), we want to keep trying to reconnect,
             # so we just log a message.
-            self.debug("Connection refused error")
-            if not self.hasBeenConnected:
+            self.debug("Error connecting: %s", log.getFailureMessage(reason))
+            if self.hasBeenConnected:
+                self.log("we've been connected before though, so going "
+                         "to retry")
+                # fall through
+            elif self.extraTenacious:
+                self.log("trying again due to +100 tenacity")
+                # fall through
+            else:
+                self.log("telling medium about connection failure")
                 self.medium.connectionFailed(reason)
+                # return
                 return
 
         fpb.ReconnectingFPBClientFactory.clientConnectionFailed(self, 
@@ -109,8 +107,18 @@ class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
         yield d
 
         try:
-            result = d.value()
-            assert result
+            try:
+                result = d.value()
+                assert result
+            except Exception, e:
+                if self.extraTenacious:
+                    self.debug('connection problem: %s', 
+                               log.getExceptionMessage(e))
+                    self.debug('we are tenacious, so trying again later')
+                    self.disconnect()
+                    yield None
+                else:
+                    raise
             # if it's not a reference, we need to respond to a
             # challenge...
             if not isinstance(result, pb.RemoteReference):
@@ -140,13 +148,14 @@ class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
             self.debug("emitted connection-refused")
 
         except Exception, e:
+            self.medium.emit('connection-error', e)
             self.medium._defaultErrback(failure.Failure(e))
 
     gotDeferredLogin = defer_generator_method(gotDeferredLogin)
         
 # FIXME: stop using signals, we can provide a richer interface with actual
 # objects and real interfaces for the views a model communicates with
-class AdminModel(medium.BaseMedium, gobject.GObject):
+class AdminModel(medium.PingingMedium, gobject.GObject):
     """
     I live in the admin client.
     I am a data model for any admin view implementing a UI to
@@ -159,6 +168,7 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
     gsignal('disconnected')
     gsignal('connection-refused')
     gsignal('connection-failed', str)
+    gsignal('connection-error', object)
     gsignal('component-property-changed', str, str, object)
     gsignal('reloading', str)
     gsignal('message', str)
@@ -171,16 +181,17 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
     # Public instance variables (read-only)
     planet = None
 
-    def __init__(self, username, password):
+    def __init__(self, authenticator):
         self.__gobject_init__()
         
         # All of these instance variables are private. Cuidado cabrones!
-        self.user = username
-        self.passwd = password
+        self.authenticator = authenticator
         self.host = self.port = self.use_insecure = None
 
+        self.managerId = '<uninitialized>'
+
         self.state = 'disconnected'
-        self.clientFactory = self._makeFactory(username, password)
+        self.clientFactory = self._makeFactory(authenticator)
         # 20 secs max for an admin to reconnect
         self.clientFactory.maxDelay = 20
 
@@ -188,24 +199,39 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
         self.planet = None
         self._workerHeavenState = None
         
-        self._views = [] # all UI views I am serving
-
     # a method so mock testing frameworks can override it
-    def _makeFactory(self, username, password):
-        return AdminClientFactory(self, username, password)
+    def _makeFactory(self, authenticator):
+        # FIXME: this needs further refactoring, so we only ever pass
+        # an authenticator.  For that we need to fix all users of this
+        # class too
+        factory = AdminClientFactory(self)
+        factory.startLogin(authenticator)
+        return factory
 
-    def connectToHost(self, host, port, use_insecure=False):
+    def connectToHost(self, host, port, use_insecure=False,
+                      keep_trying=False):
         'Connect to a host.'
         self.host = host
         self.port = port
         self.use_insecure = use_insecure
 
+        # the intention here is to give an id unique to the manager --
+        # if a program is adminning multiple managers, this id should
+        # tell them apart (and identify duplicates)
+        info = connection.PBConnectionInfo(host, port, not use_insecure,
+                                           self.authenticator)
+        self.managerId = str(info)
+
+        self.info('Connecting to manager %s with %s',
+                  self.managerId, use_insecure and 'TCP' or 'SSL')
+        if keep_trying:
+            self.info('AdminClientFactory, now with extra tenacity')
+            self.clientFactory.extraTenacious = True
+
         if use_insecure:
-            self.info('Connecting to manager %s:%d with TCP' % (host, port))
             reactor.connectTCP(host, port, self.clientFactory)
         else:
             from twisted.internet import ssl
-            self.info('Connecting to manager %s:%d with SSL' % (host, port))
             reactor.connectSSL(host, port, self.clientFactory,
                                ssl.ClientContextFactory())
 
@@ -221,6 +247,10 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
             map(model.disconnect, ids)
             d.errback(errors.ConnectionFailedError(reason))
 
+        def connection_error(model, exception, d, ids):
+            map(model.disconnect, ids)
+            d.errback(exception)
+
         d = defer.Deferred()
         ids = []
         ids.append(self.connect('connected', connected, d, ids))
@@ -228,6 +258,8 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
                                 connection_refused, d, ids))
         ids.append(self.connect('connection-failed',
                                 connection_failed, d, ids))
+        ids.append(self.connect('connection-error',
+                                connection_error, d, ids))
         return d
 
     # default Errback
@@ -246,7 +278,7 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
 
     # FIXME: give these three sensible names
     def adminInfoStr(self):
-        return '%s@%s:%s' % (self.user, self.host, self.port)
+        return self.managerId
 
     def connectionInfoStr(self):
         return '%s:%s (%s)' % (self.host, self.port,
@@ -257,42 +289,52 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
         assert self.planet
         return '%s (%s:%s)' % (self.planet.get('name'), self.host, self.port)
 
-    def connectionFailed(self, reason):
+    def connectionFailed(self, failure):
         # called by client factory
-        if reason.check(error.DNSLookupError):
-            self.debug('emitting connection-failed')
-            self.emit('connection-failed', "Could not look up host '%s'." %
-                self.host)
-            self.debug('emited connection-failed')
-        if reason.check(error.ConnectionRefusedError):
-            self.debug('emitting connection-failed')
-            self.emit('connection-failed', "Could not connect to host '%s' on port %d." %
-                (self.host, self.port))
-            self.debug('emited connection-failed')
+        if failure.check(error.DNSLookupError):
+            message = "Could not look up host '%s'." % self.host
+        elif (failure.check(error.ConnectionRefusedError)
+              or failure.check(error.ConnectionRefusedError)):
+            message = ("Could not connect to host '%s' on port %d."
+                       % (self.host, self.port))
+        else:
+            message = ("Unexpected failure.\nDebug information: %s"
+                       % log.getFailureMessage (failure))
+        self.debug('emitting connection-failed')
+        self.emit('connection-failed', message)
+        self.debug('emitted connection-failed')
 
     def setRemoteReference(self, remoteReference):
+        self.debug("setRemoteReference %r" % remoteReference)
         def writeConnection():
+            if not (self.authenticator.username
+                    and self.authenticator.password):
+                self.log('not caching connection information')
+                return
             s = ''.join(['<connection>',
                          '<host>%s</host>' % self.host,
                          '<manager>%s</manager>' % self.planet.get('name'),
                          '<port>%d</port>' % self.port,
                          '<use_insecure>%d</use_insecure>' 
                          % (self.use_insecure and 1 or 0),
-                         '<user>%s</user>' % self.user,
-                         '<passwd>%s</passwd>' % self.passwd,
+                         '<user>%s</user>' % self.authenticator.username,
+                         '<passwd>%s</passwd>' % self.authenticator.password,
                          '</connection>'])
             
-            # defgen + pychecker quirkiness
             import os
             import md5
             sum = md5.new(s).hexdigest()
             f = os.path.join(configure.registrydir, '%s.connection' % sum)
-            h = open(f, 'w')
-            h.write(s)
-            h.close()
+            try:
+                h = open(f, 'w')
+                h.write(s)
+                h.close()
+            except Exception, e:
+                self.info('failed to write connection cache file %s: %s',
+                          f, log.getExceptionMessage(e))
 
         # chain up
-        medium.BaseMedium.setRemoteReference(self, remoteReference)
+        medium.PingingMedium.setRemoteReference(self, remoteReference)
 
         # fixme: push the disconnect notification upstream
         def remoteDisconnected(remoteReference):
@@ -308,42 +350,23 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
         # monkey, Monkey, MONKEYPATCH!!!!!
         self.planet.admin = self
         self.debug('got planet state')
-        self.callViews('setPlanetState', self.planet)
 
         d = self.callRemote('getWorkerHeavenState')
         yield d
         self._workerHeavenState = d.value()
         self.debug('got worker state')
-        
+
         writeConnection()
+
         self.debug('Connected to manager and retrieved all state')
         self.state = 'connected'
         self.emit('connected')
     setRemoteReference = defer_generator_method(setRemoteReference)
 
-    def callViews(self, methodName, *args, **kwargs):
-        """
-        Call a method on all views.
-        """
-        for view in self._views:
-            if not hasattr(view, methodName):
-                msg = 'view %r does not implement %s' % (view, methodName)
-                self.warning(msg)
-                raise errors.NoMethodError(msg)
-            m = getattr(view, methodName)
-            m(*args, **kwargs)
-
     ### pb.Referenceable methods
     def remote_log(self, category, type, message):
         self.log('remote: %s: %s: %s' % (type, category, message))
         
-    def remote_componentCall(self, componentState, methodName, *args, **kwargs):
-        """
-        @todo 0.4: rename this method to remote_componentViewCall
-        """
-        self.callViews('componentCall',
-            componentState, methodName, *args, **kwargs)
-
     # IStateListener interface
     def stateSet(self, state, key, value):
         self.debug("state set on %r: key %s" % (state, key))
@@ -356,9 +379,6 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
     def stateRemove(self, state, key, value):
         self.debug("state remove on %r: key %s" % (state, key))
 
-    def remote_shutdown(self):
-        self.debug('shutting down')
-
     ### model functions; called by UI's to send requests to manager or comp
 
     ## view management functions
@@ -366,22 +386,12 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
     def isConnected(self):
         return self.state == 'connected'
 
-    def addView(self, view):
-        # FIXME: implement an IAdminView interface
-        """
-        Add a view as a client to the model.
-        """
-        if not view in self._views:
-            self._views.append(view)
+    def shutdown(self):
+        self.debug('shutting down')
+        if self.state != 'disconnected':
+            self.clientFactory.disconnect()
+        self.clientFactory.stopTrying()
 
-    def removeView(self, view):
-        # FIXME: implement an IAdminView interface
-        """
-        Remove a view as a client to the model.
-        """
-        if view in self._views:
-            self._views.remove(view)
-        
     ## generic remote call methods
     def componentCallRemote(self, componentState, methodName, *args, **kwargs):
         """
@@ -411,7 +421,7 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
                 msg = "Remote method '%s' does not exist." % methodName
                 msg += "\n" + failure.value
             else:
-                msg = failure.value
+                msg = log.getFailureMessage(failure)
 
             # FIXME: we probably need a nicer way of getting component
             # messages shown from the admin model, but this allows us to
@@ -451,6 +461,7 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
         return d
 
     def _callRemoteErrback(self, failure, type, name, methodName):
+        print "THOMAS: errback: failure %r" % failure
         if failure.check(errors.NoMethodError):
             self.warning("method '%s' on component '%s' does not exist, "
                 "component bug" % (methodName, name))
@@ -607,7 +618,12 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
         d = self.workerCallRemote(workerName, 'checkElements', elements)
         d.addErrback(self._defaultErrback)
         return d
-    
+
+    def checkImport(self, workerName, moduleName):
+        d = self.workerCallRemote(workerName, 'checkImport', moduleName)
+        d.addErrback(self._defaultErrback)
+        return d
+     
     def workerRun(self, workerName, moduleName, functionName, *args, **kwargs):
         """
         Run the given function and args on the given worker. If the
