@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -31,6 +31,7 @@ from flumotion.common import log, errors
 # make Message proxyable
 from flumotion.common import messages
 from flumotion.configure import configure
+from flumotion.twisted import pb as fpb
 
 from flumotion.admin.command.commands import commands
 
@@ -45,8 +46,11 @@ def command_usage():
     for name, desc, argspecs, proc in commands:
         sys.stdout.write('  %s -- %s\n' % (name, desc))
         sys.stdout.write('    usage: %s' % name)
-        for name, pred in argspecs:
-            sys.stdout.write(' %s' % name.upper())
+        for spec in argspecs:
+            if len(spec) > 2:
+                sys.stdout.write(' [%s]' % spec[0].upper())
+            else:
+                sys.stdout.write(' %s' % spec[0].upper())
         sys.stdout.write('\n')
 
 def usage(args, exitval=0):
@@ -68,19 +72,44 @@ def parse_commands(args):
     commandspec = matching[0]
 
     argspecs = commandspec[2]
-    if len(args[2:]) != len(argspecs):
-        print 'Error: Invalid arguments to operation %s: %r' % (op, args[2:])
+    reqspecs = [spec for spec in argspecs if len(spec) < 3]
+    nreq = len(reqspecs)
+    optspecs = [spec for spec in argspecs if len(spec) == 3 or \
+        len(spec) > 3 and not spec[3]]
+    nopt = len(optspecs)
+
+    vararg = filter(lambda spec: len(spec) > 3 and spec[3], argspecs)
+
+    # pop off argv[0] and the command name
+    cargs = args[2:]
+
+    if len(cargs) < nreq or len(cargs) > nreq + nopt and not vararg:
+        print 'Error: Invalid arguments to operation %s: %r' % (op, cargs)
         usage(args, exitval=1)
 
     vals = []
-    for argspec, arg in zip(argspecs, args[2:]):
-        name, parse = argspec
+    for name, parse in reqspecs:
+        arg = cargs.pop(0)
         try:
             vals.append(parse(arg))
         except Exception:
             err('Error: Operation %s\'s arg %s="%s" could not be '
                 'parsed as type "%s"'
                 % (op, name, arg, parse.__name__))
+    for name, parse, default in optspecs:
+        if cargs:
+            arg = cargs.pop(0)
+            try:
+                vals.append(parse(arg))
+            except Exception:
+                err('Error: Operation %s\'s arg %s="%s" could not be '
+                    'parsed as type "%s"'
+                    % (op, name, arg, parse.__name__))
+        else:
+            vals.append(default)
+
+    if vararg:
+        vals.extend(cargs)
 
     proc = commandspec[3]
 
@@ -89,89 +118,27 @@ def parse_commands(args):
 
     return command
 
-def setup_reactor(connection):
-    model = AdminModel(connection['user'], connection['passwd'])
-    d = model.connectToHost(connection['host'], connection['port'],
-                            connection['use_insecure'])
-
-    def refused(failure):
-        failure.trap(errors.ConnectionRefusedError)
-        print "Manager refused connection. Check your user and password."
-        raise
+def setup_reactor(info):
+    model = AdminModel(info.authenticator)
+    d = model.connectToHost(info.host, info.port, not info.use_ssl)
 
     def failed(failure):
-        failure.trap(errors.ConnectionFailedError)
-        message = "".join(failure.value.args)
-        print "Connection to manager failed: %s" % message
-        raise
+        if failure.check(errors.ConnectionRefusedError):
+            print "Manager refused connection. Check your user and password."
+        elif failure.check(errors.ConnectionFailedError):
+            message = "".join(failure.value.args)
+            print "Connection to manager failed: %s" % message
+        else:
+            print ("Exception while connecting to manager: %s"
+                   % log.getFailureMessage(failure))
+        return failure
 
-    d.addErrback(refused)
     d.addErrback(failed)
 
     return d
 
 pat = re.compile('^(([^:@]*)(:([^:@]+))?@)?([^:@]+)(:([0-9]+))?$')
 
-def parse_connection(manager_string, use_insecure):
-    recent = connections.get_recent_connections()
-
-    if manager_string:
-        matched = pat.search(manager_string)
-        if not matched:
-            err('invalid manager string: %s '
-                '(looking for [user[:pass]@]host[:port])'
-                % manager_string)
-
-        groups = matched.groups()
-        ret = {}
-        for k, v in (('user', 1),
-                     ('passwd', 3),
-                     ('host', 4),
-                     ('port', 6)):
-            ret[k] = groups[v]
-        ret['use_insecure'] = bool(use_insecure)
-        if ret['port']:
-            ret['port'] = int(ret['port'])
-        else:
-            if use_insecure:
-                ret['port'] = configure.defaultTCPManagerPort
-            else:
-                ret['port'] = configure.defaultSSLManagerPort
-
-        def compatible(d1, d2, *keys):
-            for k in keys:
-                if d1[k] and d1[k] != d2[k]:
-                    return False
-            return True
-
-        if not ret['user']:
-            for c in recent:
-                state = c['state']
-                if compatible(ret, state, 'host', 'port', 'use_insecure'):
-                    ret['user'] = state['user']
-                    ret['passwd'] = state['passwd']
-                    break
-        elif not ret['passwd']:
-            for c in recent:
-                state = c['state']
-                if compatible(ret, state, 'host', 'port', 'use_insecure',
-                              'user'):
-                    ret['passwd'] = state['passwd']
-                    break
-        if not (ret['user'] and ret['passwd']):
-            err('You are connecting to %s for the first time; please '
-                'specify a user and password (e.g. user:test@%s).'
-                % (manager_string, manager_string))
-
-        for k, v in ret.items():
-            assert v is not None, '%s unset, internal error' % k
-
-        return ret
-    elif recent:
-        return recent[0]['state']
-    else:
-        err('Missing --manager, and no recent connections to use.')
-
 def main(args):
     parser = optparse.OptionParser()
     parser.add_option('-d', '--debug',
@@ -204,7 +171,8 @@ def main(args):
     if options.usage or not args[1:]:
         usage(args)
 
-    connection = parse_connection(options.manager, options.no_ssl)
+    connection = connections.parsePBConnectionInfo(options.manager,
+                                                   not options.no_ssl)
 
     command = parse_commands(args)
     quit = lambda: reactor.callLater(0, reactor.stop)
