@@ -1,8 +1,9 @@
 # -*- Mode: Python -*-
+# -*- coding: UTF-8 -*-
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -114,9 +115,6 @@ class Window(log.Loggable, gobject.GObject):
             self.property_changed_cb)
         self.admin.connect('update', self.admin_update_cb)
 
-        # set ourselves as a view for the admin model
-        self.admin.addView(self)
-
     # default Errback
     def _defaultErrback(self, failure):
         self.warning('Errback: unhandled failure: %s' %
@@ -161,11 +159,12 @@ class Window(log.Loggable, gobject.GObject):
         self._trayicon.set_tooltip(_('Not connected'))
 
         # the widget containing the component view
-        self._sidepane = widgets['hpaned']
+        self._component_view = widgets['component_view']
+        self._component_view_clear()
  
         window.connect('delete-event', self.close)
 
-        self.components_view = parts.ComponentsView(widgets['component_view'])
+        self.components_view = parts.ComponentsView(widgets['components_view'])
         self.components_view.connect('has-selection', 
             self._components_view_has_selection_cb)
         self.components_view.connect('activated',
@@ -183,12 +182,12 @@ class Window(log.Loggable, gobject.GObject):
 
         return window
 
-    def on_open_connection(self, config):
-        model = AdminModel(config['user'], config['passwd'])
-        d = model.connectToHost(config['host'], config['port'],
-                                config['use_insecure'])
+    def on_open_connection(self, connectionInfo):
+        i = connectionInfo
+        model = AdminModel(i.authenticator)
+        d = model.connectToHost(i.host, i.port, not i.use_ssl)
         self._trayicon.set_tooltip(_("Connecting to %s:%s") %
-            (config['host'], config['port']))
+            (i.host, i.port))
 
         def connected(model):
             self.window.set_sensitive(True)
@@ -196,16 +195,19 @@ class Window(log.Loggable, gobject.GObject):
             self._append_recent_connections()
 
         def refused(failure):
-            failure.trap(errors.ConnectionRefusedError)
-            dialogs.connection_refused_modal_message(config['host'],
-                                                     self.window)
-            self.window.set_sensitive(True)
+            if failure.check(errors.ConnectionRefusedError):
+                d = dialogs.connection_refused_message(i.host,
+                                                       self.window)
+            else:
+                d = dialogs.connection_failed_message(i.host,
+                                                      self.window)
+            d.addCallback(lambda _: self.window.set_sensitive(True))
 
-        d.addCallback(connected).addErrback(refused)
+        d.addCallbacks(connected, refused)
         self.window.set_sensitive(False)
 
-    def on_recent_activate(self, widget, state):
-        self.on_open_connection(state)
+    def on_recent_activate(self, widget, connectionInfo):
+        self.on_open_connection(connectionInfo)
 
     def _append_recent_connections(self):
         menu = self.widgets['connection_menu'].get_submenu()
@@ -228,7 +230,7 @@ class Window(log.Loggable, gobject.GObject):
             gtk.MenuShell.append(menu, i) # $%^&* pychecker
         def append_txt(c, n):
             i = gtk.MenuItem(c['name'])
-            i.connect('activate', self.on_recent_activate, c['state'])
+            i.connect('activate', self.on_recent_activate, c['info'])
             append(i)
             
         append(gtk.SeparatorMenuItem())
@@ -246,6 +248,8 @@ class Window(log.Loggable, gobject.GObject):
     # FIXME: this method uses a file and a methodname as entries
     # FIXME: do we want to switch to imports instead so the whole file
     # is available in its namespace ?
+    # FIXME: factor this out into a ComponentView or SidePaneView class
+    # so we can reuse it
     def show_component(self, state, entryPath, fileName, methodName, data):
         """
         Show the user interface for this component.
@@ -317,24 +321,36 @@ class Window(log.Loggable, gobject.GObject):
         # instantiate the GUIClass
         instance = klass(state, self.admin)
         self.debug("Created entry instance %r" % instance)
-        d = instance.setup()
-        # F0.2: setup should return a deferred
+        self._instanceSetup(instance, klass, name)
+
+    def _instanceSetup(self, instance, klass, name):
+        self.debug('Setting up instance %r' % instance)
+        msg = None
+        d = None
+        try:
+            d = instance.setup()
+        except Exception, e:
+            msg = log.getExceptionMessage(e)
+        self.debug('Setup instance %r' % instance)
         if not d:
-            d = defer.succeed(None)
+            msg = "%r.setup() should return a deferred" % klass
+
+        if msg:
+            self.warning('Component UI bug: %s' % msg)
+            m = messages.Error(T_(
+                N_("This component has a UI bug.")),
+                    debug=msg,
+                    id=name)
+            self._messages_view.add_message(m)
+            return
+
         d.addCallback(self._setupCallback, name, instance)
+        d.addErrback(self._setupErrback, name)
 
     def _setupCallback(self, result, name, instance):
         notebook = gtk.Notebook()
         nodeWidgets = {}
         nodes = instance.getNodes()
-
-        # F0.2
-        for nodeName, node in nodes.items():
-            if not node.title:
-                self.warning('Node %s does not have a title, please fix' %
-                    nodeName)
-                node.title = nodeName
-
         self.statusbar.clear('main')
         # create pages for all nodes, and just show a loading label for
         # now
@@ -348,10 +364,7 @@ class Window(log.Loggable, gobject.GObject):
             notebook.append_page(table, gtk.Label(node.title))
             
         # put "loading" widget in
-        old = self._sidepane.get_child2()
-        self._sidepane.remove(old)
-        self._sidepane.add2(notebook)
-        notebook.show_all()
+        self._component_view_set_widget(notebook)
 
         # trigger node rendering
         d = defer.Deferred()
@@ -371,6 +384,17 @@ class Window(log.Loggable, gobject.GObject):
         d.callback(None)
         return d
 
+    def _setupErrback(self, failure, name):
+        self.warning('Could not setup component %s' % name)
+        msg = 'Could not setup component %s: %s' % (name,
+            log.getFailureMessage(failure))
+        self.debug(msg)
+        m = messages.Error(T_(
+                N_("This component has a UI bug.")),
+                    debug=msg,
+                    id=name)
+        self._messages_view.add_message(m)
+
     # called when one node gets rendered
     def _nodeRenderCallback(self, widget, nodeName, nodeWidgets, mid):
         # used by show_component
@@ -394,58 +418,22 @@ class Window(log.Loggable, gobject.GObject):
         widget.show()
 
     def _nodeRenderErrback(self, failure, nodeName):
-        self.debug('Could not render node %s: %s' % (nodeName,
-            log.getFailureMessage(failure)))
         self.warning('Could not render node %s' % nodeName)
+        debug = log.getFailureMessage(failure)
+        if failure.check(errors.NoBundleError):
+            debug = "Could not get bundle %s" % failure.value.args[0]
+        msg = 'Could not render node %s: %s' % (nodeName, debug)
+        self.debug(msg)
+        m = messages.Error(T_(
+                N_("This component has a UI bug in the %s tab."), nodeName),
+                    debug=msg,
+                    id=nodeName)
+        self._messages_view.add_message(m)
 
     def _setCurrentComponentCallback(self, _, instance):
         self.debug('setting current_component to %r' % instance)
         self.current_component = instance
 
-
-    ### IAdminView interface methods: FIXME: create interface somewhere
-    ## Confusingly enough, this procedure is called by remote objects to
-    ## operate on the client ui. I think. It is *not* for calling
-    ## methods on the remote components. Should fix this sometime.
-    def componentCall(self, componentState, methodName, *args, **kwargs):
-        # FIXME: for now, we only allow calls to go through that have
-        # their UI currently displayed.  In the future, maybe we want
-        # to create all UI's at startup regardless and allow all messages
-        # to be processed, since they're here now anyway   
-        self.log("componentCall received for %r.%s ..." % (
-            componentState, methodName))
-        state = self.components_view.get_selected_state()
-        if not state:
-            self.log("... but no component selected")
-            return
-        if componentState != state:
-            self.log("... but component is different from displayed")
-            return
-        if not self.current_component:
-            self.log("... but component is not yet shown")
-            return
-        
-        name = state.get('name')
-        localMethodName = "component_%s" % methodName
-        if not hasattr(self.current_component, localMethodName):
-            self.log("... but does not have method %s" % localMethodName)
-            self.warning("Component view %s does not implement %s" % (
-                name, localMethodName))
-            return
-        self.log("... and executing")
-        method = getattr(self.current_component, localMethodName)
-
-        # call the method, catching all sorts of stuff
-        try:
-            result = method(*args, **kwargs)
-        except TypeError:
-            msg = "component method %s did not accept *a %s and **kwa %s (or TypeError)" % (
-                methodName, args, kwargs)
-            self.debug(msg)
-            raise errors.RemoteRunError(msg)
-        self.log("component: returning result: %r to caller" % result)
-        return result
-
     def componentCallRemoteStatus(self, state, pre, post, fail,
                                   methodName, *args, **kwargs):
         if not state:
@@ -482,32 +470,59 @@ class Window(log.Loggable, gobject.GObject):
                                        methodName, *args, **kwargs)
 
     def setPlanetState(self, planetState):
+        def flowStateAppend(state, key, value):
+            self.debug('flow state append: key %s, value %r' % (key, value))
+            if key == 'components':
+                self._components[value.get('name')] = value
+                # FIXME: would be nicer to do this incrementally instead
+                self.update_components()
+
+        def flowStateRemove(state, key, value):
+            if key == 'components':
+                self._remove_component(value)
+
+        def atmosphereStateAppend(state, key, value):
+            if key == 'components':
+                self._components[value.get('name')] = value
+                # FIXME: would be nicer to do this incrementally instead
+                self.update_components()
+
+        def atmosphereStateRemove(state, key, value):
+            if key == 'components':
+                self._remove_component(value)
+
+        def planetStateAppend(state, key, value):
+            if key == 'flows':
+                if value != state.get('flows')[0]:
+                    self.warning('flumotion-admin can only handle one '
+                                 'flow, ignoring /%s', value.get('name'))
+                    return
+                self.debug('%s flow started', value.get('name'))
+                value.addListener(self, append=flowStateAppend,
+                                  remove=flowStateRemove)
+                for c in value.get('components'):
+                    flowStateAppend(value, 'components', c)
+
+        def planetStateRemove(state, key, value):
+            self.debug('something got removed from the planet')
+
         self.debug('parsing planetState %r' % planetState)
         self._planetState = planetState
 
         # clear and rebuild list of components that interests us
         self._components = {}
 
-        planetState.addListener(self)
+        planetState.addListener(self, append=planetStateAppend,
+                                remove=planetStateRemove)
 
         a = planetState.get('atmosphere')
-        a.addListener(self)
-
+        a.addListener(self, append=atmosphereStateAppend,
+                      remove=atmosphereStateRemove)
         for c in a.get('components'):
-            name = c.get('name')
-            self.debug('adding atmosphere component "%s"' % name)
-            self._components[name] = c
+            atmosphereStateAppend(a, 'components', c)
             
         for f in planetState.get('flows'):
-            if f.get('name') != 'default':
-                continue
-            f.addListener(self)
-            for c in f.get('components'):
-                name = c.get('name')
-                self.debug('adding default flow component "%s"' % name)
-                self._components[name] = c
-
-        self.update_components()
+            planetStateAppend(planetState, 'flows', f)
  
     def stateSet(self, state, key, value):
         # called by model when state of something changes
@@ -522,70 +537,15 @@ class Window(log.Loggable, gobject.GObject):
             if value == moods.sleeping.value:
                 if state.get('name') == current:
                     self._messages_view.clear()
-                    self._sidepane_clear()
+                    self._component_view_clear()
 
-    def stateAppend(self, state, key, value):
-        if isinstance(state, worker.AdminWorkerHeavenState):
-            if key == 'names':
-                self.statusbar.set('main', 'Worker %s logged in.' % value)
-        elif isinstance(state, planet.AdminFlowState):
-            self.debug('flow state append: key %s, value %r' % (key, value))
-            if state.get('name') != 'default':
-                return
-            if key == 'components':
-                self._components[value.get('name')] = value
-                # FIXME: would be nicer to do this incrementally instead
-                self.update_components()
-        elif isinstance(state, planet.AdminAtmosphereState):
-            if key == 'components':
-                self._components[value.get('name')] = value
-                # FIXME: would be nicer to do this incrementally instead
-                self.update_components()
-        elif isinstance(state, planet.AdminPlanetState):
-            if key == 'flows':
-                if value.get('name') != 'default':
-                    return
-                self.debug('default flow started')
-                value.addListener(self)
-        elif isinstance(state, planet.AdminComponentState):
-            name = state.get('name')
-            self.debug('stateAppend on component state of %s' % name)
-            if key == 'messages':
-                current = self.components_view.get_selected_name()
-                if name == current:
-                    self._messages_view.add_message(value)
-        else:
-            self.warning('stateAppend on unknown object %r' % state)
+    def whsAppend(self, state, key, value):
+        if key == 'names':
+            self.statusbar.set('main', 'Worker %s logged in.' % value)
 
-    def stateRemove(self, state, key, value):
-        self.debug('stateRemove on %r for key %s and value %r' % (
-            state, key, value))
-        if isinstance(state, worker.AdminWorkerHeavenState):
-            if key == 'names':
-                self.statusbar.set('main', 'Worker %s logged out.' % value)
-        elif isinstance(state, planet.AdminFlowState):
-            # the basic admin client only looks at the "default" flow
-            if state.get('name') != 'default':
-                return
-
-            if key == 'components':
-                self._remove_component(value)
-        elif isinstance(state, planet.AdminAtmosphereState):
-            if key == 'components':
-                self._remove_component(value)
-        elif isinstance(state, planet.AdminPlanetState):
-            self.debug('something got removed from the planet')
-            pass
-        elif isinstance(state, planet.AdminComponentState):
-            name = state.get('name')
-            self.debug('stateRemove on component state of %s' % name)
-            if key == 'messages':
-                current = self.components_view.get_selected_name()
-                if name == current:
-                    self._messages_view.clear_message(value.id)
-            self._set_stop_start_component_sensitive()
-        else:
-            self.warning('stateRemove of key %s and value %r on unknown object %r' % (key, value, state))
+    def whsRemove(self, state, key, value):
+        if key == 'names':
+            self.statusbar.set('main', 'Worker %s logged out.' % value)
 
     def _remove_component(self, state):
         name = state.get('name')
@@ -606,6 +566,10 @@ class Window(log.Loggable, gobject.GObject):
 
     ### admin model callbacks
     def admin_connected_cb(self, admin):
+        if self._planetState:
+            self._planetState.removeListener(self)
+            self._planetState = None
+        
         self.info('Connected to manager')
         if self._disconnected_dialog:
             self._disconnected_dialog.destroy()
@@ -720,20 +684,54 @@ class Window(log.Loggable, gobject.GObject):
 
     # clear the component view in the sidepane.  Called when the current
     # component goes sleeping
-    def _sidepane_clear(self):
-        old = self._sidepane.get_child2()
-        self._sidepane.remove(old)
-        sub = gtk.Label("")
-        self._sidepane.add2(sub)
-        sub.show()
+    def _component_view_clear(self):
+        empty = gtk.Label("")
+        self._component_view_set_widget(empty)
+
+    # set the given widget in the component view
+    def _component_view_set_widget(self, widget):
+        for c in self._component_view.get_children():
+            self._component_view.remove(c)
+        self._component_view.add(widget)
+        widget.show_all()
 
     ### ui callbacks
     def _components_view_has_selection_cb(self, view, state):
+        def compSet(state, key, value):
+            if key == 'message':
+                self.statusbar.set('main', value)
+            elif key == 'mood':
+                self._set_stop_start_component_sensitive()
+                current = self.components_view.get_selected_name()
+                if value == moods.sleeping.value:
+                    if state.get('name') == current:
+                        self._messages_view.clear()
+                        self._component_view_clear()
+
+        def compAppend(state, key, value):
+            name = state.get('name')
+            self.debug('stateAppend on component state of %s' % name)
+            if key == 'messages':
+                current = self.components_view.get_selected_name()
+                if name == current:
+                    self._messages_view.add_message(value)
+
+        def compRemove(state, key, value):
+            name = state.get('name')
+            self.debug('stateRemove on component state of %s' % name)
+            if key == 'messages':
+                current = self.components_view.get_selected_name()
+                if name == current:
+                    self._messages_view.clear_message(value.id)
+            self._set_stop_start_component_sensitive()
+
         if self.current_component_state:
             self.current_component_state.removeListener(self)
         self.current_component_state = state
         if self.current_component_state:
-            self.current_component_state.addListener(self)
+            self.current_component_state.addListener(self, compSet,
+                                                     compAppend,
+                                                     compRemove)
 
         self._set_stop_start_component_sensitive()
 
@@ -745,13 +743,7 @@ class Window(log.Loggable, gobject.GObject):
         mood = state.get('mood')
         messages = state.get('messages')
         self._messages_view.clear()
-        # no ui, clear; FIXME: do this nicer
-        old = self._sidepane.get_child2()
-        self._sidepane.remove(old)
-        #sub = gtk.Label('%s does not have a UI yet' % name)
-        sub = gtk.Label("")
-        self._sidepane.add2(sub)
-        sub.show()
+        self._component_view_clear()
 
         if messages:
             for m in messages:
@@ -786,9 +778,12 @@ class Window(log.Loggable, gobject.GObject):
 
         def gotEntryNoBundleErrback(failure):
             failure.trap(errors.NoBundleError)
+            self.debug("Making generic UI for component %s" % name)
 
-            self.debug("No UI for component %s" % name)
-            self.statusbar.set('main', _("No UI for component %s") % name)
+            # make a generic ui
+            from flumotion.component.base import admin_gtk
+            instance = admin_gtk.BaseAdminGtk(state, self.admin)
+            self._instanceSetup(instance, admin_gtk.BaseAdminGtk, name)
 
         def gotEntrySleepingComponentErrback(failure):
             failure.trap(errors.SleepingComponentError)
@@ -891,7 +886,7 @@ class Window(log.Loggable, gobject.GObject):
 
         dialog = dialogs.ProgressDialog("Reloading",
             _("Reloading component code for %s") % name, self.window)
-        d = self.admin.reloadComponent(name)
+        d = self.admin.reloadComponent(state)
         d.addCallback(lambda result, d: d.destroy(), dialog)
         # add error
         d.addErrback(lambda failure, d: d.destroy(), dialog)
@@ -929,7 +924,7 @@ class Window(log.Loggable, gobject.GObject):
             return None
 
         mid = self.statusbar.push('main', "%s component %s" % (doing, name))
-        d = self.admin.callRemote('component'+action, state)
+        d = self.admin.callRemote('component' + action, state)
 
         def _actionCallback(result, self, mid):
             self.statusbar.remove('main', mid)
@@ -948,9 +943,9 @@ class Window(log.Loggable, gobject.GObject):
  
  
     # menubar/toolbar callbacks
-    def on_have_connection(self, d, state):
+    def on_have_connection(self, d, connectionInfo):
         d.destroy()
-        self.on_open_connection(state)
+        self.on_open_connection(connectionInfo)
 
     def file_open_cb(self, button):
         d = gtkconnections.ConnectionsDialog(self.window)
@@ -1067,10 +1062,26 @@ class Window(log.Loggable, gobject.GObject):
  
     def debug_start_shell_cb(self, button):
         if sys.version_info[1] >= 4:
-            from flumotion.common import code
+            from flumotion.extern import code
         else:
             import code
-        code.interact()
+
+        vars = \
+            {
+                "admin": self.admin,
+                "components": self._components
+            }
+        message = ("  Flumotion Admin Debug Shell\n"
+                   "\n"
+                   "Local variables are:\n"
+                   "  admin      (flumotion.admin.admin.AdminModel)\n"
+                   "  components (dict: name -> flumotion.common.planet.AdminComponentState)\n"
+                   "\n"
+                   "You can do remote component calls using:\n"
+                   "  admin.componentCallRemote(components['component-name'],\n"
+                   "         'methodName', arg1, arg2)\n\n")
+
+        code.interact(local=vars, banner=message)
 
     def help_about_cb(self, button):
         dialog = gtk.Dialog(_('About Flumotion'), self.window,
@@ -1092,11 +1103,16 @@ class Window(log.Loggable, gobject.GObject):
         version.set_use_markup(True)
         version.show()
 
-        text = _('Flumotion is a streaming media server.\n\n(C) 2004-2005 Fluendo S.L.')
+        text = _('Flumotion is a streaming media server.\n\n'
+            '© 2004, 2005, 2006, 2007 Fluendo S.L.')
         authors = ('Andy Wingo',
                    'Johan Dahlin',
+                   'Mike Smith',
                    'Thomas Vander Stichele',
-                   'Wim Taymans')
+                   'Wim Taymans',
+                   'Zaheer Abbas Merali',
+                   'Sébastien Merle'
+        )
         text += '\n\n<small>' + _('Authors') + ':\n'
         for author in authors:
             text += '  %s\n' % author
