@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -76,7 +76,7 @@ class Connections(GladeWidget):
         for x in connections.get_recent_connections():
             i = self.model.append()
             self.model.set(i, self.STR_COL, x['name'], self.FILE_COL, x['file'],
-                           self.STATE_COL, x['state'])
+                           self.STATE_COL, x['info'])
 
     def _clear_iter(self, i):
         os.unlink(self.model.get_value(i, self.FILE_COL))
