@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -26,51 +26,60 @@ import sys
 import gettext
 import gtk.glade
 
-from twisted.internet import reactor
-
+from twisted.internet import reactor, defer
+from flumotion.admin import connections
 from flumotion.admin.admin import AdminModel
 from flumotion.admin.gtk import dialogs
 from flumotion.admin.gtk.client import Window
 from flumotion.common import log, errors
 from flumotion.configure import configure
+from flumotion.twisted import pb as fpb
 
-def _runInterface(conf_file, options, thegreeter=None):
-    if conf_file:
-        # load the conf file here
-        raise NotImplementedError()
-
+def Greeter():
     # We do the import here so gettext has been set up and class strings
     # from greeter are translated
     from flumotion.admin.gtk import greeter
-    g = thegreeter or greeter.Greeter()
-    state = g.run()
-    if not state:
-        reactor.callLater(0, reactor.stop)
-        return
-    g.set_sensitive(False)
+    return greeter.Greeter()
 
-    model = AdminModel(state['user'], state['passwd'])
-    d = model.connectToHost(state['host'], state['port'], state['use_insecure'])
+def startAdminFromGreeter(greeter):
+    def got_state(state):
+        greeter.set_sensitive(False)
+        authenticator = fpb.Authenticator(username=state['user'],
+                                          password=state['passwd'])
+        model = AdminModel(authenticator)
+        return model.connectToHost(state['host'], state['port'],
+                                   state['use_insecure'])
 
-    def connected(model, greeter):
-        greeter.destroy()
-        Window(model).show()
-
-    def refused(failure, greeter):
+    def refused(failure):
         failure.trap(errors.ConnectionRefusedError)
-        dialogs.connection_refused_modal_message(state['host'],
-                                                 greeter.window)
-        _runInterface(None, None, greeter)
+        dret = dialogs.connection_refused_message(state['host'],
+                                                  greeter.window)
+        dret.addCallback(lambda _: startAdminFromGreeter(greeter))
+        return dret
 
-    def failed(failure, greeter):
+    def failed(failure):
         failure.trap(errors.ConnectionFailedError)
         message = "".join(failure.value.args)
-        dialogs.connection_failed_modal_message(message, greeter.window)
-        _runInterface(None, None, greeter)
+        dret = dialogs.connection_failed_message(message, greeter.window)
+        dret.addCallback(lambda _: startAdminFromGreeter(greeter))
+        return dret
 
-    d.addCallback(connected, g)
-    d.addErrback(refused, g)
-    d.addErrback(failed, g)
+    def connected(model):
+        greeter.destroy()
+        return model
+
+    d = greeter.run_async()
+    d.addCallback(got_state)
+    d.addCallback(connected)
+    d.addErrback(refused)
+    d.addErrback(failed)
+    return d
+
+def startAdminFromManagerString(managerString, useSSL):
+    info = connections.parsePBConnectionInfo(managerString, useSSL)
+    model = AdminModel(info.authenticator)
+    d = model.connectToHost(info.host, info.port, not info.use_ssl)
+    return d
 
 def main(args):
     parser = optparse.OptionParser()
@@ -84,7 +93,13 @@ def main(args):
                       action="store_true", dest="version",
                       default=False,
                       help="show version information")
-    
+    parser.add_option('-m', '--manager',
+                      action="store", type="string", dest="manager",
+                      help="the manager to connect to, e.g. localhost:7531")
+    parser.add_option('', '--no-ssl',
+                      action="store_true", dest="no_ssl",
+                      help="disable encryption when connecting to the manager")
+ 
     options, args = parser.parse_args(args)
 
     if options.version:
@@ -106,15 +121,23 @@ def main(args):
     gtk.glade.bindtextdomain('flumotion', localedir)
     gtk.glade.textdomain('flumotion')
 
-    conf_files = args[1:]
-
-    if conf_files and len(conf_files) > 1:
+    if len(args) > 1:
         log.error('flumotion-admin',
-            'too many configuration files: %r' % conf_files)
+                  'too many argument: %r' % (args[1:],))
 
-    elif conf_files:
-        _runInterface(conf_files[0], options)
+    if options.manager:
+        d = startAdminFromManagerString(options.manager,
+                                        not options.no_ssl)
     else:
-        _runInterface(None, options)
+        d = startAdminFromGreeter(Greeter())
+
+    def failure(failure):
+        message = "".join(failure.value.args)
+        log.warning('admin', "Failed to connect: %s",
+                    log.getFailureMessage(failure))
+        sys.stderr.write("Connection to manager failed: %s\n" % message)
+        reactor.stop()
+
+    d.addCallbacks(lambda model: Window(model).show(), failure)
 
     reactor.run()
