@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -30,7 +30,7 @@ import gtk.glade
 
 from flumotion.configure import configure
 
-from flumotion.common import pygobject
+from flumotion.common import pygobject, log
 from flumotion.common.pygobject import gsignal
 from flumotion.ui.glade import GladeWidget, GladeWindow
 
@@ -81,16 +81,12 @@ from flumotion.ui.glade import GladeWidget, GladeWindow
 # in the example above. STEP1... are the WizardStep specialized classes (not
 # instances).
 #
-# The wizard is run with the run() method. It returns the state object
-# accumulated by the pages, or None if the user closes the wizard window.
-#
-# w = Wizard('foo', 'first-step', FirstStep)
-# w.show()
-# w.run() => {'bank-account': 'foo'}
 
+class WizardCancelled(Exception):
+    pass
 
 # fixme: doc vmethods
-class WizardStep(GladeWidget):
+class WizardStep(GladeWidget, log.Loggable):
     # all values filled in by subclasses
     name = None
     title = None
@@ -99,8 +95,16 @@ class WizardStep(GladeWidget):
     next_pages = None
     # also, all widgets from the glade file will become attributes
     page = None # filled from glade
+    wizard = None # set by wizard
+    logCategory = "wizardstep"
 
-    def __init__(self, glade_prefix=''):
+    def __init__(self, wizard, glade_prefix=''):
+        """
+        @type  wizard:       L{Wizard}
+        @type  glade_prefix: str
+        @param glade_prefix: prefix used for glade files for the step
+        """
+        self.wizard = wizard
         self.glade_file = glade_prefix + self.name + '.glade'
         GladeWidget.__init__(self)
 
@@ -112,10 +116,17 @@ class WizardStep(GladeWidget):
         pass
 
 class Wizard(GladeWindow):
-    '''
+    """
     A generic wizard.
-    '''
 
+    This wizard runs its own GObject main loop.
+    The wizard is run with the run() method.
+    
+    Example:
+      w = Wizard('foo', 'first-step', FirstStep)
+      w.show()
+      w.run() => {'bank-account': 'foo'}
+    """
     # should by filled by subclasses
     name = None
     steps = []
@@ -130,17 +141,22 @@ class Wizard(GladeWindow):
     gsignal('finished')
 
     def __init__(self, initial_page):
+        """
+        @type  initial_page: str
+        @param initial_page: name of the WizardStep to start on
+        """
         GladeWindow.__init__(self)
 
         # these should be filled by subclasses
         assert self.name
         assert self.steps
 
-        for x in self.steps:
-            p = self.pages[x.name] = x(self.name+'-')
-            p.show()
+        # instantiate steps
+        for cls in self.steps:
+            page = cls(self, self.name + '-')
+            self.pages[cls.name] = page
+            page.show()
 
-        self.loop = gobject.MainLoop()
         self._setup_ui()
         self.set_page(initial_page)
 
@@ -193,43 +209,75 @@ class Wizard(GladeWindow):
 
     def on_delete_event(self, *window):
         self.state = None
-        self.loop.quit()
+        self.emit('finished')
 
-    def on_next(self, *button):
+    def on_next(self, button):
+        button.set_sensitive(False)
         next_page = self.page.on_next(self.state)
         
         if not next_page:
             # the input is incorrect
             pass
         elif next_page == '*finished*':
+            button.set_sensitive(True)
             self.emit('finished')
+        elif next_page == '*signaled*':
+            # page wants to do more stuff and will signal us next_page when done
+            def on_finished(page, next_page):
+                button.set_sensitive(True)
+                self.page.disconnect(self._finished_id)
+                if next_page == '*finished*':
+                    self.emit('finished')
+                else:
+                    self.page_stack.append(self.page)
+                    self.set_page(next_page)
+            self._finished_id = self.page.connect('finished', on_finished)
         else:
+            button.set_sensitive(True)
             self.page_stack.append(self.page)
             self.set_page(next_page)
         
     def on_prev(self, button):
-        self.set_page(self.page_stack.pop().name)
-
-    def show(self):
-        self.window.show()
-
-    def destroy(self):
-        assert hasattr(self, 'window')
-        self.window.destroy()
-        del self.window
+        page = self.page_stack.pop()
+        self.set_page(page.name)
 
     def set_sensitive(self, is_sensitive):
         self.window.set_sensitive(is_sensitive)
 
     def run(self):
+        """
+        Run in a recursive main loop. Will block until the user finishes
+        or closes the wizard.
+        """
+        loop = gobject.MainLoop()
+        d = self.run_async()
+        def async_done(_):
+            loop.quit()
+        d.addCallbacks(async_done, async_done)
+        loop.run()
+        return self.state
+
+    def run_async(self):
+        """
+        Show the wizard. Returns a deferred that fires when the user has
+        closed the wizard, either via completing the process or has
+        cancelled somehow.
+
+        @returns: a deferred that will fire the state dict accumulated
+        by the pages, or None on cancel
+        """
         assert self.window
         self.set_sensitive(True)
         self.show()
+        from twisted.internet import defer
+        d = defer.Deferred()
         def finished(x):
             self.disconnect(i)
-            self.loop.quit()
+            if self.state:
+                d.callback(self.state)
+            else:
+                d.errback(WizardCancelled())
         i = self.connect('finished', finished)
-        self.loop.run()
-        return self.state
+        return d
 
 pygobject.type_register(Wizard)
