@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -23,9 +23,10 @@
 bundles of files used to implement caching over the network
 """
 
+import errno
 import md5
 import os
-import errno
+import sys
 import zipfile
 import StringIO
 
@@ -286,10 +287,10 @@ class BundlerBasket:
         @type dependencies: list of strings
         """
         # note that a bundler doesn't necessarily need to be registered yet
-        if not self._graph.hasObject(depender):
+        if not self._graph.hasNode(depender):
             self._graph.addNode(depender)
         for dep in dependencies:
-            if not self._graph.hasObject(dep):
+            if not self._graph.hasNode(dep):
                 self._graph.addNode(dep)
             self._graph.addEdge(depender, dep)
 
@@ -326,3 +327,122 @@ class BundlerBasket:
         if self._files.has_key(filename):
             return self._files[filename]
         return None
+
+class MergedBundler(Bundler):
+    """
+    I am a bundler, with the extension that I can also bundle other
+    bundlers.
+
+    The effect is that when you call bundle() on a me, you get one
+    bundle with a union of all subbundlers' files, in addition to any
+    loose files that you added to me.
+    """
+    def __init__(self, name='merged-bundle'):
+        Bundler.__init__(self, name)
+        self._subbundlers = {}
+
+    def addBundler(self, bundler):
+        """Add to me all of the files managed by another bundler.
+
+        @param bundler: The bundler whose files you want in this
+        bundler.
+        @type  bundler: L{Bundler}
+        """
+        if bundler.name not in self._subbundlers:
+            self._subbundlers[bundler.name] = bundler
+            for bfile in bundler._files.values():
+                self.add(bfile.source, bfile.destination)
+
+    def getSubBundlers(self):
+        """
+        @returns: A list of all of the bundlers that have been added to
+        me.
+        """
+        return self._subbundlers.values()
+
+def makeBundleFromLoadedModules(outfile, outreg, *prefixes):
+    """
+    Make a bundle from a subset of all loaded modules, also writing out
+    a registry file that can apply to that subset of the global
+    registry. Suitable for use as a FLU_ATEXIT handler.
+
+    @param outfile: The path to which a zip file will be written.
+    @type  outfile: str
+    @param outreg: The path to which a registry file will be written.
+    @type  outreg: str
+    @param prefixes: A list of prefixes to which to limit the export. If
+    not given, package up all modules. For example, "flumotion" would
+    limit the output to modules that start with "flumotion".
+    @type  prefixes: list of str
+    """
+    from flumotion.common import registry, log
+    from twisted.python import reflect
+
+    def getUsedModules(prefixes):
+        ret = {}
+        for modname in sys.modules:
+            if prefixes and not filter(modname.startswith, prefixes):
+                continue
+            try:
+                module = reflect.namedModule(modname)
+                if hasattr(module, '__file__'):
+                    ret[modname] = module
+                else:
+                    log.info('makebundle', 'Module %s has no file', module)
+            except ImportError:
+                log.info('makebundle', 'Could not import %s', modname)
+        return ret
+
+    def calculateModuleBundleMap():
+        allbundles = registry.getRegistry().getBundles()
+        ret = {}
+        for bundle in allbundles:
+            for directory in bundle.getDirectories():
+                for file in directory.getFiles():
+                    path = os.path.join(directory.getName(), file.getLocation())
+                    parts = path.split(os.path.sep)
+                    if parts[-1].startswith('__init__.py'):
+                        parts.pop()
+                    elif parts[-1].endswith('.py'):
+                        parts[-1] = parts[-1][:-3]
+                    else:
+                        # not a bundled module
+                        continue
+                    modname = '.'.join(parts)
+                    ret[modname] = bundle
+        return ret
+
+    def makeMergedBundler(modules, modulebundlemap):
+        ret = MergedBundler()
+        basket = registry.getRegistry().makeBundlerBasket()
+        for modname in modules:
+            modfilename = modules[modname].__file__
+            if modname in modulebundlemap:
+                bundleName = modulebundlemap[modname].getName()
+                for depBundleName in basket.getDependencies(bundleName):
+                    ret.addBundler(basket.getBundlerByName(depBundleName))
+            else:
+                if modfilename.endswith('.pyc'):
+                    modfilename = modfilename[:-1]
+                if os.path.isdir(modfilename):
+                    with_init = os.path.join(modfilename, '__init__.py')
+                    if os.path.exists(with_init):
+                        modfilename = with_init
+                nparts = len(modname.split('.'))
+                if '__init__' in modfilename:
+                    nparts += 1
+                relpath = os.path.join(*modfilename.split(os.path.sep)[-nparts:])
+                ret.add(modfilename, relpath)
+        return ret
+
+    modules = getUsedModules(prefixes)
+    modulebundlemap = calculateModuleBundleMap()
+    bundler = makeMergedBundler(modules, modulebundlemap)
+
+    print 'Writing bundle to', outfile
+    open(outfile, 'w').write(bundler.bundle().getZip())
+
+    print 'Writing registry to', outreg
+    bundlers_used = [b.name for b in bundler.getSubBundlers()]
+    regwriter = registry.RegistrySubsetWriter(onlyBundles=bundlers_used)
+    regwriter.dump(open(outreg, 'w'))
