@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -25,7 +25,6 @@ small common functions used by all processes
 
 import errno
 import os 
-import socket
 import sys
 import time
 import signal
@@ -74,13 +73,15 @@ def formatStorage(units, precision = 2):
     format = "%%.%df %%s" % precision
     return format % (value, prefix)
 
-def formatTime(seconds):
+def formatTime(seconds, fractional=0):
     """
     Nicely format time in a human-readable format.
     Will chunks weeks, days, hours and minutes.
 
-    @param seconds: the time in seconds to format.
-    @type  seconds: int or float
+    @param seconds:    the time in seconds to format.
+    @type  seconds:    int or float
+    @param fractional: how many digits to show for the fractional part.
+    @type  fractional: int
 
     @rtype: string
     @returns: a nicely formatted time string.
@@ -113,7 +114,12 @@ def formatTime(seconds):
     elif days == 1:
         chunks.append('1 day')
 
-    chunks.append('%02d:%02d' % (hours, minutes))
+    chunk = '%02d:%02d' % (hours, minutes)
+    if fractional > 0:
+        chunk += ':%0*.*f' % (fractional + 3, fractional, seconds)
+
+    chunks.append(chunk)
+
 
     return " ".join(chunks)
 
@@ -162,12 +168,19 @@ def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null',
     will be opened and be used to replace the standard file descriptors
     in sys.stdin, sys.stdout, and sys.stderr.
     These arguments are optional and default to /dev/null.
-    Note that stderr is opened unbuffered, so
-    if it shares a file with stdout then interleaved output
-    may not appear in the order that you expect.
 
     The fork will switch to the given directory.
     '''
+    # Redirect standard file descriptors.
+    si = open(stdin, 'r')
+    os.dup2(si.fileno(), sys.stdin.fileno())
+    try:
+        log.outputToFiles(stdout, stderr)
+    except IOError, e:
+        if e.errno == errno.EACCES:
+            print dir(e)
+            log.error('common', 'Permission denied writing to log file %s.',
+                e.filename)
 
     # first fork
     try: 
@@ -198,15 +211,86 @@ def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null',
         sys.exit(1)
 
     # Now I am a daemon!
-    
-    # Redirect standard file descriptors.
-    si = open(stdin, 'r')
-    so = open(stdout, 'a+')
-    se = open(stderr, 'a+', 0)
-    os.dup2(si.fileno(), sys.stdin.fileno())
-    os.dup2(so.fileno(), sys.stdout.fileno())
-    os.dup2(se.fileno(), sys.stderr.fileno())
+    # don't add stuff here that can fail, because from now on the program
+    # will keep running regardless of tracebacks
 
+def startup(processType, processName, daemonize=False, daemonizeTo='/'):
+    """
+    Prepare a process for starting, logging appropriate standarised messages.
+    First daemonizes the process, if daemonize is true.
+    """
+    log.info(processType, "Starting %s '%s'", processType, processName)
+
+    if daemonize:
+        daemonizeHelper(processType, daemonizeTo, processName)
+
+    log.info(processType, "Started %s '%s'", processType, processName)
+
+    def shutdownStarted():
+        log.info(processType, "Stopping %s '%s'", processType, processName)
+    def shutdownEnded():
+        log.info(processType, "Stopped %s '%s'", processType, processName)
+
+    # import inside function so we avoid affecting startup
+    from twisted.internet import reactor
+    reactor.addSystemEventTrigger('before', 'shutdown',
+                                  shutdownStarted)
+    reactor.addSystemEventTrigger('after', 'shutdown',
+                                  shutdownEnded)
+    
+def daemonizeHelper(processType, daemonizeTo='/', processName=None):
+    """
+    Daemonize a process, writing log files and PID files to conventional
+    locations.
+
+    @param processType: The process type, for example 'worker'. Used
+    as part of the log file and PID file names.
+    @type  processType: str
+    @param daemonizeTo: The directory that the daemon should run in.
+    @type  daemonizeTo: str
+    @param processName: The service name of the process. Used to
+    disambiguate different instances of the same daemon.
+    @type  processName: str
+    """
+
+    ensureDir(configure.logdir, "log file")
+    ensureDir(configure.rundir, "run file")
+
+    pid = getPid(processType, processName)
+    if pid:
+        raise SystemError(
+            "A %s service named '%s' is already running with pid %d"
+            % (processType, processName or processType, pid))
+
+    log.debug(processType, "%s service named '%s' daemonizing", 
+        processType, processName)
+
+    if processName:
+        logPath = os.path.join(configure.logdir,
+                               '%s.%s.log' % (processType, processName))
+    else:
+        logPath = os.path.join(configure.logdir,
+                               '%s.log' % (processType,))
+    log.debug(processType, 'Further logging will be done to %s', logPath)
+
+    # here we daemonize; so we also change our pid
+    daemonize(stdout=logPath, stderr=logPath, directory=daemonizeTo)
+
+    log.debug(processType, 'Started daemon')
+
+    # from now on I should keep running until killed, whatever happens
+    path = writePidFile(processType, processName)
+    log.debug(processType, 'written pid file %s', path)
+
+    # import inside function so we avoid affecting startup
+    from twisted.internet import reactor
+    def _deletePidFile():
+        log.debug(processType, 'deleting pid file')
+        deletePidFile(processType, processName)
+    reactor.addSystemEventTrigger('after', 'shutdown',
+                                  _deletePidFile)
+
+    
 def argRepr(args=(), kwargs={}, max=-1):
     """
     Return a string representing the given args.
@@ -249,31 +333,46 @@ def ensureDir(dir, description):
             raise errors.SystemError, "could not create %s directory %s" % (
                 description, dir)
 
-def getPidPath(type, name):
+def getPidPath(type, name=None):
     """
     Get the full path to the pid file for the given process type and name.
     """
-    return os.path.join(configure.rundir, '%s.%s.pid' % (type, name))
+    path = os.path.join(configure.rundir, '%s.pid' % type)
+    if name:
+        path = os.path.join(configure.rundir, '%s.%s.pid' % (type, name))
+    log.debug('common', 'getPidPath for type %s, name %r: %s' % (
+        type, name, path))
+    return path
  
-def writePidFile(type, name):
+def writePidFile(type, name=None):
     """
     Write a pid file in the run directory, using the given process type
     and process name for the filename.
+
+    @rtype:   str
+    @returns: full path to the pid file that was written
     """
     ensureDir(configure.rundir, "rundir")
     pid = os.getpid()
-    file = open(getPidPath(type, name), 'w')
+    path = getPidPath(type, name)
+    file = open(path, 'w')
     file.write("%d\n" % pid)
     file.close()
+    return path
  
-def deletePidFile(type, name):
+def deletePidFile(type, name=None):
     """
     Delete the pid file in the run directory, using the given process type
     and process name for the filename.
+
+    @rtype:   str
+    @returns: full path to the pid file that was written
     """
-    os.unlink(getPidPath(type, name))
+    path = getPidPath(type, name)
+    os.unlink(path)
+    return path
  
-def getPid(type, name):
+def getPid(type, name=None):
     """
     Get the pid from the pid file in the run directory, using the given
     process type and process name for the filename.
@@ -282,6 +381,7 @@ def getPid(type, name):
     """
     
     pidPath = getPidPath(type, name)
+    log.log('common', 'pidfile for %s %s is %s' % (type, name, pidPath))
     if not os.path.exists(pidPath):
         return
     
@@ -337,7 +437,7 @@ def checkPidRunning(pid):
             raise
     return False
  
-def waitPidFile(type, name):
+def waitPidFile(type, name=None):
     """
     Wait for the given process type and name to have started and created
     a pid file.
@@ -375,56 +475,6 @@ def waitForTerm():
     waiter = Waiter()
     waiter.sleep()
 
-def checkPortFree(port):
-    """
-    Check if the given local port is free to accept on.
-
-    @type port: int
-
-    @rtype: boolean
-    """
-    assert type(port) == int
-    fd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    try:
-        fd.bind(('', port))
-    except socket.error:
-        return False
-    
-    return True
-    
-def getFirstFreePort(startPort):
-    """
-    Get the first free port, starting from the given port.
-
-    @type startPort: int
-
-    @rtype: int
-    """
-    port = startPort
-    while 1:
-        if checkPortFree(port):
-            return port
-        port += 1
-
-def checkRemotePort(host, port):
-    """
-    Check if the given remote host/port is accepting connections.
-
-    @type port: int
-
-    @rtype: boolean
-    """
-    assert type(port) == int
-    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    try:
-        s.connect((host, port))
-    except socket.error:
-        s.close()
-        return False
-    
-    s.close()
-    return True
-
 def addressGetHost(a):
     """
     Get the host name of an IPv4 address.
@@ -459,10 +509,84 @@ def addressGetPort(a):
 def componentPath(componentName, parentName):
     """
     Create a path string out of the name of a component and its parent.
+
+    @depreciated: Use @componentId instead
     """
-    
     return '/%s/%s' % (parentName, componentName)
 
+def componentId(parentName, componentName):
+    """
+    Create a C{componentId} based on the C{parentName} and C{componentName}.
+
+    A C{componentId} uniquely identifies a component within a planet.
+
+    @since: 0.3.1
+
+    @rtype: str
+    """
+    return '/%s/%s' % (parentName, componentName)
+
+def parseComponentId(componentId):
+    """
+    @since: 0.3.1
+
+    @rtype:  tuple of (str, str)
+    @return: tuple of (flowName, componentName)
+    """
+    list = componentId.split("/")
+    assert len(list) == 3
+    assert list[0] == ''
+    return (list[1], list[2])
+
+def feedId(componentName, feedName):
+    """
+    Create a C{feedId} based on the C{componentName} and C{feedName}.
+
+    A C{feedId} uniquely identifies a feed within a flow or atmosphere.
+    It identifies the feed from a feeder to an eater.
+
+    @since: 0.3.1
+
+    @rtype: str
+    """
+    return "%s:%s" % (componentName, feedName)
+
+def parseFeedId(feedId):
+    """
+    @since: 0.3.1
+
+    @rtype:  tuple of (str, str)
+    @return: tuple of (componentName, feedName)
+    """
+    list = feedId.split(":")
+    assert len(list) == 2, "feedId %s should contain exactly one ':'" % feedId
+    return (list[0], list[1])
+
+def fullFeedId(flowName, componentName, feedName):
+    """
+    Create a C{fullFeedId} based on the C{flowName}, C{componentName} and
+    C{feedName}.
+
+    A C{fullFeedId} uniquely identifies a feed within a planet.
+
+    @since: 0.3.1
+
+    @rtype: str
+    """
+    return feedId(componentId(flowName, componentName), feedName)
+
+def parseFullFeedId(fullFeedId):
+    """
+    @since: 0.3.1
+
+    @rtype:  tuple of (str, str, str)
+    @return: tuple of (flowName, componentName, feedName)
+    """
+    list = fullFeedId.split(":")
+    assert len(list) == 2
+    flowName, componentName = parseComponentId(list[0])
+    return (flowName, componentName, list[1])
+
 def objRepr(object):
     """
     Return a string giving the fully qualified class of the given object.
@@ -554,6 +678,22 @@ def compareVersions(first, second):
 
     return 0
 
+def versionTupleToString(versionTuple):
+    """
+    Converts a version tuple to a string.  If the tuple has a zero nano number,
+    it is dropped from the string.
+
+    @since: 0.4.1
+
+    @type versionTuple: tuple
+
+    @rtype: str
+    """
+    if len(versionTuple) == 4 and versionTuple[3] == 0:
+        versionTuple = versionTuple[:3]
+
+    return ".".join([str(i) for i in versionTuple])
+
 def _uniq(l, key=lambda x: x):
     """
     Filters out duplicate entries in a list.
@@ -624,3 +764,14 @@ class InitMixin(object):
 
     def __init__(self, *args, **kwargs):
         call_each_method_reversed(self, 'init', *args, **kwargs)
+
+def strToBool(string):
+    """
+    @type  string: str
+
+    @return: True if the string represents a value we interpret as true.
+    """
+    if string in ('True', 'true', '1', 'yes'):
+        return True
+
+    return False
