@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -30,7 +30,10 @@ class WorkerComponentUIState(flavors.StateCacheable):
     pass
 
 class ManagerComponentUIState(flavors.StateCacheable, flavors.StateRemoteCache):
-    pass
+    def processUniqueID(self):
+        # Make sure proxies for the same object are the same, if we are
+        # later cached by someone else. See bug #519.
+        return id(self.__dict__)
 
 class AdminComponentUIState(flavors.StateRemoteCache):
     pass
