@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -30,10 +30,12 @@ API Stability: stabilizing
 Maintainer: U{Thomas Vander Stichele <thomas at apestaart dot org>}
 """
 
+import errno
 import sys
 import os
 import fnmatch
 import time
+import types
 import traceback
 
 # environment variables controlling levels for each category
@@ -134,22 +136,107 @@ def scrubFilename(filename):
     
     return filename
 
-def _handle(level, object, category, message):
-    def getFileLine():
-        # Return a tuple of (file, line) for the first stack entry
-        # outside of log.py (which would be the caller of log)
-        frame = sys._getframe()
-        while frame:
-            co = frame.f_code
+def getFileLine(where=-1):
+    """
+    Return the filename and line number for the given location.
+    
+    If where is a negative integer, look for the code entry in the current
+    stack that is the given number of frames above this module.
+    If where is a function, look for the code entry of the function.
+
+    @param where: how many frames to go back up, or function
+    @type  where: int (negative) or function
+
+    @return: tuple of (file, line)
+    @rtype:  tuple of (str, int)
+    """
+    co = None
+    lineno = None
+    
+    if isinstance(where, types.FunctionType):
+        co = where.func_code
+        lineno = co.co_firstlineno
+    elif isinstance(where, types.MethodType):
+        co = where.im_func.func_code
+        lineno = co.co_firstlineno
+    else:
+        stackFrame = sys._getframe()
+        while stackFrame:
+            co = stackFrame.f_code
             if not co.co_filename.endswith('log.py'):
-                return scrubFilename(co.co_filename), frame.f_lineno
-            frame = frame.f_back
-            
-        return "Not found", 0
+                # wind up the stack according to frame
+                while where < -1:
+                    stackFrame = stackFrame.f_back
+                    where += 1
+                co = stackFrame.f_code
+                lineno = stackFrame.f_lineno
+                break
+            stackFrame = stackFrame.f_back
+
+    if not co:
+        return "<unknown file>", 0
+
+    return scrubFilename(co.co_filename), lineno
+
+def ellipsize(o):
+    """
+    Ellipsize the representation of the given object.
+    """
+    r = repr(o)
+    if len(r) < 800:
+        return r
+
+    r = r[:60] + ' ... ' + r[-15:]
+    return r
+ 
+def getFormatArgs(startFormat, startArgs, endFormat, endArgs, args, kwargs):
+    """
+    Helper function to create a format and args to use for logging.
+    This avoids needlessly interpolating variables.
+    """
+    debugArgs = startArgs[:]
+    for a in args:
+        debugArgs.append(ellipsize(a))
+
+    for items in kwargs.items():
+        debugArgs.extend(items)
+    debugArgs.extend(endArgs)
+    format = startFormat \
+              + ', '.join(('%s', ) * len(args)) \
+              + ', '.join(('%s=%r, ', ) * len(kwargs)) \
+              + endFormat
+    return format, debugArgs
+
+def doLog(level, object, category, format, args, where=-1,
+    file=None, line=None):
+    """
+    @param where: what to log file and line number for;
+                  -1 for one frame above log.py; -2 and down for higher up;
+                  a function for a (future) code object
+    @type  where: int or callable
+    @param file:  file to show the message as coming from, if caller knows best
+    @type  file:  str
+    @param line:  line to show the message as coming from, if caller knows best
+    @type  line:  int
+
+    @return: dict of calculated variables, if they needed calculating.
+             currently contains file and line; this prevents us from
+             doing this work in the caller when it isn't needed because
+             of the debug level
+    """
+    ret = {}
+
+    if args:
+        message = format % args
+    else:
+        message = format
 
     # first all the unlimited ones
     if _log_handlers:
-        (file, line) = getFileLine()
+        if file is None and line is None:
+            (file, line) = getFileLine(where=where)
+        ret['file'] = file
+        ret['line'] = line
         for handler in _log_handlers:
             try:
                 handler(level, object, category, file, line, message)
@@ -157,64 +244,89 @@ def _handle(level, object, category, message):
                 raise SystemError, "handler %r raised a TypeError" % handler
 
     if level > getCategoryLevel(category):
-        return
-
-    # set this a second time, just in case there weren't unlimited
-    # loggers there before
-    (file, line) = getFileLine()
+        return ret
 
     for handler in _log_handlers_limited:
+        # set this a second time, just in case there weren't unlimited
+        # loggers there before
+        if file is None and line is None:
+            (file, line) = getFileLine(where=where)
+        ret['file'] = file
+        ret['line'] = line
         try:
             handler(level, object, category, file, line, message)
         except TypeError:
             raise SystemError, "handler %r raised a TypeError" % handler
+
+        return ret
     
-def errorObject(object, cat, *args):
+def errorObject(object, cat, format, *args):
     """
-    Log a fatal error message in the given category. \
+    Log a fatal error message in the given category.
     This will also raise a L{flumotion.common.errors.SystemError}.
     """
-    _handle(ERROR, object, cat, ' '.join(args))
+    doLog(ERROR, object, cat, format, args)
 
     # we do the import here because having it globally causes weird import
     # errors if our gstreactor also imports .log, which brings in errors
     # and pb stuff
     from flumotion.common.errors import SystemError
-    raise SystemError(' '.join(args))
+    if args:
+        raise SystemError(format % args)
+    else:
+        raise SystemError(format)
 
-def warningObject(object, cat, *args):
+def warningObject(object, cat, format, *args):
     """
     Log a warning message in the given category.
     This is used for non-fatal problems.
     """
-    _handle(WARN, object, cat, ' '.join(args))
+    doLog(WARN, object, cat, format, args)
 
-def infoObject(object, cat, *args):
+def infoObject(object, cat, format, *args):
     """
     Log an informational message in the given category.
     """
-    _handle(INFO, object, cat, ' '.join(args))
+    doLog(INFO, object, cat, format, args)
 
-def debugObject(object, cat, *args):
+def debugObject(object, cat, format, *args):
     """
     Log a debug message in the given category.
     """
-    _handle(DEBUG, object, cat, ' '.join(args))
+    doLog(DEBUG, object, cat, format, args)
 
-def logObject(object, cat, *args):
+def logObject(object, cat, format, *args):
     """
     Log a log message.  Used for debugging recurring events.
     """
-    _handle(LOG, object, cat, ' '.join(args))
+    doLog(LOG, object, cat, format, args)
 
-error = lambda cat, *args: errorObject(None, cat, *args)
-warning = lambda cat, *args: warningObject(None, cat, *args)
-info = lambda cat, *args: infoObject(None, cat, *args)
-debug = lambda cat, *args: debugObject(None, cat, *args)
-log = lambda cat, *args: logObject(None, cat, *args)
+def error(cat, format, *args):
+    errorObject(None, cat, format, *args)
 
-#warningFailure = lambda failure: Loggable.warningFailure(None, '', failure)
-warningFailure = lambda failure: warning('', getFailureMessage(failure))
+def warning(cat, format, *args):
+    warningObject(None, cat, format, *args)
+
+def info(cat, format, *args):
+    infoObject(None, cat, format, *args)
+
+def debug(cat, format, *args):
+    debugObject(None, cat, format, *args)
+
+def log(cat, format, *args):
+    logObject(None, cat, format, *args)
+
+def warningFailure(failure, swallow=True):
+    """
+    Log a warning about a Failure. Useful as an errback handler:
+    d.addErrback(warningFailure)
+
+    @param swallow: whether to swallow the failure or not
+    @type  swallow: bool
+    """
+    warning('', getFailureMessage(failure))
+    if not swallow:
+        return failure
 
 class Loggable:
     """
@@ -233,50 +345,80 @@ class Loggable:
         if _canShortcutLogging(self.logCategory, ERROR):
             return
         errorObject(self.logObjectName(), self.logCategory,
-            self.logFunction(*args))
+            *self.logFunction(*args))
         
     def warning(self, *args):
         """Log a warning.  Used for non-fatal problems."""
         if _canShortcutLogging(self.logCategory, WARN):
             return
         warningObject(self.logObjectName(), self.logCategory,
-            self.logFunction(*args))
+            *self.logFunction(*args))
         
     def info(self, *args):
         """Log an informational message.  Used for normal operation."""
         if _canShortcutLogging(self.logCategory, INFO):
             return
         infoObject(self.logObjectName(), self.logCategory,
-            self.logFunction(*args))
+            *self.logFunction(*args))
 
     def debug(self, *args):
         """Log a debug message.  Used for debugging."""
         if _canShortcutLogging(self.logCategory, DEBUG):
             return
         debugObject(self.logObjectName(), self.logCategory,
-            self.logFunction(*args))
+            *self.logFunction(*args))
 
     def log(self, *args):
         """Log a log message.  Used for debugging recurring events."""
         if _canShortcutLogging(self.logCategory, LOG):
             return
         logObject(self.logObjectName(), self.logCategory,
-            self.logFunction(*args))
+            *self.logFunction(*args))
 
-    def warningFailure(self, failure):
+    def doLog(self, level, where, format, *args, **kwargs):
+        """
+        Log a message at the given level, with the possibility of going
+        higher up in the stack.
+
+        @param level: log level
+        @type  level: int
+        @param where: how many frames to go back from the last log frame;
+                      or a function (to log for a future call)
+        @type  where: int (negative), or function
+
+        @param kwargs: a dict of pre-calculated values from a previous
+                       doLog call
+
+        @return: a dict of calculated variables, to be reused in a
+                 call to doLog that should show the same location
+        @rtype:  dict
+        """
+        if _canShortcutLogging(self.logCategory, level):
+            return {}
+        args = self.logFunction(*args)
+        return doLog(level, self.logObjectName(), self.logCategory,
+            format, args, where=where, **kwargs)
+
+    def warningFailure(self, failure, swallow=True):
         """
         Log a warning about a Failure. Useful as an errback handler:
         d.addErrback(self.warningFailure)
-        Swallows the failure since it doesn't return it.
+
+        @param swallow: whether to swallow the failure or not
+        @type  swallow: bool
         """
         if _canShortcutLogging(self.logCategory, WARN):
-            return
+            if swallow:
+                return
+            return failure
         warningObject(self.logObjectName(), self.logCategory,
-            self.logFunction(getFailureMessage(failure)))
+            *self.logFunction(getFailureMessage(failure)))
+        if not swallow:
+            return failure
 
-    def logFunction(self, message):
+    def logFunction(self, *args):
         """Overridable log function.  Default just returns passed message."""
-        return message
+        return args
 
     def logObjectName(self):
         """Overridable object name function."""
@@ -313,7 +455,7 @@ class FluLogObserver(Loggable):
                 self.log("Failure %r" % f)
 
                 method = debug # tracebacks from errors at debug level
-                msg = "A python traceback occurred."
+                msg = "A twisted traceback occurred."
                 if getCategoryLevel("twisted") < WARN:
                     msg += "  Run with debug level >= 2 to see the traceback."
                 # and an additional warning
@@ -333,6 +475,8 @@ class FluLogObserver(Loggable):
                     'text': text.replace("\n", "\n\t")
                   }
         msgStr = " [%(system)s] %(text)s\n" % fmtDict
+        # because msgstr can contain %, as in a backtrace, make sure we
+        # don't try to splice it
         method('twisted', msgStr)
 
     def ignoreErrors(self, *types):
@@ -356,7 +500,7 @@ def _getTheFluLogObserver():
 
 def stderrHandler(level, object, category, file, line, message):
     """
-    A log handler that writes to stdout.
+    A log handler that writes to stderr.
 
     @type level:    string
     @type object:   string (or None)
@@ -383,9 +527,12 @@ def stderrHandler(level, object, category, file, line, message):
         #    level, o, category, where, os.getpid(),
         #    "", time.strftime("%b %d %H:%M:%S"), message))
         sys.stderr.flush()
-    except IOError:
-        # happens in SIGCHLDHandler for example
-        pass
+    except IOError, e:
+        if e.errno == errno.EPIPE:
+            # if our output is closed, exit; e.g. when logging over an
+            # ssh connection and the ssh connection is closed
+            os._exit(os.EX_OSERR)
+        # otherwise ignore it, there's nothing you can do
 
 def addLogHandler(func, limited=True):
     """
@@ -427,6 +574,53 @@ def init():
 
     _initialized = True
 
+_stdout = None
+_stderr = None
+def reopenOutputFiles():
+    """
+    Reopens the stdout and stderr output files, as set by
+    L{flumotion.common.log.outputToFiles}.
+    """
+    if not (_stdout and _stderr):
+        debug('log', 'told to reopen log files, but log files not set')
+        return
+
+    oldmask = os.umask(0026)
+    try:
+        so = open(_stdout, 'a+') 
+        se = open(_stderr, 'a+', 0)
+    finally:
+        os.umask(oldmask)
+
+    os.dup2(so.fileno(), sys.stdout.fileno()) 
+    os.dup2(se.fileno(), sys.stderr.fileno())
+
+    debug('log', 'opened log %r', _stderr)
+
+def outputToFiles(stdout, stderr):
+    """
+    Redirect stdout and stderr to named files.
+
+    Records the file names so that a future call to reopenOutputFiles()
+    can open the same files. Installs a SIGHUP handler that will reopen
+    the output files.
+
+    Note that stderr is opened unbuffered, so if it shares a file with
+    stdout then interleaved output may not appear in the order that you
+    expect.
+    """
+    global _stdout, _stderr
+    _stdout, _stderr = stdout, stderr
+    reopenOutputFiles()
+
+    def sighup(signum, frame):
+        info('log', "Received SIGHUP, reopening logs")
+        reopenOutputFiles()
+
+    debug('log', 'installing SIGHUP handler')
+    import signal
+    signal.signal(signal.SIGHUP, sighup)
+
 _initializedTwisted = False
 
 def logTwisted():
@@ -496,8 +690,8 @@ def getExceptionMessage(exception, frame=-1, filename=None):
     msg = ""
     # a shortcut to extract a useful message out of most flumotion errors
     # for now
-    if len(exception.args) == 1 and isinstance(exception.args[0], str):
-        msg = ": %s" % exception.args[0]
+    if str(exception):
+        msg = ": %s" % str(exception)
     return "exception %(exc)s at %(filename)s:%(line)s: %(func)s()%(msg)s" % locals()
 
 def getFailureMessage(failure):
