@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -33,9 +33,10 @@ import gobject
 from twisted.internet import reactor, error, defer
 from twisted.cred import error as crederror
 from twisted.spread import pb
+from twisted.python import reflect
 
 from flumotion.common import interfaces, errors, log, planet, medium, pygobject
-from flumotion.common import componentui, common, messages
+from flumotion.common import componentui, common, registry, messages, interfaces
 from flumotion.common.planet import moods
 from flumotion.configure import configure
 from flumotion.twisted import credentials
@@ -43,12 +44,15 @@ from flumotion.twisted import pb as fpb
 from flumotion.twisted.compat import implements
 from flumotion.common.pygobject import gsignal
 
+from flumotion.common.messages import N_
+T_ = messages.gettexter('flumotion')
 
 class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
     """
     I am a client factory for a component logging in to the manager.
     """
     logCategory = 'component'
+    perspectiveInterface = interfaces.IComponentMedium
     def __init__(self, component):
         """
         @param component: L{flumotion.component.component.BaseComponent}
@@ -58,30 +62,21 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
         
         self.component = component
         # make a medium to interface with the manager
-        self.medium = component.component_medium_class(component)
+        self.medium = component.componentMediumClass(component)
         component.setMedium(self.medium)
 
         self.maxDelay = 10
         # get the interfaces implemented by the component medium class
-        # FIXME: interface
+        #FIXME: interface
         #self.interfaces = self.medium.__class__.__implements__
 
         self.logName = component.name
-        
-    # override log.Loggable method so we don't traceback
-    def error(self, message):
-        self.warning('Shutting down because of %s' % message)
-        print >> sys.stderr, 'ERROR: [%d] %s' % (os.getpid(), message)
-        # FIXME: do we need to make sure that this cannot shut down the
-        # manager if it's the manager's bouncer ?
-        reactor.stop()
-        self.component.setMood(moods.sad)
 
     # vmethod implementation
     def gotDeferredLogin(self, d):
         def remoteDisconnected(remoteReference):
             if reactor.killed:
-                self.log('Connection to manager lost due to SIGINT shutdown')
+                self.log('Connection to manager lost due to shutdown')
             else:
                 self.warning('Lost connection to manager, '
                              'will attempt to reconnect')
@@ -89,24 +84,26 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
         def loginCallback(reference):
             self.info("Logged in to manager")
             self.debug("remote reference %r" % reference)
+            self._previously_connected = True
+        
             self.medium.setRemoteReference(reference)
             reference.notifyOnDisconnect(remoteDisconnected)
 
         def accessDeniedErrback(failure):
             failure.trap(crederror.UnauthorizedLogin)
-            self.error('Access denied.')
+            self.warning('Access denied.')
             
         def connectionRefusedErrback(failure):
             failure.trap(error.ConnectionRefusedError)
-            self.error('Connection to manager refused.')
+            self.warning('Connection to manager refused.')
                                                           
         def alreadyLoggedInErrback(failure):
             failure.trap(errors.AlreadyConnectedError)
-            self.error('Component named %s is already logged in.'
-                       % self.component.name)
+            self.warning('Component with id %s is already logged in.',
+                self.medium.authenticator.avatarId)
                                                           
         def loginFailedErrback(failure):
-            self.error('Login failed, reason: %s' % failure)
+            self.warning('Login failed, reason: %s' % failure)
 
         d.addCallback(loginCallback)
         d.addErrback(accessDeniedErrback)
@@ -114,33 +111,38 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
         d.addErrback(alreadyLoggedInErrback)
         d.addErrback(loginFailedErrback)
 
-    def startLogin(self, keycard):
-        fpb.ReconnectingFPBClientFactory.startLogin(self, keycard, self.medium,
-                                                    interfaces.IComponentMedium)
-        
+    # we want to save the authenticator
+    def startLogin(self, authenticator):
+        self.medium.setAuthenticator(authenticator)
+        return fpb.ReconnectingFPBClientFactory.startLogin(self, authenticator)
+
 # needs to be before BaseComponent because BaseComponent references it
-class BaseComponentMedium(medium.BaseMedium):
+class BaseComponentMedium(medium.PingingMedium):
     """
     I am a medium interfacing with a manager-side avatar.
     I implement a Referenceable for the manager's avatar to call on me.
     I have a remote reference to the manager's avatar to call upon.
+    I am created by the L{ComponentClientFactory}.
+
+    @cvar authenticator: the authenticator used to log in to manager
+    @type authenticator: L{flumotion.twisted.pb.Authenticator}
     """
 
     implements(interfaces.IComponentMedium)
-    logCategory = 'basecompmedium'
+    logCategory = 'basecompmed'
 
     def __init__(self, component):
         """
         @param component: L{flumotion.component.component.BaseComponent}
         """
         self.comp = component
-        self.logName = component.name
-        
+        self.authenticator = None
+
     ### our methods
     def setup(self, config):
         pass
 
-    def getIP(self):
+    def getManagerIP(self):
         """
         Return the manager IP as seen by us.
         """
@@ -152,9 +154,32 @@ class BaseComponentMedium(medium.BaseMedium):
             host = peer[1]
 
         res = socket.gethostbyname(host)
-        self.debug("getIP(): we think the manager's IP is %r" % res)
+        self.debug("getManagerIP(): we think the manager's IP is %r" % res)
         return res
 
+    def getIP(self):
+        """
+        Return the IP of this component based on connection to the manager.
+
+        Note: this is insufficient in general, and should be replaced by 
+        network mapping stuff later.
+        """
+        assert self.remote
+        host = self.remote.broker.transport.getHost()
+        self.debug("getIP(): using %r as our IP", host.host)
+        return host.host
+
+    def setAuthenticator(self, authenticator):
+        """
+        Set the authenticator the client factory has used to log in to the
+        manager.  Can be reused by the component's medium to make
+        feed connections which also get authenticated by the manager's
+        bouncer.
+
+        @type  authenticator: L{flumotion.twisted.pb.Authenticator}
+        """
+        self.authenticator = authenticator
+
     ### pb.Referenceable remote methods
     ### called from manager by our avatar
     def remote_getState(self):
@@ -167,9 +192,7 @@ class BaseComponentMedium(medium.BaseMedium):
         """
         # we can only get the IP after we have a remote reference, so add it
         # here
-        self.comp.state.set('ip', self.getIP())
-        self.debug('remote_getState of f: returning %r' % self.comp.state)
-
+        self.comp.state.set('manager-ip', self.getManagerIP())
         return self.comp.state
         
     def remote_getConfig(self):
@@ -179,38 +202,27 @@ class BaseComponentMedium(medium.BaseMedium):
         @rtype:   dict
         @returns: component's current configuration
         """
-        self.debug('remote_getConfig of f: returning %r' % self.comp.config)
-        return self.comp.config
+        try:
+            return self.comp.config
+        except AttributeError:
+            self.debug('getConfig(), but component is not set up yet')
+            return None
         
     def remote_setup(self, config):
         """
         Set up the component and the component's medium with the given config,
         in that order.
         """
-        self.debug('remote_setup(%r)' % config)
         d = self.comp.setup(config)
         d.addCallback(lambda r, c: self.setup(c), config)
         return d
         
     def remote_start(self, *args, **kwargs):
-        self.debug('remote_start(args=%r, kwargs=%r)' % (args, kwargs))
         return self.comp.start(*args, **kwargs)
        
     def remote_stop(self):
-        self.info('Stopping job')
-        d = self.comp.stop()
-        d.addCallback(self._destroyCallback)
-
-        return d
-
-    def _destroyCallback(self, result):
-        self.debug('_destroyCallback: scheduling destroy')
-        reactor.callLater(0, self._destroyCallLater)
-
-    def _destroyCallLater(self):
-        self.debug('_destroyCalllater: losing connection and stopping reactor')
-        self.remote.broker.transport.loseConnection()
-        reactor.stop()
+        self.info('Stopping component')
+        return self.comp.stop()
 
     def remote_reloadComponent(self):
         """Reload modules in the component."""
@@ -255,17 +267,17 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
     """
     I am the base class for all Flumotion components.
 
-    @ivar name: the name of the component
-    @type name: string
+    @ivar name:   the name of the component
+    @type name:   string
+    @ivar medium: the component's medium
+    @type medium: L{BaseComponentMedium}
 
-    @cvar component_medium_class: the medium class to use for this component
-    @type component_medium_class: child class of L{BaseComponentMedium}
+    @cvar componentMediumClass: the medium class to use for this component
+    @type componentMediumClass: child class of L{BaseComponentMedium}
     """
 
     logCategory = 'basecomp'
-
-    component_medium_class = BaseComponentMedium
-    _heartbeatInterval = configure.heartbeatInterval
+    componentMediumClass = BaseComponentMedium
     
     def __init__(self):
         # FIXME: name is unique where ? only in flow, so not in worker
@@ -298,10 +310,9 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         self.name = None
         
         #self.state.set('name', name)
-        self.state.set('mood', moods.sleeping.value)
         self.state.set('pid', os.getpid())
+        self.state.set('mood', moods.waking.value)
 
-        self._HeartbeatDC = None
         self.medium = None # the medium connecting us to the manager's avatar
  
         self.uiState = componentui.WorkerComponentUIState()
@@ -312,6 +323,13 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         self.lastTime = time.time()
         self.lastClock = time.clock()
 
+        self.plugs = {}
+
+        # Start the cpu-usage updating.
+        self._cpuCallLater = reactor.callLater(5, self._updateCPUUsage)
+
+        self._shutdownHook = None
+
     def do_check(self):
         """
         Subclasses can implement me to run any checks before the component
@@ -326,7 +344,7 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         error message, this method should set the mood to sad and raise the
         error on its own.
 
-        self.config will be set when this is called.
+        self.config will be set before this is called.
 
         @Returns: L{twisted.internet.defer.Deferred}
         """
@@ -340,7 +358,7 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         Non-programming errors should not be raised, but returned as a
         failing deferred.
 
-        self.config will be set when this is called.
+        self.config will be set before this is called.
 
         @Returns: L{twisted.internet.defer.Deferred}
         """
@@ -360,19 +378,17 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
        
     def do_stop(self):
         """
-        BaseComponent vmethod for stopping. If you override this
-        method, you are responsible for arranging that the component
-        becomes sleeping.
+        BaseComponent vmethod for stopping.
+        The component should do any cleanup it needs, but must not set the
+        component's mood to sleeping.
 
         @Returns: L{twisted.internet.defer.Deferred}
         """
-        # default behavior
-        self.setMood(moods.sleeping)
         return defer.succeed(None)
  
     ### BaseComponent implementation related to compoment protocol
     ### called by manager through medium
-    def setup(self, config):
+    def setup(self, config, *args, **kwargs):
         """
         Sets up the component with the given config.  Called by the manager
         through the medium.
@@ -381,18 +397,85 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         @raise    flumotion.common.errors.ComponentSetupError:
                   when an error happened during setup of the component
         """
-        self._setConfig(config)
-        d = self.do_check()
+        def setup_plugs():
+            # by this time we have a medium, so we can load bundles
+            reg = registry.getRegistry()
+
+            def load_bundles():
+                modules = {}
+                for plugs in config['plugs'].values():
+                    for plug in plugs:
+                        modules[plug['type']] = True
+                for plugtype in modules.keys():
+                    # we got this far, it should work
+                    entry = reg.getPlug(plugtype).getEntry()
+                    modules[plugtype] = entry.getModuleName()
+                if not modules:
+                    return defer.succeed(True) # shortcut
+                elif not self.medium:
+                    self.warning('Not connected to a medium, cannot '
+                                 'load bundles -- assuming all modules '
+                                 'are available')
+                    return defer.succeed(True)
+                else:
+                    loader = self.medium.bundleLoader
+                    return loader.getBundles(moduleName=modules.values())
+                
+            def make_plugs():
+                for socket, plugs in config['plugs'].items():
+                    self.plugs[socket] = []
+                    for plug in plugs:
+                        entry = reg.getPlug(plug['type']).getEntry()
+                        module = reflect.namedAny(entry.getModuleName())
+                        proc = getattr(module, entry.getFunction())
+                        instance = proc(plug)
+                        self.plugs[socket].append(instance)
+
+            try:
+                d = load_bundles()
+                d.addCallback(lambda x: make_plugs())
+                return d
+            except Exception, e:
+                self.debug("Exception while loading bundles: %s" % 
+                    log.getExceptionMessage(e))
+                return defer.fail(e)
+
         def checkErrorCallback(result):
             # if the mood is now sad, it means an error was encountered
             # during check, and we should return a failure here.
-            current = self.state.get('mood')
+            # since the checks are responsible for adding a message,
+            # this is a handled error.
+            current = self.state.get('mood')            
             if current == moods.sad.value:
                 self.warning('Running checks made the component sad.')
-                raise errors.ComponentSetupError()
+                raise errors.ComponentSetupHandledError()
 
+        self.debug("setup() called with config %r", config)
+        self.setMood(moods.waking)
+        self._setConfig(config)
+        # now we have a name, set it on the medium too
+        if self.medium:
+            self.medium.logName = self.getName()
+        d = setup_plugs()
+        d.addCallback(lambda r: self.do_check())
         d.addCallback(checkErrorCallback)
         d.addCallback(lambda r: self.do_setup())
+        def setupErrback(failure):
+            # pass through handled errors
+            if failure.check(errors.ComponentSetupHandledError):
+                return failure
+
+            self.warning('Could not set up component: %s',
+                log.getFailureMessage(failure))
+            m = messages.Error(T_(N_("Could not setup component.")),
+                debug=log.getFailureMessage(failure),
+                id="component-setup-%s" % self.name)
+            self.state.append('messages', m)
+            self.setMood(moods.sad)
+            raise errors.ComponentSetupHandledError(
+                'Could not set up component')
+
+        d.addErrback(setupErrback)
         return d
 
     def start(self, *args, **kwargs):
@@ -407,14 +490,31 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         Again, don't override this method. Thanks.
         """
         self.debug('BaseComponent.start')
-        self.setMood(moods.waking)
-        self.startHeartbeat()
 
-        ret = self.do_start(*args, **kwargs)
+        def start_plugs():
+            for socket, plugs in self.plugs.items():
+                for plug in plugs:
+                    self.debug('Starting plug %r on socket %s', plug, socket)
+                    plug.start(self)
 
-        self.debug('start: returning value %s' % ret)
+        try:
+            start_plugs()
+            ret = self.do_start(*args, **kwargs)
+            assert isinstance(ret, defer.Deferred), \
+                   "do_start %r must return a deferred" % self.do_start
+            self.debug('start: returning value %s' % ret)
+            return ret
+        except Exception, e:
+            self.debug("Exception during component do_start: %s" % 
+                log.getExceptionMessage(e))
+            return defer.fail(e)
 
-        return ret
+    def setShutdownHook(self, shutdownHook):
+        """
+        Set the shutdown hook for this component (replacing any previous hook).
+        When a component is stopped, then this hook will be fired.
+        """
+        self._shutdownHook = shutdownHook
         
     def stop(self):
         """
@@ -424,20 +524,30 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         """
         self.debug('BaseComponent.stop')
 
+        def stop_plugs(ret):
+            for socket, plugs in self.plugs.items():
+                for plug in plugs:
+                    self.debug('Stopping plug %r on socket %s', plug, socket)
+                    plug.stop(self)
+            return ret
+
+        def fireShutdownHook(ret):
+            if self._shutdownHook:
+                self.debug('_stoppedCallback: firing shutdown hook')
+                self._shutdownHook()
+
         self.setMood(moods.waking)
         for message in self.state.get('messages'):
             self.state.remove('messages', message)
-        self.stopHeartbeat()
 
-        return self.do_stop()
+        if self._cpuCallLater:
+            self._cpuCallLater.cancel()
+            self._cpuCallLater = None
 
-    # FIXME: privatize
-    def startHeartbeat(self):
-        """
-        Start sending heartbeats.
-        """
-        self.debug('start sending heartbeats')
-        self._heartbeat()
+        d = self.do_stop()
+        d.addCallback(stop_plugs)
+        d.addBoth(fireShutdownHook)
+        return d
 
     ### GObject methods
     def emit(self, name, *args):
@@ -460,9 +570,6 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
     def setMedium(self, medium):
         assert isinstance(medium, BaseComponentMedium)
         self.medium = medium
-        # send a heartbeat right now
-        if self._HeartbeatDC:
-            self._HeartbeatDC.reset(0)
 
     def setMood(self, mood):
         """
@@ -480,9 +587,15 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
 
         self.debug('MOOD changed to %r' % mood)
         self.state.set('mood', mood.value)
-        # send a heartbeat right now
-        if self._HeartbeatDC:
-            self._HeartbeatDC.reset(0)
+
+    def getMood(self):
+        """
+        Gets the mood on the component.
+
+        @rtype: int
+        """
+        return self.state.get('mood')
+
         
     def addMessage(self, message):
         """
@@ -496,6 +609,36 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
             self.debug('error message, turning sad')
             self.setMood(moods.sad)
         
+    def fixRenamedProperties(self, properties, list):
+        """
+        Fix properties that have been renamed from a previous version,
+        and add a warning for them.
+
+        @param properties: properties; will be modified as a result.
+        @type  properties: dict
+        @param list:       list of (old, new) tuples of property names.
+        @type  list:       list of tuple of (str, str)
+        """
+        found = []
+        for old, new in list:
+            if properties.has_key(old):
+                found.append((old, new))
+
+        if found:
+            m = messages.Warning(T_(N_(
+                "Your configuration uses deprecated properties.  "
+                "Please update your configuration and correct them.\n")),
+                id = "deprecated")
+            for old, new in found:
+                m.add(T_(N_(
+                "Please rename '%s' to '%s'.\n"),
+                        old, new))
+                self.debug("Setting new property '%s' to %r", new,
+                    properties[old])
+                properties[new] = properties[old]
+                del properties[old]
+            self.addMessage(m)
+
     def adminCallRemote(self, methodName, *args, **kwargs):
         """
         Call a remote method on all admin client views on this component.
@@ -524,33 +667,14 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         self.config = config
         self.name = config['name']
 
-    # FIXME: privatize
-    def stopHeartbeat(self):
-        """
-        Stop sending heartbeats.
-        """
-        self.debug('stop sending heartbeats')
-        if self._HeartbeatDC:
-            self.debug('canceling pending heartbeat')
-            self._HeartbeatDC.cancel()
-        self._HeartbeatDC = None
-         
-    def _heartbeat(self):
-        """
-        Send heartbeat to manager and reschedule.
-        """
-        #self.log('Sending heartbeat')
-        if self.medium:
-            self.medium.callRemote('heartbeat', self.state.get('mood'))
-        self._HeartbeatDC = reactor.callLater(self._heartbeatInterval,
-            self._heartbeat)
-
+    def _updateCPUUsage(self):
         # update CPU time stats
         nowTime = time.time()
         nowClock = time.clock()
         deltaTime = nowTime - self.lastTime
         deltaClock = nowClock - self.lastClock
         CPU = deltaClock/deltaTime
+        self.log('latest CPU use: %r' % CPU)
         self.state.set('cpu', CPU)
         deltaTime = nowTime - self.baseTime
         deltaClock = nowClock
@@ -558,5 +682,6 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         self.lastTime = nowTime
         self.lastClock = nowClock
 
+        self._cpuCallLater = reactor.callLater(5, self._updateCPUUsage)
 
 pygobject.type_register(BaseComponent)
