@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -20,8 +20,10 @@
 # Headers in this file shall remain intact.
 
 import os
+import socket
 import time
 import errno
+import string
 import resource
 
 import gst
@@ -40,6 +42,8 @@ from flumotion.common import errors
 
 from flumotion.common import common, log, keycards
 
+from flumotion.component.base import http as httpbase
+
 __all__ = ['HTTPStreamingResource', 'MultifdSinkStreamer']
 
 HTTP_NAME = 'FlumotionHTTPServer'
@@ -58,58 +62,9 @@ ERROR_TEMPLATE = """<!doctype html public "-//IETF//DTD HTML 2.0//EN">
 
 HTTP_SERVER = '%s/%s' % (HTTP_NAME, HTTP_VERSION)
 
-### This is new Issuer code that eventually should move to e.g.
-### flumotion.common.keycards or related
-
-class Issuer(log.Loggable):
-    """
-    I am a base class for all Issuers.
-    An issuer issues keycards of a given class based on an object
-    (incoming HTTP request, ...)
-    """
-    def issue(self, *args, **kwargs):
-        """
-        Return a keycard, or None, based on the given arguments. 
-        """
-        raise NotImplementedError
-
-class HTTPAuthIssuer(Issuer):
-    """
-    I create L{flumotion.common.keycards.KeycardUACPP} keycards based on
-    an incoming L{twisted.protocols.http.Request} request's standard
-    HTTP authentication information.
-    """
-    def issue(self, request):
-        # for now, we're happy with a UACPP keycard; the password arrives
-        # plaintext anyway
-        keycard = keycards.KeycardUACPP(
-            request.getUser(),
-            request.getPassword(), request.getClientIP())
-        self.debug('Asking for authentication, user %s, password %s, ip %s' % (
-            keycard.username, keycard.password, keycard.address))
-        return keycard
- 
-class HTTPTokenIssuer(Issuer):
-    """
-    I create L{flumotion.common.keycards.KeycardToken} keycards based on
-    an incoming L{twisted.protocols.http.Request} request's GET "token"
-    parameter.
-    """
-    def issue(self, request):
-        if not 'token' in request.args.keys():
-            return None
-
-        # args can have lists as values, if more than one specified
-        token = request.args['token']
-        if not isinstance(token, str):
-            token = token[0]
-        
-        keycard = keycards.KeycardToken(token,
-            request.getClientIP())
-        return keycard
- 
 ### the Twisted resource that handles the base URL
-class HTTPStreamingResource(web_resource.Resource, log.Loggable):
+class HTTPStreamingResource(web_resource.Resource, httpbase.HTTPAuthentication,
+    log.Loggable):
 
     __reserve_fds__ = 50 # number of fd's to reserve for non-streaming
 
@@ -124,61 +79,44 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         """
         @param streamer: L{MultifdSinkStreamer}
         """
-        self._logfile = None
-        self._logfilename = None
-            
         streamer.connect('client-removed', self._streamer_client_removed_cb)
         self.streamer = streamer
         
         self._requests = {}            # request fd -> Request
-        self._fdToKeycard = {}         # request fd -> Keycard
-        self._idToKeycard = {}         # keycard id -> Keycard
-        self._fdToDurationCall = {}    # request fd -> IDelayedCall for duration
-        self._domain = None            # used for auth challenge and on keycard
-        self._issuer = HTTPAuthIssuer() # issues keycards; default for compat
-        self.bouncerName = None
-        self.requesterName = streamer.getName() # avatarId of streamer component
         
-        self.maxclients = -1
+        self.maxclients = self.getMaxAllowedClients(-1)
         
+        self.loggers = \
+            streamer.plugs['flumotion.component.plugs.loggers.Logger']
+
+        self.logfilter = None
+            
         web_resource.Resource.__init__(self)
+        httpbase.HTTPAuthentication.__init__(self, streamer)
 
     def _streamer_client_removed_cb(self, streamer, sink, fd, reason, stats):
         # this is the callback attached to our flumotion component,
         # not the GStreamer element
-        try:
+        if fd in self._requests:
             request = self._requests[fd]
             self._removeClient(request, fd, stats)
-        except KeyError:
+        else:
             self.warning('[fd %5d] not found in _requests' % fd)
 
     def setRoot(self, path):
         self.putChild(path, self)
-        
-    def setLogfile(self, logfilename):
-        self._logfilename = logfilename
-        self._logfile = open(logfilename, 'a')
 
-    def rotateLogfile(self):
+    def setLogFilter(self, logfilter):
+        self.logfilter = logfilter
+        
+    def rotateLogs(self):
         """
         Close the logfile, then reopen using the previous logfilename
         """
-        self.debug('rotating log file %s' % self._logfile)
-
-        if self._logfile:
-            self._logfile.close()
-        if self._logfilename:
-            self.setLogfile(self._logfilename)
+        for logger in self.loggers:
+            self.debug('rotating logger %r' % logger)
+            logger.rotate()
             
-    def setDomain(self, domain):
-        """
-        Set a domain name on the resource, used in HTTP auth challenges and
-        on the keycard.
-        
-        @type domain: string
-        """
-        self._domain = domain
-        
     def logWrite(self, fd, ip, request, stats):
 
         headers = request.getAllHeaders()
@@ -190,55 +128,27 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
             bytes_sent = -1
             time_connected = -1
 
-        # ip address
-        # ident
-        # authenticated name (from http header)
-        # date
-        # request
-        # request response
-        # bytes sent
-        # referer
-        # user agent
-        # time connected
-        ident = '-'
-        username = '-'
-        date = time.strftime('%d/%b/%Y:%H:%M:%S +0000', time.gmtime())
-        request_str = '%s %s %s' % (request.method,
-                                    request.uri,
-                                    request.clientproto)
-        response = request.code
-        referer = headers.get('referer', '-')
-        user_agent = headers.get('user-agent', '-')
-        format = "%s %s %s [%s] \"%s\" %d %d %s \"%s\" %d\n"
-        msg = format % (ip, ident, username, date, request_str,
-                        response, bytes_sent, referer, user_agent,
-                        time_connected)
-        # make streamer notify manager of this msg
-        self.streamer.sendLog(msg)
-        if not self._logfile:
-            return
-        self._logfile.write(msg)
-        self._logfile.flush()
+        args = {'ip': ip,
+                'time': time.gmtime(),
+                'method': request.method,
+                'uri': request.uri,
+                'username': '-', # FIXME: put the httpauth name
+                'get-parameters': request.args,
+                'clientproto': request.clientproto,
+                'response': request.code,
+                'bytes-sent': bytes_sent,
+                'referer': headers.get('referer', None),
+                'user-agent': headers.get('user-agent', None),
+                'time-connected': time_connected}
+
+        for logger in self.loggers:
+            logger.event('http_session_completed', args)
 
     def setUserLimit(self, limit):
         self.info('setting maxclients to %d' % limit)
-        self.maxclients = limit
-
-    def setBouncerName(self, bouncerName):
-        self.bouncerName = bouncerName
-
-    def setRequesterName(self, requesterName):
-        self.requesterName = requesterName
-
-    def setIssuerClass(self, issuerClass):
-        # FIXME: in the future, we want to make this pluggable and have it
-        # look up somewhere ?
-        if issuerClass == 'HTTPTokenIssuer':
-            self._issuer = HTTPTokenIssuer()
-        elif issuerClass == 'HTTPAuthIssuer':
-            self._issuer = HTTPAuthIssuer()
-        else:
-            raise ValueError, "issuerClass %s not accepted" % issuerClass
+        self.maxclients = self.getMaxAllowedClients(limit)
+        # Log what we actually managed to set it to.
+        self.info('set maxclients to %d' % self.maxclients)
 
     # FIXME: rename to writeHeaders
     """
@@ -275,18 +185,24 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         setHeader('Content-type', content)
         
         # ASF needs a Pragma header for live broadcasts
-        if content in [
-            "video/x-ms-asf",
-            "audio/x-ms-asf",
-        ]:
-            setHeader('Pragma', 'features=broadcast')
+        # Apparently ASF breaks on WMP port 80 if you use the pragma header
+        # - Sep 5 2006
+        #if content in [
+        #    "video/x-ms-asf",
+        #    "audio/x-ms-asf",
+        #]:
+            #setHeader('Pragma', 'features=broadcast')
             
         #self.debug('setting Content-type to %s' % mime)
         ### FIXME: there's a window where Twisted could have removed the
         # fd because the client disconnected.  Catch EBADF correctly here.
         try:
+            # TODO: This is a non-blocking socket, we really should check
+            # return values here, or just let twisted handle all of this
+            # normally, and not hand off the fd until after twisted has
+            # finished writing the headers.
             os.write(fd, 'HTTP/1.0 200 OK\r\n%s\r\n' % ''.join(headers))
-            # tell Twisted we already wrote headers ourselves
+            # tell TwistedWeb we already wrote headers ourselves
             request.startedWriting = True
             return True
         except OSError, (no, s):
@@ -307,41 +223,35 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         
         return True
 
-    def maxAllowedClients(self):
+    def getMaxAllowedClients(self, maxclients):
         """
         maximum number of allowed clients based on soft limit for number of
-        open file descriptors and fd reservation
+        open file descriptors and fd reservation. Increases soft limit to
+        hard limit if possible.
         """
-        limit = resource.getrlimit(resource.RLIMIT_NOFILE)
-        fd_limit = limit[0] - self.__reserve_fds__
-        if self.maxclients != -1 and fd_limit >= self.maxclients:
-            return self.maxclients
+        (softmax, hardmax) = resource.getrlimit(resource.RLIMIT_NOFILE)
+        import sys
+        version = sys.version_info
+
+        if maxclients != -1:
+            neededfds = maxclients + self.__reserve_fds__
+
+            # Bug in python 2.4.3, see http://sourceforge.net/tracker/index.php?func=detail&aid=1494314&group_id=5470&atid=105470
+            if version[:3] == (2,4,3) and not hasattr(socket,"has_2_4_3_patch"):
+                hardmax = 1024
+
+            if neededfds > softmax:
+                lim = min(neededfds, hardmax)
+                resource.setrlimit(resource.RLIMIT_NOFILE, (lim, hardmax))
+                return lim - self.__reserve_fds__
+            else:
+                return maxclients
         else:
-            return fd_limit
+            return softmax - self.__reserve_fds__
 
     def reachedMaxClients(self):
-        return len(self._requests) >= self.maxAllowedClients()
+        return len(self._requests) >= self.maxclients and self.maxclients >= 0
     
-    def authenticate(self, request):
-        """
-        Returns: a deferred returning a keycard or None
-        """
-        keycard = self._issuer.issue(request)
-        if not keycard:
-            self.debug('no keycard from issuer, firing None')
-            return defer.succeed(None)
-
-        keycard.requesterName = self.requesterName
-        keycard._fd = request.transport.fileno()
-        
-        if self.bouncerName == None:
-            self.debug('no bouncer, accepting')
-            return defer.succeed(keycard)
-
-        keycard.setDomain(self._domain)
-        self.debug('sending keycard to bouncer %r' % self.bouncerName)
-        return self.streamer.medium.authenticate(self.bouncerName, keycard)
-
     def _addClient(self, request):
         """
         Add a request, so it can be used for statistics.
@@ -353,6 +263,16 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         fd = request.transport.fileno()
         self._requests[fd] = request
 
+    def _logRequestFromIP(self, ip):
+        """
+        Returns whether we want to log a request from this IP; allows us to
+        filter requests from automated monitoring systems.
+        """
+        if self.logfilter:
+            return not self.logfilter.isInRange(ip)
+        else:
+            return True
+
     def _removeClient(self, request, fd, stats):
         """
         Removes a request and add logging.
@@ -369,82 +289,38 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         """
 
         ip = request.getClientIP()
-        self.logWrite(fd, ip, request, stats)
+        if self._logRequestFromIP(ip):
+            self.logWrite(fd, ip, request, stats)
         self.info('[fd %5d] Client from %s disconnected' % (fd, ip))
 
-        #self.debug('calling request.finish() on [fd %5d]' % fd)
-        #request.finish()
-        #self.debug('called request.finish() on [fd %5d]' % fd)
-        
-        # alternative method of finishing the request; since we already
-        # "stole" the fd, we don't rely on Twisted's Request anymore,
-        # we just loseConnection on the transport,
-        # and delete the Request object,
-        # hopefully triggering garbage collection on all its objects too
-
-        # by doing a callLater we hope to avoid any new clients getting
-        # this fd before we actually purged it completely, since this
-        # code is called from a signal handler.
-        if self.bouncerName and self._fdToKeycard.has_key(fd):
-            id = self._fdToKeycard[fd].id
-            del self._fdToKeycard[fd]
-            del self._idToKeycard[id]
-            self.debug('[fd %5d] asking bouncer %s to remove keycard id %s' % (
-                fd, self.bouncerName, id))
-            self.streamer.medium.removeKeycardId(self.bouncerName, id)
-        if self._fdToDurationCall.has_key(fd):
-            self.debug('[fd %5d] canceling later expiration call' % fd)
-            self._fdToDurationCall[fd].cancel()
-            del self._fdToDurationCall[fd]
+        # We can't call request.finish(), since we already "stole" the fd, we 
+        # just loseConnection on the transport directly, and delete the 
+        # Request object, after cleaning up the bouncer bits.
+        self.cleanupAuth(fd)
 
         self.debug('[fd %5d] closing transport %r' % (fd, request.transport))
+        # This will close the underlying socket. We first remove the request
+        # from our fd->request map, since the moment we call this the fd might
+        # get re-added.
+        del self._requests[fd]
         request.transport.loseConnection()
+
         self.debug('[fd %5d] closed transport %r' % (fd, request.transport))
 
-        # FIXME: os.close is certainly wrong, since the actual socket
-        # will os.close the fd again during its garbage collection
-        #try:
-            #request.transport.close()
-        #except OSError, e:
-        #    if e.errno == errno.EBADF:
-        #        self.warning("Tried to close [fd %5d] which was not open" % fd)
-        #    else:
-        #        self.warning("Error closing [fd %5d]" % fd)
-        #        self.debug("error: %s (%d)" % (e.strerror, e.errno))
-
-        #del request
-
-        del self._requests[fd]
-
-    def _durationCallLater(self, fd):
-        """
-        Expire a client due to a duration expiration.
-        """
-        self.debug('[fd %5d] duration exceeded, expiring client' % fd)
-
-        # we're called from a callLater, so we've already run; just delete
-        if self._fdToDurationCall.has_key(fd):
-            del self._fdToDurationCall[fd]
-            
-        self.debug('[fd %5d] asking streamer to remove client' % fd)
+    def clientDone(self, fd):
         self.streamer.remove_client(fd)
 
-    def expireKeycard(self, keycardId):
-        """
-        Expire a client's connection associated with the keycard Id.
-        """
-        keycard = self._idToKeycard[keycardId]
-        fd = keycard._fd
+    def handleAuthenticatedRequest(self, res, request):
+        if request.method == 'GET':
+            self._handleNewClient(request)
+        elif request.method == 'HEAD':
+            self.debug('handling HEAD request')
+            self._writeHeaders(request)
+            request.finish()
+        else:
+            raise AssertionError
 
-        self.debug('[fd %5d] expiring client' % fd)
-
-        if self._fdToDurationCall.has_key(fd):
-            self.debug('[fd %5d] canceling later expiration call' % fd)
-            self._fdToDurationCall[fd].cancel()
-            del self._fdToDurationCall[fd]
-
-        self.debug('[fd %5d] asking streamer to remove client' % fd)
-        self.streamer.remove_client(fd)
+        return res
 
     ### resource.Resource methods
 
@@ -466,22 +342,25 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
             return self._handleMaxClients(request)
 
         self.debug('_render(): asked for (possible) authentication')
-        d = self.authenticate(request)
-        d.addCallback(self._authenticatedCallback, request)
-        d.addErrback(self._authenticatedErrback, request)
+        d = self.startAuthentication(request)
+        d.addCallback(self.handleAuthenticatedRequest, request)
 
         # we MUST return this from our _render.
-        # FIXME: check if this is true
-        # FIXME: check how we later handle not authorized
         return server.NOT_DONE_YET
 
+    def authenticateKeycard(self, bouncerName, keycard):
+        return self.streamer.medium.authenticate(bouncerName, keycard)
+
+    def cleanupKeycard(self, bouncerName, keycard):
+        return self.streamer.medium.removeKeycardId(bouncerName, keycard.id)
+
     def _handleNotReady(self, request):
         self.debug('Not sending data, it\'s not ready')
         return server.NOT_DONE_YET
         
     def _handleMaxClients(self, request):
         self.debug('Refusing clients, client limit %d reached' %
-            self.maxAllowedClients())
+            self.maxclients)
 
         request.setHeader('content-type', 'text/html')
         request.setHeader('server', HTTP_VERSION)
@@ -492,59 +371,6 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         return ERROR_TEMPLATE % {'code': error_code,
                                  'error': http.RESPONSES[error_code]}
 
-    def _authenticatedCallback(self, keycard, request):
-        # !: since we are a callback, the incoming fd might have gone away
-        # and closed
-        self.debug('_authenticatedCallback: keycard %r' % keycard)
-        if not keycard:
-            self._handleUnauthorized(request)
-            return
-
-        # properly authenticated
-        if request.method == 'GET':
-            fd = request.transport.fileno()
-
-            if self.bouncerName:
-                self._fdToKeycard[fd] = keycard
-                self._idToKeycard[keycard.id] = keycard
-
-            if keycard.duration:
-                self.debug('new connection on %d will expire in %f seconds' % (
-                    fd, keycard.duration))
-                self._fdToDurationCall[fd] = reactor.callLater(
-                    keycard.duration, self._durationCallLater, fd)
-
-            self._handleNewClient(request)
-
-        elif request.method == 'HEAD':
-            self.debug('handling HEAD request')
-            self._writeHeaders(request)
-            request.finish()
-        else:
-            raise AssertionError
-
-    def _authenticatedErrback(self, failure, request):
-        failure.trap(errors.UnknownComponentError)
-        self._handleUnauthorized(request)
-        
-    def _handleUnauthorized(self, request):
-        self.debug('client from %s is unauthorized' % (request.getClientIP()))
-        request.setHeader('content-type', 'text/html')
-        request.setHeader('server', HTTP_VERSION)
-        if self._domain:
-            request.setHeader('WWW-Authenticate',
-                              'Basic realm="%s"' % self._domain)
-            
-        error_code = http.UNAUTHORIZED
-        request.setResponseCode(error_code)
-        
-        # we have to write data ourselves,
-        # since we already returned NOT_DONE_YET
-        html = ERROR_TEMPLATE % {'code': error_code,
-                                 'error': http.RESPONSES[error_code]}
-        request.write(html)
-        request.finish()
-
     def _handleNewClient(self, request):
         # everything fulfilled, serve to client
         fdi = request.fdIncoming
@@ -557,13 +383,16 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         # the reactor
         # spiv told us to remove* on request.transport, and that works
         # then we figured out that a new request is only a Reader, so we
-        # remove the removedWriter
+        # remove the removedWriter - this is because we never write to the
+        # socket through twisted, only with direct os.write() calls from
+        # _writeHeaders.
         fd = fdi
         self.debug("taking away [fd %5d] from Twisted" % fd)
         reactor.removeReader(request.transport)
         #reactor.removeWriter(request.transport)
     
-        # check if it's really an open fd
+        # check if it's really an open fd (i.e. that twisted didn't close it
+        # before the removeReader() call)
         import fcntl
         try:
             fcntl.fcntl(fd, fcntl.F_GETFL)
@@ -584,5 +413,17 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
     render_GET = _render
     render_HEAD = _render
 
-class HTTPRoot(web_resource.Resource):
-    pass
+class HTTPRoot(web_resource.Resource, log.Loggable):
+    logCategory = "httproot"
+
+    def getChildWithDefault(self, path, request):
+        # we override this method so that we can look up tree resources
+        # directly without having their parents.
+        # There's probably a more Twisted way of doing this, but ...
+        fullPath = path
+        if request.postpath:
+            fullPath += '/' + string.join(request.postpath, '/')
+        self.debug("Incoming request %r for path %s" % (request, fullPath))
+        r = web_resource.Resource.getChildWithDefault(self, fullPath, request)
+        self.debug("Returning resource %r" % r)
+        return r
