@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -31,6 +31,8 @@ from flumotion.component.converters.overlay import genimg
 import tempfile
 
 class Overlay(feedcomponent.ParseLaunchComponent):
+    checkTimestamp = True
+    checkOffset = True
     _filename = None
 
     def get_pipeline_string(self, properties):
@@ -38,23 +40,10 @@ class Overlay(feedcomponent.ParseLaunchComponent):
         # function.  PLEASE MAKE THE PAIN GO AWAY ? <- might not be
         # necessary still
 
-        # GST 0.8
-
         # we need an element that does RGBA -> AYUV so we can overlay png
         # this got added to ffmpegcolorspace in 0.8.5
         addalpha = 'ffmpegcolorspace'
 
-        import gst
-        if gst.gst_version < (0,9):
-            from flumotion.worker.checks import video
-            if not video.check_ffmpegcolorspace_AYUV():
-                # alphacolor element works too for the colorspace conversion,
-                # but has bugs for non-multiples of 4 or eight
-                addalpha = 'ffmpegcolorspace ! alpha'
-                log.info('Using gst-plugins older than 0.8.5, consider '
-                         'upgrading if you notice a diagonal green line '
-                         'in your video output.')
-
         source = self.config['source'][0]
         eater = '@ eater:%s @' % source
 
@@ -67,31 +56,32 @@ class Overlay(feedcomponent.ParseLaunchComponent):
         
         return pipeline
 
-    # FIXME: move to configure_pipeline
-    def do_start(self, eatersData, feedersData, clocking):
+    def configure_pipeline(self, pipeline, properties):
+        self.fixRenamedProperties(properties, [ 
+                ('show_text',    'show-text'), 
+                ('fluendo_logo', 'fluendo-logo'), 
+                ('cc_logo',      'cc-logo'), 
+                ('xiph_logo',    'xiph-logo')
+            ]) 
+
         # create temp file
         (fd, self._filename) = tempfile.mkstemp('flumotion.png')
         os.close(fd)
 
-        props = self.config['properties']
-
         text = None
-        if props.get('show_text', False):
-            text = props.get('text', 'set the "text" property')
+        if properties.get('show-text', False):
+            text = properties.get('text', 'set the "text" property')
         genimg.generate_overlay(self._filename,
                                 text,
-                                props.get('fluendo_logo', False),
-                                props.get('cc_logo', False),
-                                props.get('xiph_logo', False),
-                                props['width'],
-                                props['height'])
+                                properties.get('fluendo-logo', False),
+                                properties.get('cc-logo', False),
+                                properties.get('xiph-logo', False),
+                                properties['width'],
+                                properties['height'])
         
         source = self.get_element('source')
         source.set_property('location', self._filename)
 
-        return feedcomponent.ParseLaunchComponent.do_start(self,
-            eatersData, feedersData, clocking)
-
     def do_stop(self):
         # clean up our temp file
         if self._filename:
@@ -99,6 +89,7 @@ class Overlay(feedcomponent.ParseLaunchComponent):
             os.unlink(self._filename)
             self._filename = None
         else:
-            self.debug('Temporary overlay already gone, did we not start up correctly ?')
-        return defer.succeed(None)
+            self.debug("Temporary overlay already gone, " \
+                "did we not start up correctly ?")
+        return feedcomponent.ParseLaunchComponent.do_stop(self)
         
