@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -46,20 +46,20 @@ class Colorbalance(feedcomponent.Effect):
         self.debug("colorbalance init")
         feedcomponent.Effect.__init__(self, name)
         self._element = element
-        # In 0.8 there was a signal on the element, in 0.10 it is now a
-        # message on the bus
-        if gst.gst_version < (0,9):
-            element.connect('state-change', self._source_state_changed_cb,
+        if pipeline:
+            bus = pipeline.get_bus()
+            bus.connect('message::state-changed', 
+                self._bus_message_received_cb,
                 hue, saturation, brightness, contrast)
-        else:
-            if pipeline:
-                bus = pipeline.get_bus()
-                bus.connect('message::state-changed', 
-                    self._bus_message_received_cb,
-                    hue, saturation, brightness, contrast)
-        
+    
         self._channels = None
 
+    def setUIState(self, state):
+        feedcomponent.Effect.setUIState(self, state)
+        if state:
+            for k in 'Hue', 'Saturation', 'Brightness', 'Contrast':
+                state.addKey('colorbalance-%s' % k, 0.0)
+
     # State change handling for 0.10
     def _bus_message_received_cb(self, bus, message, hue, saturation, 
         brightness, contrast):
@@ -74,7 +74,6 @@ class Colorbalance(feedcomponent.Effect):
             if old == gst.STATE_READY and new == gst.STATE_PAUSED:
                 self._setInitialColorBalance(hue, 
                     saturation, brightness, contrast)
-            
 
     def effect_setColorBalanceProperty(self, which, value):
         """
@@ -90,66 +89,36 @@ class Colorbalance(feedcomponent.Effect):
 
         for i in self._channels:
             if i.label == which:
-                device_value = _percent_to_value(value,
-                    i.min_value, i.max_value)
-                self.debug("setting percentage: %.1f, value %d <= %d <= %d" % (
-                    value, i.min_value, device_value, i.max_value))
-                self._element.set_value(i, device_value)
+                if value:
+                    device_value = _percent_to_value(value,
+                        i.min_value, i.max_value)
+                    self.debug("setting percentage: %.1f, "
+                               "value %d <= %d <= %d",
+                               value, i.min_value, device_value,
+                               i.max_value)
+                    self._element.set_value(i, device_value)
                 device_value = self._element.get_value(i)
-                self.debug("actually set: %.1f, value %d <= %d <= %d" % (
-                    value, i.min_value, device_value, i.max_value))
                 percent = _value_to_percent(device_value,
                     i.min_value, i.max_value)
+                self.debug('device says %s=%.1f', i.label, percent)
                 # notify all others too
-                if not self.component:
-                    self.warning("effect %s doesn't have a component" %
+                if not self.uiState:
+                    self.warning("effect %s doesn't have a uiState" %
                         self.name)
                 else:
-                    self.component.adminCallRemote("effectPropertyChanged",
-                        self.name, which, percent)
+                    self.uiState.set('colorbalance-%s' % which, percent)
                 return percent
 
         # didn't find it
         return value
 
-    def effect_getColorBalanceProperties(self):
-        """
-        Returns: a list of (label, value) tuples.
-        """
-        retval = []
-        if not self._channels:
-            return retval
-        for i in self._channels:
-            self.debug('colorbalance %s: %d <= %d <= %d' % (
-                i.label, i.min_value, self._element.get_value(i), i.max_value))
-            percent_value = _value_to_percent(self._element.get_value(i),
-                 i.min_value, i.max_value)
-            self.debug('colorbalance value: %f' % percent_value)
-            retval.append([i.label, percent_value])
-
-        return retval
-
-    # called to set initial properties based on state change
-    def _source_state_changed_cb(self, element, old, new, 
-                                 hue, saturation, brightness, contrast):
-        if not (old == gst.STATE_READY and new == gst.STATE_PAUSED):
-            return
-
-        self.debug('source is PAUSED, setting initial color balance properties')
-
-        self._setInitialColorBalance(hue, saturation, brightness, contrast)
-    
     def _setInitialColorBalance(self, hue, saturation, brightness, contrast):
         self._channels = self._element.list_colorbalance_channels()
         self.debug('colorbalance channels: %d' % len(self._channels))
-        if hue:
-            self.effect_setColorBalanceProperty('Hue', hue)
-        if saturation:
-            self.effect_setColorBalanceProperty('Saturation', saturation)
-        if brightness:
-            self.effect_setColorBalanceProperty('Brightness', brightness)
-        if contrast:
-            self.effect_setColorBalanceProperty('Contrast', contrast)
+        self.effect_setColorBalanceProperty('Hue', hue)
+        self.effect_setColorBalanceProperty('Saturation', saturation)
+        self.effect_setColorBalanceProperty('Brightness', brightness)
+        self.effect_setColorBalanceProperty('Contrast', contrast)
 
 def _value_to_percent(value, min, max):
     """
