@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -21,8 +21,4 @@
 
 import gst
 
-
-if gst.gst_version < (0, 9):
-    from vorbis08 import Vorbis
-else:
-    from vorbis010 import Vorbis
+from vorbis010 import Vorbis
