@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -29,10 +29,7 @@ class Multipart(feedcomponent.ParseLaunchComponent):
 
         pipeline = 'multipartmux name=muxer '
         for eater in sources:
-            if gst.gst_version < (0, 9):
-                tmpl = '{ @ eater:%s @ ! queue } ! muxer. '
-            else:
-                tmpl = '@ eater:%s @ ! queue ! muxer. '
+            tmpl = '@ eater:%s @ ! queue ! muxer. '
             pipeline += tmpl % eater
 
         pipeline += 'muxer.'
