@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -23,7 +23,9 @@ import gst
 
 from flumotion.component import feedcomponent
 
-class Ogg(feedcomponent.ParseLaunchComponent):
+class Ogg(feedcomponent.MultiInputParseLaunchComponent):
+    checkTimestamp = True
+
     def do_check(self):
         self.debug('running Ogg check')
         import checks
@@ -35,21 +37,11 @@ class Ogg(feedcomponent.ParseLaunchComponent):
         for m in result.messages:
             self.addMessage(m)
 
-    def get_pipeline_string(self, properties):
-        sources = self.config['source']
-
+    def get_muxer_string(self, properties):
         maxDelay = 500 * 1000 * 1000
         maxPageDelay = 500 * 1000 * 1000
-        pipeline = 'oggmux name=muxer max-delay=%d max-page-delay=%d ' % (
+        muxer = 'oggmux name=muxer max-delay=%d max-page-delay=%d' % (
             maxDelay, maxPageDelay)
 
-        for eater in sources:
-            if gst.gst_version < (0, 9):
-                tmpl = '{ @ eater:%s @ ! queue max-size-buffers=16 } ! muxer. '
-            else:
-                tmpl = '@ eater:%s @ ! queue max-size-buffers=16 ! muxer. '
-            pipeline += tmpl % eater
-                
-        pipeline += 'muxer.'
-        
-        return pipeline
+        return muxer
+
