@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -26,14 +26,13 @@ class SoundcardAdminGtk(admin_gtk.BaseAdminGtk):
     def setup(self):
         self._nodes = {}
         volume = vadmin_gtk.VolumeAdminGtkNode(self.state, self.admin,
-                                               'inputVolume')
-        self._nodes['Volume'] = volume
-
-    def getNodes(self):
-        return self._nodes
+                                               'inputVolume',
+                                               'Input Volume')
+        self.nodes['Volume'] = volume
+        return admin_gtk.BaseAdminGtk.setup(self)
 
     def component_volumeChanged(self, channel, rms, peak, decay):
-        volume = self._nodes['Volume']
+        volume = self.nodes['Volume']
         volume.volumeChanged(channel, rms, peak, decay)
 
     def component_effectVolumeSet(self, effect, volume):
@@ -44,7 +43,7 @@ class SoundcardAdminGtk(admin_gtk.BaseAdminGtk):
         if effect != 'inputVolume':
             self.warning('Unknown effect %s in %r' % (effect, self))
             return
-        v = self._nodes['Volume']
+        v = self.nodes['Volume']
         v.volumeSet(volume)
 
 
