@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -21,28 +21,18 @@
 
 import gst
 
+from flumotion.common import errors, gstreamer, messages
 from flumotion.component import feedcomponent
 
-class VideoTestMedium(feedcomponent.FeedComponentMedium):
-    def __init__(self, comp):
-        feedcomponent.FeedComponentMedium.__init__(self, comp)
+from flumotion.common.messages import N_
+T_ = messages.gettexter('flumotion')
 
-    def setup(self, config):
-        # called after component's setup
-        # connect to pattern notify
-        source = self.comp.get_element('source')
-        source.connect('notify::pattern', self.cb_pattern_notify)
-
-    def cb_pattern_notify(self, object, pspec):
-        pattern = object.get_property('pattern')
-        self.debug('pattern has changed, notifying')
-        self.callRemote('adminCallRemote', 'propertyChanged', 'pattern',
-            int(pattern))
 
 class VideoTest(feedcomponent.ParseLaunchComponent):
 
-    component_medium_class = VideoTestMedium
-    
+    def init(self):
+        self.uiState.addKey('pattern', 0)
+
     def get_pipeline_string(self, properties):
         format = properties.get('format', 'video/x-raw-yuv')
 
@@ -57,26 +47,47 @@ class VideoTest(feedcomponent.ParseLaunchComponent):
 
         if 'framerate' in properties:
             framerate = properties['framerate']
-            if gst.gst_version < (0,9):
-                struct['framerate'] = float(framerate[0]) / framerate[1]
-            else:
-                struct['framerate'] = gst.Fraction(framerate[0], framerate[1])
+            struct['framerate'] = gst.Fraction(framerate[0], framerate[1])
 
         # If RGB, set something ffmpegcolorspace can convert.
         if format == 'video/x-raw-rgb':
             struct['red_mask'] = 0xff00
         caps = gst.Caps(struct)
         
-        if gst.gst_version < (0,9):
-            is_live = 'sync=true'
-        else:
-            is_live = 'is-live=true'
+        is_live = 'is-live=true'
 
-        return 'videotestsrc %s name=source ! %s' % (is_live, caps)
+        return "videotestsrc %s name=source ! " \
+            "identity name=identity silent=TRUE ! %s" % (
+            is_live, caps)
         
     # Set properties
     def configure_pipeline(self, pipeline, properties):
+        def notify_pattern(obj, pspec):
+            self.uiState.set('pattern', int(obj.get_property('pattern')))
+
         source = self.get_element('source')
+        source.connect('notify::pattern', notify_pattern)
         if 'pattern' in properties:
             source.set_property('pattern', properties['pattern'])
 
+        if 'drop-probability' in properties:
+            vt = gstreamer.get_plugin_version('coreelements')
+            if not vt:
+                raise errors.MissingElementError('identity')
+            if not vt > (0, 10, 12, 0):
+                self.addMessage(
+                    messages.Warning(T_(N_(
+                        "The 'drop-probability' property is specified, but "
+                        "it only works with GStreamer core newer than 0.10.12. "
+                        "You should update your version of GStreamer."))))
+            else:
+                drop_probability = properties['drop-probability']
+                if drop_probability < 0.0 or drop_probability > 1.0:
+                    self.addMessage(
+                        messages.Warning(T_(N_(
+                            "The 'drop-probability' property can only be "
+                            "between 0.0 and 1.0."))))
+                else:
+                    identity = self.get_element('identity')
+                    identity.set_property('drop-probability',
+                        drop_probability)
