@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -26,11 +26,10 @@ class WebcamAdminGtk(BaseAdminGtk):
     def setup(self):
         self._nodes = {}
         colorbalance = ColorbalanceAdminGtkNode(self.state, self.admin,
-                                                'outputColorbalance')
-        self._nodes['Colorbalance'] = colorbalance
-
-    def getNodes(self):
-        return self._nodes
+                                                'outputColorbalance',
+                                                'Output Colorbalance')
+        self.nodes['Colorbalance'] = colorbalance
+        return BaseAdminGtk.setup(self)
 
     def component_effectPropertyChanged(self, effectName, propertyName, value):
         self.debug("effect %s has property %s changed to %r" % (
@@ -40,7 +39,7 @@ class WebcamAdminGtk(BaseAdminGtk):
             self.warning("Unknown effect '%s'" % effectName)
             return
             
-        cb = self._nodes['Colorbalance']
+        cb = self.nodes['Colorbalance']
         cb.propertyChanged(propertyName, value)
 
 GUIClass = WebcamAdminGtk
