@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -25,6 +25,7 @@ the job-side half of the worker-job connection
 
 import os
 import resource
+import sys
 
 # I've read somewhere that importing the traceback module messes up the
 # exception state, so it's better to import it globally instead of in the
@@ -32,74 +33,22 @@ import resource
 # import traceback
 
 from twisted.cred import credentials
-from twisted.internet import reactor
-from twisted.python import reflect, failure
+from twisted.internet import reactor, defer
+from twisted.python import failure
 from twisted.spread import pb
 
 from flumotion.common import config, errors, interfaces, log, registry, keycards
 from flumotion.common import medium, package
+from flumotion.common.reflectcall import createComponent
 from flumotion.component import component
+
+from flumotion.twisted import fdserver
+from flumotion.twisted import pb as fpb
+from flumotion.twisted import defer as fdefer
+
 from flumotion.twisted.defer import defer_generator_method
 from flumotion.twisted.compat import implements
 
-def createComponent(moduleName, methodName):
-    """
-    @param moduleName: name of the module to create the component from
-    @type  moduleName: string
-    @param methodName: the factory method to use to create the component
-    @type  methodName: string
-
-    Invokes the entry point for a component in the given module using the
-    given factory method, thus creating the component.
-
-    @rtype: L{flumotion.component.component.BaseComponent}
-    """
-    log.debug('component', 'Loading moduleName %s' % moduleName)
-    try:
-        module = reflect.namedAny(moduleName)
-    except ValueError:
-        raise errors.ComponentCreateError(
-            "module %s could not be found" % moduleName)
-    except ImportError, e:
-        raise errors.ComponentCreateError(
-            "module %s could not be imported (%s)" % (moduleName,
-                log.getExceptionMessage(e, filename='flumotion')))
-    except SyntaxError, e:
-        raise errors.ComponentCreateError(
-            "module %s has a syntax error in %s:%d" % (
-                moduleName, e.filename, e.lineno))
-    except Exception, e:
-        raise errors.ComponentCreateError(
-            "Exception %r during import of module %s (%r)" % (
-                e.__class__.__name__, moduleName, e.args))
-        
-    if not hasattr(module, methodName):
-        msg = 'no function %s in module %s' % (methodName, moduleName)
-        raise errors.ComponentCreateError(msg)
-        
-    # Create the component with the specified configuration
-    # directives. Note that this can't really be moved from here
-    # since it gets called by the job from another process
-    # and we don't want to create it in the main process, since
-    # we're going to listen to ports and other stuff which should
-    # be separated from the main process.
-
-    log.debug('job', 'calling entry point %s.%s()' % (
-        moduleName, methodName))
-    try:
-        component = getattr(module, methodName)()
-    except errors.ComponentCreateError:
-        # already nicely formatted, so fall through
-        log.debug('job', 'letting ComponentCreateError fall through')
-        raise
-    except Exception, e:
-        msg = log.getExceptionMessage(e)
-        log.warning('job', msg)
-        log.warning('job', 'raising errors.ComponentCreateError')
-        raise errors.ComponentCreateError(msg)
-    log.debug('job', 'returning component %r' % component)
-    return component
-
 class JobMedium(medium.BaseMedium):
     """
     I am a medium between the job and the worker's job avatar.
@@ -110,6 +59,7 @@ class JobMedium(medium.BaseMedium):
     @type component: L{flumotion.component.component.BaseComponent}
     """
     logCategory = 'jobmedium'
+    remoteLogName = 'jobavatar'
 
     implements(interfaces.IJobMedium)
 
@@ -125,8 +75,10 @@ class JobMedium(medium.BaseMedium):
         self._managerKeycard = None
         self._componentClientFactory = None # from component to manager
 
+        self._hasStoppedReactor = False
+
     ### pb.Referenceable remote methods called on by the WorkerBrain
-    def remote_bootstrap(self, workerName, host, port, transport, keycard,
+    def remote_bootstrap(self, workerName, host, port, transport, authenticator,
             packagePaths):
         """
         I receive the information on how to connect to the manager. I also set
@@ -134,32 +86,33 @@ class JobMedium(medium.BaseMedium):
         
         Called by the worker's JobAvatar.
         
-        @param workerName:   the name of the worker running this job
-        @type  workerName:   str
-        @param host:         the host that is running the manager
-        @type  host:         str
-        @param port:         port on which the manager is listening
-        @type  port:         int
-        @param transport:    'tcp' or 'ssl'
-        @type  transport:    str
-        @param keycard:      credentials used to log in to the manager
-        @type  keycard:      L{flumotion.common.keycards.Keycard}
-        @param packagePaths: ordered list of (package name, package path) tuples
-        @type  packagePaths: list of (str, str)
+        @param workerName:    the name of the worker running this job
+        @type  workerName:    str
+        @param host:          the host that is running the manager
+        @type  host:          str
+        @param port:          port on which the manager is listening
+        @type  port:          int
+        @param transport:     'tcp' or 'ssl'
+        @type  transport:     str
+        @param authenticator: remote reference to the worker-side authenticator
+        @type  authenticator: L{twisted.spread.pb.RemoteReference} to a
+                              L{flumotion.twisted.pb.Authenticator}
+        @param packagePaths:  ordered list of
+                              (package name, package path) tuples
+        @type  packagePaths:  list of (str, str)
         """
         assert isinstance(workerName, str)
         assert isinstance(host, str)
         assert isinstance(port, int)
         assert transport in ('ssl', 'tcp')
-        assert isinstance(keycard, keycards.Keycard)
+        assert isinstance(authenticator, pb.RemoteReference)
         assert isinstance(packagePaths, list)
 
-        self.debug('remote_bootstrap')
         self._workerName = workerName
         self._managerHost = host
         self._managerPort = port
         self._managerTransport = transport
-        self._managerKeycard = keycard
+        self._authenticator = fpb.RemoteAuthenticator(authenticator)
         
         packager = package.getPackager()
         for name, path in packagePaths:
@@ -167,7 +120,7 @@ class JobMedium(medium.BaseMedium):
             self.log('... from path %s' % path)
             packager.registerPackagePath(path, name)
 
-    def remote_create(self, avatarId, type, moduleName, methodName, config):
+    def remote_create(self, avatarId, type, moduleName, methodName, nice=0):
         """
         I am called on by the worker's JobAvatar to create a component.
         
@@ -179,21 +132,41 @@ class JobMedium(medium.BaseMedium):
         @type  moduleName: str
         @param methodName: the factory method to use to create the component
         @type  methodName: str
-        @param config:     the configuration dictionary
-        @type  config:     dict
+        @param nice:       the nice level
+        @type  nice:       int
         """
-        self.debug('remote_create, avatarId %s' % avatarId)
         self.avatarId = avatarId
         self.logName = avatarId
 
         self.component = self._createComponent(avatarId, type, moduleName,
-            methodName, config)
+            methodName, nice)
+        self.component.setShutdownHook(self._componentStopped)
+
+    def _componentStopped(self):
+        # stop reactor from a callLater so remote methods finish nicely
+        reactor.callLater(0, self.shutdown)
 
     def remote_stop(self):
-        self.debug('remote_stop() called')
-        # stop reactor from a callLater so this remote method finishes
-        # nicely
-        reactor.callLater(0, self.shutdown)
+        if self.component:
+            self.debug('stopping component and shutting down')
+            self.component.stop()
+        else:
+            reactor.callLater(0, self.shutdown)
+
+    def shutdownHandler(self):
+        dlist = []
+        if self.hasRemoteReference():
+            # tell the worker we are shutting down
+            dlist.append(self.callRemote("cleanShutdown"))
+        if self.component:
+            medium = self.component.medium
+            if medium.hasRemoteReference():
+                dlist.append(medium.callRemote("cleanShutdown"))
+
+        # We mustn't fire the deferred returned from here except from a 
+        # callLater.
+        dl = defer.DeferredList(dlist, fireOnOneErrback=False)
+        return fdefer.defer_call_later(dl)
 
     ### our methods
     def shutdown(self):
@@ -201,16 +174,12 @@ class JobMedium(medium.BaseMedium):
         Shut down the job process completely, cleaning up the component
         so the reactor can be left from.
         """
-        if self.component:
-            self.debug('stopping component')
-            self.component.stop()
-            self.debug('stopped component')
-        self.debug('stopping reactor')
-        reactor.stop()
-        self.debug('reactor stopped, exiting process')
-
-        # FIXME: temporary hack
-        # os._exit(0)
+        if self._hasStoppedReactor:
+            self.debug("Not stopping reactor again, already shutting down")
+        else:
+            self._hasStoppedReactor = True
+            self.info("Stopping reactor in job process")
+            reactor.stop()
 
     def _setNice(self, nice):
         if not nice:
@@ -233,8 +202,7 @@ class JobMedium(medium.BaseMedium):
             
         resource.setrlimit(resource.RLIMIT_CORE, (hard, hard))
         
-    # FIXME: we only use "nice" from config anymore
-    def _createComponent(self, avatarId, type, moduleName, methodName, config):
+    def _createComponent(self, avatarId, type, moduleName, methodName, nice=0):
         """
         Create a component of the given type.
         Log in to the manager with the given avatarId.
@@ -247,22 +215,16 @@ class JobMedium(medium.BaseMedium):
         @type  moduleName: str
         @param methodName: name of the factory method to create the component
         @type  methodName: str
-        @param config:     the configuration dictionary
-        @type  config:     dict
+        @param nice:       the nice level to run with
+        @type  nice:       int
         """
-        
-        self.debug('_createComponent(): config dictionary is: %r' % config)
-
         self.info('Creating component "%s" of type "%s"' % (avatarId, type))
         #self.info('setting up signals')
         #signal.signal(signal.SIGINT, signal.SIG_IGN)
 
-        self._setNice(config.get('nice', 0))
+        self._setNice(nice)
         self._enableCoreDumps()
         
-        # FIXME: we put avatarId in the config for now
-        # but it'd be nicer to do this outside of config, so do this
-        config['avatarId'] = avatarId
         try:
             comp = createComponent(moduleName, methodName)
         except Exception, e:
@@ -274,7 +236,14 @@ class JobMedium(medium.BaseMedium):
                 msg = e.args[0]
             self.warning(
                 "raising ComponentCreateError(%s) and stopping job" % msg)
-            reactor.callLater(0, self.shutdown)
+            # This is a Nasty Hack. We raise ComponentCreateError, which can be
+            # caught on the other side and marshalled as a reasonably 
+            # comprehensible error message. However, if we shutdown immediately,
+            # the PB connection won't be available, so the worker will just get
+            # an error about that! So, instead, we shut down in a tenth of a
+            # second, usually allowing the worker to get scheduled and read the
+            # exception over PB. Ick!
+            reactor.callLater(0.1, self.shutdown)
             raise errors.ComponentCreateError(msg)
 
         comp.setWorkerName(self._workerName)
@@ -284,14 +253,13 @@ class JobMedium(medium.BaseMedium):
         managerClientFactory = component.ComponentClientFactory(comp)
         self._componentClientFactory = managerClientFactory
         self.debug('created ComponentClientFactory %r' % managerClientFactory)
-        keycard = self._managerKeycard
-        keycard.avatarId = avatarId
-        managerClientFactory.startLogin(keycard)
+        self._authenticator.avatarId = avatarId
+        managerClientFactory.startLogin(self._authenticator)
 
         host = self._managerHost
         port = self._managerPort
         transport = self._managerTransport
-        self.debug('logging in')
+        self.debug('logging in with authenticator %r' % self._authenticator)
         if transport == "ssl":
             from twisted.internet import ssl
             self.info('Connecting to manager %s:%d with SSL' % (host, port))
@@ -301,19 +269,60 @@ class JobMedium(medium.BaseMedium):
             self.info('Connecting to manager %s:%d with TCP' % (host, port))
             reactor.connectTCP(host, port, managerClientFactory)
         else:
-            self.error('Unknown transport protocol %s' % self._managerTransport)
+            self.warning('Unknown transport protocol %s' % self._managerTransport)
 
         return comp
         
+class JobClientBroker(pb.Broker, log.Loggable):
+    """
+    A pb.Broker subclass that handles FDs being passed (with associated data)
+    over the same connection as the normal PB data stream.
+    When an FD is seen, the FD should be added to a given eater or feeder
+    element.
+    """
+    def __init__(self, connectionClass, **kwargs):
+        """
+        @param connectionClass: a subclass of L{twisted.internet.tcp.Connection}
+        """
+        pb.Broker.__init__(self, **kwargs)
+
+        self._connectionClass = connectionClass
+
+    def fileDescriptorsReceived(self, fds, message):
+        # file descriptors get delivered to the component
+        self.debug('received fds %r, message %r' % (fds, message))
+        if message.startswith('sendFeed '):
+            def parseargs(_, feedName, eaterId=None):
+                return feedName, eaterId
+            feedName, eaterId = parseargs(*message.split(' '))
+            self.factory.medium.component.feedToFD(feedName, fds[0],
+                                                   os.close, eaterId)
+        elif message.startswith('receiveFeed '):
+            feedId = message[len('receiveFeed '):]
+            self.factory.medium.component.eatFromFD(feedId, fds[0])
+        elif message == 'redirectStdout':
+            self.debug('told to rotate stdout to fd %d', fds[0])
+            os.dup2(fds[0], sys.stdout.fileno())
+            os.close(fds[0])
+            self.debug('rotated stdout')
+        elif message == 'redirectStderr':
+            self.debug('told to rotate stderr to fd %d', fds[0])
+            os.dup2(fds[0], sys.stderr.fileno())
+            os.close(fds[0])
+            self.info('rotated stderr')
+        else:
+            self.warning('Unknown message received: %r' % message)
+
 class JobClientFactory(pb.PBClientFactory, log.Loggable):
     """
     I am a client factory that logs in to the WorkerBrain.
-    I live in the flumotion-worker job process.
+    I live in the flumotion-job process spawned by the worker.
 
     @cvar medium: the medium for the JobHeaven to access us through
     @type medium: L{JobMedium}
     """
     logCategory = "job"
+    perspectiveInterface = interfaces.IJobMedium    
 
     def __init__(self, id):
         """
@@ -325,8 +334,16 @@ class JobClientFactory(pb.PBClientFactory, log.Loggable):
         self.medium = JobMedium()
         self.logName = id
         self.login(id)
+
+        # use an FD-passing broker instead
+        self.protocol = JobClientBroker
             
     ### pb.PBClientFactory methods
+    def buildProtocol(self, addr):
+        p = self.protocol(fdserver.FDServer)
+        p.factory = self
+        return p
+
     # FIXME: might be nice if jobs got a password to use to log in to brain
     def login(self, username):
         self.info('Logging in to worker')
@@ -348,7 +365,10 @@ class JobClientFactory(pb.PBClientFactory, log.Loggable):
     
     # the only way stopFactory can be called is if the WorkerBrain closes
     # the pb server.  Ideally though we would have gotten a notice before.
+    # This ensures we shut down the component/job in ALL cases where the worker
+    # goes away.
     def stopFactory(self):
         self.debug('shutting down medium')
         self.medium.shutdown()
         self.debug('shut down medium')
+
