@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -27,7 +27,7 @@ from twisted.internet import reactor
 from flumotion.configure import configure
 from flumotion.common import log, keycards, common, errors
 from flumotion.job import job
-from flumotion.twisted import credentials
+from flumotion.twisted import credentials, fdserver
 
 def main(args):
     parser = optparse.OptionParser()
@@ -40,15 +40,19 @@ def main(args):
 
     # handle all options
     if options.version:
-        print common.version("flumotion-worker")
+        print common.version("flumotion-job")
         return 0
 
     # check if a config file was specified; if so, parse config and copy over
     if len(args) != 3:
-        parser.error("must pass an avatarId and a path to the socket: %r" % args)
+        parser.error("must pass an avatarId and a path to the socket: %r" %
+            args)
     avatarId = args[1]
     socket = args[2]
         
+    # log our standardized starting marker
+    log.info('job', "Starting job '%s'" % avatarId)
+
     # register all package paths (FIXME: this should go away when
     # components and all deps come from manager)
     # this is still necessary so that code from other projects can be imported
@@ -58,8 +62,8 @@ def main(args):
     log.info('job', 'Connecting to worker on socket %s' % (socket))
 
     job_factory = job.JobClientFactory(avatarId)
-    reactor.connectUNIX(socket, job_factory)
-    log.info('job', 'Started job on pid %d' % os.getpid())
+    reactor.connectWith(fdserver.FDConnector, socket, job_factory,
+        10, checkPID=False)
 
     # should probably move this to boot
     if 'FLU_PROFILE' in os.environ:
@@ -77,10 +81,18 @@ def main(args):
         except ImportError, e:
             print ('Profiling requested, but statprof is not available (%s)'
                    % e)
+
+    reactor.addSystemEventTrigger('before', 'shutdown', 
+        job_factory.medium.shutdownHandler)
     
-    log.debug('job', 'Starting reactor')
+    # log our standardized started marker
+    log.info('job', "Started job '%s'" % avatarId)
+
     reactor.run()
 
-    log.debug('job', 'Reactor stopped')
+    # log our standardized stopping marker
+    log.info('job', "Stopping job '%s'" % avatarId)
+    # log our standardized stopped marker
+    log.info('job', "Stopped job '%s'" % avatarId)
 
     return 0
