@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -69,12 +69,23 @@ def main(args):
         print '\nAvailable components:\n'
         for name, c in components:
             print '  %s' % name
+        plugs = [(p.getType(), p) for p in r.getPlugs()]
+        plugs.sort()
+        print '\nAvailable plugs:\n'
+        for name, p in plugs:
+            print '  %s' % name
         print
     elif len(args) == 2:
         cname = args[1]
+        handled = False
         if r.hasComponent(cname):
+            handled = True
             c = r.getComponent(cname)
-            print '\n%s' % cname
+            print '\nComponent:'
+            print '  %s' % cname
+            desc = c.getDescription()
+            if desc:
+                print '  %s' % desc
             print '\nSource:'
             print '  %s' % c.getSource()
             print '  in %s' % c.getBase()
@@ -106,13 +117,45 @@ def main(args):
             print '\nProperties:'
             if properties:
                 for k, v in properties:
+                    desc = v.getDescription()
                     print ('  %s: type %s, %s%s'
                            % (k, v.getType(),
                               v.isRequired() and 'required' or 'optional',
                               v.isMultiple() and ', multiple ok' or ''))
-        else:
-            err('Unknown component `%s\'' % cname)
+                    if desc:
+                        print '  %s  %s' % (' ' * len(k), desc)
+            sockets = c.getSockets()
+            print '\nClocking:'
+            print '  Needs synchronisation: %r' % c.getNeedsSynchronization()
+            if c.getClockPriority() is not None and c.getNeedsSynchronization():
+                print '  Clock priority: %d' % c.getClockPriority()
+            print '\nSockets:'
+            for socket in sockets:
+                print '  %s' % socket
+            print
+        if r.hasPlug(cname):
+            handled = True
+            p = r.getPlug(cname)
+            print '\nPlug:'
+            print '  %s' % cname
+            print '\nType:'
+            print '  %s' % p.getType()
+            print '\nEntry:'
+            e = p.getEntry()
+            print '  %s() in %s' % (e.getFunction(), e.getModuleName())
+            print '\nProperties:'
+            properties = [(x.getName(), x) for x in p.getProperties()]
+            properties.sort()
+            if properties:
+                for k, v in properties:
+                    print ('  %s: type %s, %s%s'
+                           % (k, v.getType(),
+                              v.isRequired() and 'required' or 'optional',
+                              v.isMultiple() and ', multiple ok' or ''))
+            print
+        if not handled:
+            err('Unknown component or plug `%s\'' % cname)
     else:
-        err('Usage: flumotion-inspect [COMPONENT]')
+        err('Usage: flumotion-inspect [COMPONENT-OR-PLUG]')
 
     return 0
