@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -121,7 +121,7 @@ class TestStateSet(unittest.TestCase):
     def testStateSet(self):
         self.reset()
         # get the state
-        d = self.admin.perspective.callRemote('workerGetState')
+        d = self.admin.remoteRoot.callRemote('workerGetState')
         if weHaveAnOldTwisted():
             state = unittest.deferredResult(d)
 
@@ -132,7 +132,7 @@ class TestStateSet(unittest.TestCase):
             self.failUnlessEqual(state.get('name'), 'lois')
 
             # change state by setting the name
-            d = self.admin.perspective.callRemote('workerSetName', 'clark')
+            d = self.admin.remoteRoot.callRemote('workerSetName', 'clark')
             unittest.deferredResult(d)
 
             self.failUnlessEqual(state.get('name'), 'clark')
@@ -143,7 +143,7 @@ class TestStateSet(unittest.TestCase):
                 self.failUnless(state.hasKey('children'))
                 self.failUnlessEqual(state.get('name'), 'lois')
 
-                d = self.admin.perspective.callRemote('workerSetName', 'clark')
+                d = self.admin.remoteRoot.callRemote('workerSetName', 'clark')
 
                 def workerSetNameCallback(result):
                     self.failUnlessEqual(state.get('name'), 'clark')
@@ -156,14 +156,14 @@ class TestStateSet(unittest.TestCase):
         # change state by appending children
         self.reset()
         # get the state
-        d = self.admin.perspective.callRemote('workerGetState')
+        d = self.admin.remoteRoot.callRemote('workerGetState')
         if weHaveAnOldTwisted():
             state = unittest.deferredResult(d)
             self.failUnless(state)
             self.failUnless(state.hasKey('children'))
 
             # change state by adding children
-            d = self.admin.perspective.callRemote('workerBearChild', 'batman')
+            d = self.admin.remoteRoot.callRemote('workerBearChild', 'batman')
             unittest.deferredResult(d)
 
             self.failUnlessEqual(state.get('children'), ['batman', ])
@@ -172,7 +172,7 @@ class TestStateSet(unittest.TestCase):
                 self.failUnless(state)
                 self.failUnless(state.hasKey('children'))
 
-                d = self.admin.perspective.callRemote('workerBearChild', 
+                d = self.admin.remoteRoot.callRemote('workerBearChild', 
                     'batman')
                 def workerBearChildCallback(result):
                     self.failUnlessEqual(state.get('children'), ['batman', ])
@@ -181,26 +181,20 @@ class TestStateSet(unittest.TestCase):
             d.addCallback(workerGetStateCallback)
             return d
 
-    # listener interface
-    implements(flavors.IStateListener)
-    
-    def stateSet(self, state, key, value):
-        self.changes.append(('set', state, key, value))
-
-    def stateAppend(self, state, key, value):
-        self.changes.append(('append', state, key, value))
-
-    def stateRemove(self, state, key, value):
-        self.changes.append(('remove', state, key, value))
+    def listen(self, state):
+        def event(type):
+            return lambda o, k, v: self.changes.append((type, o, k, v))
+        state.addListener(self, event('set'), event('append'),
+                          event('remove'))
 
     # This is a simple test to show that explicitly del'ing the
     # state causes a decache further up and so no changes to be
     # sent from the Cachable object.  This is only for Twisted 2.x
     def testSimpleStateListener(self):
         def getStateCallback(state):
-            state.addListener(self)
+            self.listen(state)
             self._state = state
-            return self.admin.perspective.callRemote('workerBearChild',
+            return self.admin.remoteRoot.callRemote('workerBearChild',
                                                      'batman')
 
         def workerBearChildCallback(res):
@@ -214,7 +208,7 @@ class TestStateSet(unittest.TestCase):
             del state
 
         self.reset()
-        d = self.admin.perspective.callRemote('workerGetState')
+        d = self.admin.remoteRoot.callRemote('workerGetState')
         d.addCallback(getStateCallback)
         d.addCallback(workerBearChildCallback)
         return d
@@ -226,30 +220,30 @@ class TestStateSet(unittest.TestCase):
     if weHaveAnOldTwisted():
         def testStateListener(self):
             self.reset()
-            d = self.admin.perspective.callRemote('workerGetState')
+            d = self.admin.remoteRoot.callRemote('workerGetState')
             state = unittest.deferredResult(d)
 
-            state.addListener(self)
+            self.listen(state)
 
             self.failUnless(state)
             self.failUnless(state.hasKey('children'))
             self.failIf(self.changes, self.changes)
 
             # change state by adding children
-            d = self.admin.perspective.callRemote('workerBearChild', 'batman')
+            d = self.admin.remoteRoot.callRemote('workerBearChild', 'batman')
             unittest.deferredResult(d)
             c = self.changes.pop()
             self.failUnlessEqual(c, ('append', state, 'children', 'batman'))
             # make sure this is the only change
             self.failIf(self.changes, self.changes)
 
-            d = self.admin.perspective.callRemote('workerBearChild', 'robin')
+            d = self.admin.remoteRoot.callRemote('workerBearChild', 'robin')
             unittest.deferredResult(d)
             c = self.changes.pop()
             self.failUnlessEqual(c, ('append', state, 'children', 'robin'))
             self.failIf(self.changes, self.changes)
 
-            d = self.admin.perspective.callRemote('workerHaveAdopted', 'batman')
+            d = self.admin.remoteRoot.callRemote('workerHaveAdopted', 'batman')
             unittest.deferredResult(d)
             c = self.changes.pop()
             self.failUnlessEqual(c, ('remove', state, 'children', 'batman'))
@@ -259,12 +253,12 @@ class TestStateSet(unittest.TestCase):
     else:
         def testStateListener(self):
             def getStateCallback(state):
-                state.addListener(self)
+                self.listen(state)
                 self._state = state
                 self.failUnless(state)
                 self.failUnless(state.hasKey('children'))
                 self.failIf(self.changes, self.changes)
-                return self.admin.perspective.callRemote('workerBearChild', 'batman')
+                return self.admin.remoteRoot.callRemote('workerBearChild', 'batman')
 
             def workerBearChildCallback(res):
                 state = self._state
@@ -273,7 +267,7 @@ class TestStateSet(unittest.TestCase):
                 self.failUnlessEqual(c, ('append', state, 'children', 'batman'))
                 # make sure this is the only change
                 self.failIf(self.changes, self.changes)
-                return self.admin.perspective.callRemote('workerBearChild', 'robin')
+                return self.admin.remoteRoot.callRemote('workerBearChild', 'robin')
 
             def workerBearChildRobinCallback(res):
                 state = self._state
@@ -281,7 +275,7 @@ class TestStateSet(unittest.TestCase):
                 c = self.changes.pop()
                 self.failUnlessEqual(c, ('append', state, 'children', 'robin'))
                 self.failIf(self.changes, self.changes)
-                return self.admin.perspective.callRemote('workerHaveAdopted', 'batman')
+                return self.admin.remoteRoot.callRemote('workerHaveAdopted', 'batman')
 
             def workerHaveAdoptedCallback(res):
                 state = self._state
@@ -294,7 +288,7 @@ class TestStateSet(unittest.TestCase):
                 del state
 
             self.reset()
-            d = self.admin.perspective.callRemote('workerGetState')
+            d = self.admin.remoteRoot.callRemote('workerGetState')
             d.addCallback(getStateCallback)
             d.addCallback(workerBearChildCallback)
             d.addCallback(workerBearChildRobinCallback)
@@ -308,16 +302,16 @@ class TestStateSet(unittest.TestCase):
         def testStateListenerIntermediate(self):
             self.reset()
             # get the state
-            d = self.admin.perspective.callRemote('workerGetState')
+            d = self.admin.remoteRoot.callRemote('workerGetState')
             state = unittest.deferredResult(d)
-            state.addListener(self)
+            self.listen(state)
 
             self.failUnless(state)
             self.failUnless(state.hasKey('children'))
             self.failIf(self.changes, self.changes)
 
             # change state by adding children
-            d = self.admin.perspective.callRemote('workerBearChild', 'batman')
+            d = self.admin.remoteRoot.callRemote('workerBearChild', 'batman')
             unittest.deferredResult(d)
             c = self.changes.pop()
             self.failUnlessEqual(c, ('append', state, 'children', 'batman'))
@@ -329,19 +323,19 @@ class TestStateSet(unittest.TestCase):
             del state
 
             # get the state again
-            d = self.admin.perspective.callRemote('workerGetState')
+            d = self.admin.remoteRoot.callRemote('workerGetState')
             state = unittest.deferredResult(d)
 
-            state.addListener(self)
+            self.listen(state)
             self.assertEquals(len(state.get('children')), 1)
 
-            d = self.admin.perspective.callRemote('workerBearChild', 'robin')
+            d = self.admin.remoteRoot.callRemote('workerBearChild', 'robin')
             unittest.deferredResult(d)
             c = self.changes.pop()
             self.failUnlessEqual(c, ('append', state, 'children', 'robin'))
             self.failIf(self.changes, self.changes)
 
-            d = self.admin.perspective.callRemote('workerHaveAdopted', 'batman')
+            d = self.admin.remoteRoot.callRemote('workerHaveAdopted', 'batman')
             unittest.deferredResult(d)
             c = self.changes.pop()
             self.failUnlessEqual(c, ('remove', state, 'children', 'batman'))
@@ -351,12 +345,12 @@ class TestStateSet(unittest.TestCase):
     else:
         def testStateListenerIntermediate(self):
             def workerGetStateCallback(state):
-                state.addListener(self)
+                self.listen(state)
                 self.failUnless(state)
                 self.failUnless(state.hasKey('children'))
                 self.failIf(self.changes, self.changes)
                 self._state = state
-                return self.admin.perspective.callRemote('workerBearChild',
+                return self.admin.remoteRoot.callRemote('workerBearChild',
                     'batman')
                         
             def workerBearChildCallback(result):
@@ -371,13 +365,13 @@ class TestStateSet(unittest.TestCase):
                 self.assertEquals(len(state.get('children')), 1)
                 state.removeListener(self)
                 del state
-                return self.admin.perspective.callRemote('workerGetState')
+                return self.admin.remoteRoot.callRemote('workerGetState')
             
             def workerGetStateAgainCallback(state):
-                state.addListener(self)
+                self.listen(state)
                 self.assertEquals(len(state.get('children')), 1)
                 self._state = state
-                return self.admin.perspective.callRemote('workerBearChild',
+                return self.admin.remoteRoot.callRemote('workerBearChild',
                     'robin')
                 
             def workerBearChildAgainCallback(result):
@@ -388,7 +382,7 @@ class TestStateSet(unittest.TestCase):
                     'children', 'robin'))
                 self.failIf(self.changes, self.changes)
                 del state
-                return self.admin.perspective.callRemote(
+                return self.admin.remoteRoot.callRemote(
                     'workerHaveAdopted', 'batman')
                         
             def workerHaveAdoptedCallback(result):
@@ -403,7 +397,7 @@ class TestStateSet(unittest.TestCase):
                 del state
             
             # get the state again
-            d = self.admin.perspective.callRemote('workerGetState')
+            d = self.admin.remoteRoot.callRemote('workerGetState')
             d.addCallback(workerGetStateCallback)
             d.addCallback(workerBearChildCallback)
             d.addCallback(workerGetStateAgainCallback)
@@ -416,11 +410,11 @@ class TestStateSet(unittest.TestCase):
         # to work
         self.reset()
         # get the state
-        d = self.admin.perspective.callRemote('workerGetState')
+        d = self.admin.remoteRoot.callRemote('workerGetState')
         if weHaveAnOldTwisted():
             state = unittest.deferredResult(d)
 
-            state.addListener(self)
+            self.listen(state)
 
             self.failUnless(state)
             self.failUnless(state.hasKey('children'))
@@ -428,19 +422,19 @@ class TestStateSet(unittest.TestCase):
             del state
 
             # change state by adding children
-            d = self.admin.perspective.callRemote('workerBearChild', 'batman')
+            d = self.admin.remoteRoot.callRemote('workerBearChild', 'batman')
             unittest.deferredResult(d)
             self.failIf(self.changes)
         else:
             def workerGetStateCallback(state):
-                state.addListener(self)
+                self.listen(state)
                 self.failUnless(state)
                 self.failUnless(state.hasKey('children'))
 
                 del state
 
                 # change state by adding children
-                d = self.admin.perspective.callRemote('workerBearChild', 'batman')
+                d = self.admin.remoteRoot.callRemote('workerBearChild', 'batman')
                 def workerBearChildCallback(res):
                     self.failIf(self.changes)
                 d.addCallback(workerBearChildCallback)
