@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -50,11 +50,7 @@ class FakeComponent:
 
 def run_it_a_little_while(p):
     p.set_state(gst.STATE_PLAYING)
-    if gst.gst_version < (0, 9):
-        for i in range(10):
-            p.iterate()
-    else:
-        m = p.get_bus().poll(gst.MESSAGE_EOS, -1)
+    m = p.get_bus().poll(gst.MESSAGE_EOS, -1)
     p.set_state(gst.STATE_NULL)
 
 class DeepNotify(unittest.TestCase):
@@ -72,12 +68,13 @@ class TestProperty(unittest.TestCase):
         self.failIf(b)
 
     def testHasPropertyValue(self):
-        b = gstreamer.element_factory_has_property_value('fakesrc', 'num-buffers', 1)
+        b = gstreamer.element_factory_has_property_value(
+            'fakesrc', 'num-buffers', 1)
         self.failUnless(b)
         # setting string values for enums only works in 0.10, or something
-        if gst.gst_version > (0, 9, 0):
-            b = gstreamer.element_factory_has_property_value('fakesrc', 'sizetype', 'fixed')
-            self.failUnless(b)
-            b = gstreamer.element_factory_has_property_value('fakesrc', 'sizetype', 'no-way')
-            self.failIf(b)
-     
+        b = gstreamer.element_factory_has_property_value(
+            'fakesrc', 'sizetype', 'fixed')
+        self.failUnless(b)
+        b = gstreamer.element_factory_has_property_value(
+            'fakesrc', 'sizetype', 'no-way')
+        self.failIf(b)
