@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -134,7 +134,13 @@ class TestErrorDialog(unittest.TestCase):
         self.failUnless(isinstance(button, gtk.Button))
         
         gobject.timeout_add(1 * INTERVAL, lambda b: b.emit('clicked'), button)
-        dialog.run()
-        
-        self.failUnlessEqual(dialog.get_property('visible'), False)
-   
+        d = dialog.run()
+        def check_visibility():
+            self.failUnlessEqual(dialog.get_property('visible'), False)
+            gtk.main_quit()
+        d.addCallback(lambda _: check_visibility())
+        # have to explicitly use the gtk main loop here for some most
+        # terrible reason
+        gtk.main()
+        return d
+    testDialogRun = common.deferred_result(testDialogRun)
