@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -27,8 +27,8 @@ class LogTester(log.Loggable):
     logCategory = 'testlog'
 
 class LogFunctionTester(log.Loggable):
-    def logFunction(self, message):
-        return "override " + message
+    def logFunction(self, format, *args):
+        return (("override " + format),) + args[1:]
 
 class TestLog(unittest.TestCase):
     def setUp(self):
@@ -82,6 +82,15 @@ class TestLog(unittest.TestCase):
         assert self.level == log.WARN
         assert self.message == 'also visible'
   
+    def testFormatStrings(self):
+        log.setFluDebug("testlog:3")
+        log.addLogHandler(self.handler)
+        
+        self.tester.info("%d %s", 42, 'the answer')
+        assert self.category == 'testlog'
+        assert self.level == log.INFO
+        assert self.message == '42 the answer'
+  
     def testLimitedError(self):
         log.setFluDebug("testlog:3")
         log.addLogHandler(self.handler)
