@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -41,7 +41,7 @@ class TestAdminAvatar(unittest.TestCase):
         self.heaven = FakeHeaven(vishnu)
 
     def testHasRemoteReference(self):
-        avatar = admin.AdminAvatar(self.heaven, 'admin')
+        avatar = admin.AdminAvatar(self.heaven, 'admin', None)
         avatar.hasRemoteReference()
 
 if __name__ == '__main__':
