@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -19,7 +19,7 @@
 
 # Headers in this file shall remain intact.
 
-import common
+from common import deferred_result
 
 from twisted.trial import unittest
 
@@ -47,8 +47,9 @@ class MyListener(log.Loggable):
     # a certain state has a certain key set to a certain value
     implements(flavors.IStateListener)
 
-    def __init__(self):
+    def __init__(self, state):
         self._setters = {} # (state, key, value) tuple -> list of deferred
+        state.addListener(self, set=self.stateSet)
         
     def notifyOnSet(self, state, key, value):
         self.debug("notify on state %r key %r set to value %r" % (
@@ -73,12 +74,9 @@ class MyListener(log.Loggable):
         if t in self._setters.keys():
             list = self._setters[t]
             for d in list:
-                self.debug("firing deferred %d" % d)
+                self.debug("firing deferred %s" % d)
                 d.callback(None)
             del self._setters[t]
-
-    def stateAppend(self, object, key, value): pass
-    def stateRemove(self, object, key, value): pass
     
 class FakeComponentAvatar(log.Loggable):
     ### since we fake out componentavatar, eaters need to be specified fully
@@ -87,7 +85,7 @@ class FakeComponentAvatar(log.Loggable):
                  listen_host='127.0.0.1'):
         self.name = name
         self.parent = parent
-        self.avatarId = common.componentPath(name, parent)
+        self.avatarId = common.componentId(parent, name)
         self.eaters = eaters
         self.port = port
         self.listen_host = listen_host
@@ -152,7 +150,7 @@ class TestComponentMapper(unittest.TestCase):
 
         # a componentAvatar gets created with this avatarId
         # lookup mapper and add
-        avatar = component.ComponentAvatar(self.heaven, id)
+        avatar = component.ComponentAvatar(self.heaven, id, None)
         m = self._mappers[id]
         m.avatar = avatar
         # insert an avatar -> mapper ref
@@ -193,7 +191,7 @@ class TestComponentHeaven(unittest.TestCase):
         self.heaven = component.ComponentHeaven(manager.Vishnu('test'))
 
     def testCreateAvatar(self):
-        p = self.heaven.createAvatar('foo-bar-baz')
+        p = self.heaven.createAvatar('foo-bar-baz', None)
         self.failUnless(isinstance(p, component.ComponentAvatar))
 
         #self.assertRaises(AssertionError,
@@ -209,12 +207,12 @@ class TestComponentHeaven(unittest.TestCase):
 
         
     def testGetComponent(self):
-        a = self.heaven.createAvatar('prod')
+        a = self.heaven.createAvatar('prod', None)
         self.assertEqual(self.heaven.getAvatar('prod'), a)
         a.cleanup()
 
     def testHasComponent(self):
-        a = self.heaven.createAvatar('prod')
+        a = self.heaven.createAvatar('prod', None)
         self.failUnless(self.heaven.hasAvatar('prod'))
 
         self.heaven.removeComponent(a)
@@ -251,10 +249,6 @@ class TestComponentHeaven(unittest.TestCase):
         a3 = FakeComponentAvatar(name='baz', port=1001, listen_host='baz-host')
         self.heaven.avatars[a2.avatarId] = a3
 
-        set = self.heaven._getFeederSet(a)
-        set.addFeeders(a2)
-        set.addFeeders(a3)
-
 class FakeTransport:
     def getPeer(self):
         from twisted.internet.address import IPv4Address
@@ -284,8 +278,8 @@ class FakeMind(log.Loggable):
                     name, result))
                 return defer.succeed(result)
             except Exception, e:
-                self.warning('callRemote(%s) failed with %s: %s' % (
-                    name, str(e.__class__), ", ".join(e.args)))
+                self.warning('callRemote(%s) failed with %s' % (
+                    name, log.getExceptionMessage(e)))
                 return defer.fail(e)
         else:
             raise AttributeError('no method %s on self %r' % (name, self))
@@ -299,7 +293,10 @@ class FakeWorkerMind(FakeMind):
         self.avatarId = avatarId
 
     def remote_getPorts(self):
-        return range(7600,7610)
+        return range(7600,7608)
+
+    def remote_getFeedServerPort(self):
+        return 7609
 
     def remote_create(self, avatarId, type, moduleName, methodName, config):
         self.debug('remote_create(%s): logging in component' % avatarId)
@@ -307,6 +304,9 @@ class FakeWorkerMind(FakeMind):
             avatarId, moduleName, methodName, type, config)
         # need to return the avatarId for comparison
         return avatarId
+    
+    def remote_getComponents(self):
+        return []
 
 class FakeComponentMind(FakeMind):
 
@@ -315,6 +315,9 @@ class FakeComponentMind(FakeMind):
     def __init__(self, testcase, workerName, avatarId, type,
         moduleName, methodName, config):
         FakeMind.__init__(self, testcase)
+        self.avatarId = avatarId
+        self.logName = avatarId
+        self.config = None
 
         self.info('Creating component mind for %s' % avatarId)
         state = planet.ManagerJobState()
@@ -323,28 +326,42 @@ class FakeComponentMind(FakeMind):
             'pid': 1,
             'cpu': 0.1,
             'mood': moods.waking.value,
-            'ip': '0.0.0.0',
+            'manager-ip': '0.0.0.0',
             'workerName': workerName,
             'feederNames': [],
-            'eaterNames': [] }
+            'eaterNames': [],
+            'messages': [] }
         self.state = state
 
     def remote_getState(self):
         self.debug('remote_getState: returning %r' % self.state)
         return self.state
 
+    def remote_getConfig(self):
+        return self.config
+
     def remote_provideMasterClock(self, port):
         return ("127.0.0.1", port, 0L)
 
     def remote_setup(self, config):
-        self.debug('remote_setup(%r)' % config)
+        self.debug('remote_setup(%r)', config)
+        self.config = config
 
-    def remote_start(self, eatersData, feedersData, clocking):
-        self.debug('remote_start(%r, %r)' % (eatersData, feedersData))
+    def remote_start(self, clocking):
+        self.debug('remote_start(%r)' % clocking)
         self.testcase.failUnless(hasattr(self, 'state'))
         self.testcase.failUnless(hasattr(self.state, 'observe_set'))
         
         self.state.observe_set('mood', moods.happy.value)
+
+    def remote_eatFrom(self, fullFeedId, host, port):
+        # pretend this works
+        return
+
+    def remote_feedTo(self, componentId, feedId, host, port):
+        # pretend this works
+        return
+    
     
 class TestVishnu(log.Loggable, unittest.TestCase):
 
@@ -360,32 +377,37 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         self._components = {} # id -> avatar
 
     # helper functions
+    def _requestAvatar(self, avatarId, mind, iface, avatarDict):
+        d = self.vishnu.dispatcher.requestAvatar(avatarId, None,
+            mind, pb.IPerspective, iface)
+
+        def got_result((iface, avatar, cleanup)):
+            # hack for cleanup
+            avatar._cleanup = cleanup
+            avatar._mind = mind
+            avatar._avatarId = avatarId
+
+            # trigger attached
+            # twisted 2.2.0 TestCase does not have a runReactor method
+            # and according to twisted changeset 15556 it was always
+            # deprecated
+            from twisted.internet import reactor
+            reactor.iterate()
+            avatarDict[avatarId] = avatar
+            return avatar
+        d.addCallback(got_result)
+        return d
+        
     def _loginWorker(self, avatarId):
         # create a worker and log it in
         # return the avatar
 
         # log in a worker
-        mind = FakeWorkerMind(self, avatarId)
-
-        tuple = self.vishnu.dispatcher.requestAvatar(avatarId, mind,
-            pb.IPerspective, interfaces.IWorkerMedium)
-
-        avatar = tuple[1]
-
-        # hack for cleanup
-        avatar._tuple = tuple
-        avatar._mind = mind
-        avatar._avatarId = avatarId
-
-        # trigger attached
-        # twisted 2.2.0 TestCase does not have a runReactor method
-        # and according to twisted changeset 15556 it was always
-        # deprecated
-        from twisted.internet import reactor
-        reactor.iterate()
-        self._workers[avatarId] = avatar
-        return avatar
-
+        return self._requestAvatar(avatarId,
+                                   FakeWorkerMind(self, avatarId), 
+                                   interfaces.IWorkerMedium,
+                                   self._workers)
+        
     def _loginComponent(self, workerName, avatarId, type, moduleName,
         methodName, config):
         # create a component and log it in
@@ -393,34 +415,18 @@ class TestVishnu(log.Loggable, unittest.TestCase):
 
         mind = FakeComponentMind(self, workerName, avatarId, type,
             moduleName, methodName, config)
-
-        tuple = self.vishnu.dispatcher.requestAvatar(avatarId, mind,
-            pb.IPerspective, interfaces.IComponentMedium)
-
-        avatar = tuple[1]
-
-        # hack for cleanup
-        avatar._tuple = tuple
-        avatar._mind = mind
-        avatar._avatarId = avatarId
-
-        # trigger attached
-        # twisted 2.2.0 TestCase does not have a runReactor method
-        # and according to twisted changeset 15556 it was always
-        # deprecated
-        from twisted.internet import reactor
-        reactor.iterate()
-    
-        self._components[avatarId] = avatar
-        return avatar
+        return self._requestAvatar(avatarId, mind,
+                                   interfaces.IComponentMedium,
+                                   self._components)
 
     def _logoutAvatar(self, avatar):
         # log out avatar
-        logout = avatar._tuple[2]
+        self.debug('_logoutAvatar %r' % avatar)
+        logout = avatar._cleanup
         mind = avatar._mind
         avatarId = avatar._avatarId
 
-        logout(avatar, mind, avatarId)
+        logout(avatarId, avatar, mind)
         
         # trigger detached
         # twisted 2.2.0 TestCase does not have a runReactor method
@@ -433,24 +439,28 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         names = self.vishnu.workerHeaven.state.get('names')
         self.failUnlessEqual(len(names), 0)
 
-        avatar = self._loginWorker('worker')
+        def got_avatar(avatar):
+            # check
+            names = self.vishnu.workerHeaven.state.get('names')
+            self.failUnlessEqual(len(names), 1)
+            self.failUnless('worker' in names)
 
-        # check
-        names = self.vishnu.workerHeaven.state.get('names')
-        self.failUnlessEqual(len(names), 1)
-        self.failUnless('worker' in names)
+            self._logoutAvatar(avatar)
 
-        self._logoutAvatar(avatar)
+            # check
+            names = self.vishnu.workerHeaven.state.get('names')
+            self.failUnlessEqual(len(names), 0)
 
-        # check
-        names = self.vishnu.workerHeaven.state.get('names')
-        self.failUnlessEqual(len(names), 0)
+        d = self._loginWorker('worker')
+        d.addCallback(got_avatar)
+        return d
+    testWorker = deferred_result(testWorker)
 
     def testLoadConfiguration(self):
         __thisdir = os.path.dirname(os.path.abspath(__file__))
         file = os.path.join(__thisdir, 'test.xml')
         
-        self.vishnu.loadConfiguration(file)
+        self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
         s = self.vishnu.state
         
         l = s.get('flows')
@@ -463,7 +473,24 @@ class TestVishnu(log.Loggable, unittest.TestCase):
 
         # FIXME: why a second time ? Maybe to check that reloading doesn't
         # change things ?
-        self.vishnu.loadConfiguration(file)
+        self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+
+        # now lets empty planet and make sure theres nothing in the depgraph
+        d = self.vishnu.emptyPlanet()
+
+        def verifyEmptyDAG(result):
+            # remove workers that components in test.xml depend on
+            self.vishnu._depgraph.removeWorker("streamer")
+            self.vishnu._depgraph.removeWorker("worker")
+            # make sure the depgraph is empty
+            self.assertEqual(self.vishnu._depgraph._dag._nodes,{})
+
+        if weHaveAnOldTwisted():
+            unittest.deferredResult(d)
+            verifyEmptyDAG(None)
+        else:
+            d.addCallback(verifyEmptyDAG)
+            return d
 
     def testConfigBeforeWorker(self):
         # test a config with three components being loaded before the worker
@@ -475,98 +502,148 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         __thisdir = os.path.dirname(os.path.abspath(__file__))
         file = os.path.join(__thisdir, 'test.xml')
         
-        self.vishnu.loadConfiguration(file)
+        def confLoaded(_):
+            # verify component mapper
+            # 3 component states + avatarId's gotten from the config
+            self.assertEqual(len(mappers.keys()), 6)
+            # verify depgraph
+            id = '/testflow/producer-video-test'
+            state = mappers[id].state
+            assert state, state
+            dag = self.vishnu._depgraph._dag
+            o = dag.getOffspringTyped(state, "COMPONENTSTART")
+            names = [s.get('name') for s,t in o]
+            self.failIf('producer-video-test' in names)
+            self.failUnless('converter-ogg-theora' in names)
+            self.failUnless('streamer-ogg-theora' in names)
 
-        # verify component mapper
-        # 3 component states + avatarId's gotten from the config
-        self.assertEqual(len(mappers.keys()), 6)
+            # verify that nothing should be started
+            start = self.vishnu._depgraph.whatShouldBeStarted()
+            # should be nothing because we have no worker
+            assert start == []
+            
+            # log in a worker and verify components get started
+            return self._loginWorker('worker')
 
-        # verify dag edges
-        id = '/testflow/producer-video-test'
-        state = mappers[id].state
-        assert not state, state
-        o = self.vishnu._dag.getOffspring(state)
-        names = [s.get('name') for s in o]
-        self.failIf('producer-video-test' in names)
-        self.failUnless('converter-ogg-theora' in names)
-        self.failUnless('streamer-ogg-theora' in names)
+        def gotWorker(workerAvatar):
+            d = self._verifyConfigAndOneWorker()
+            d.addCallback(lambda _: workerAvatar)
+            return d
         
-        # log in a worker and verify components get started
-        avatar = self._loginWorker('worker')
+        def confChecked(workerAvatar):
+            # log out the producer and verify the mapper
+            id = '/testflow/producer-video-test'
+            avatar = self._components[id]
+            m = mappers[avatar]
 
-        self._verifyConfigAndOneWorker()
+            self._logoutAvatar(avatar)
 
-        # log out the producer and verify the mapper
-        id = '/testflow/producer-video-test'
-        avatar = self._components[id]
-        m = mappers[avatar]
+            #import pprint
+            #pprint.pprint(mappers.keys())
+            self.assertEqual(len(mappers.keys()), 8)
 
-        self._logoutAvatar(avatar)
+            # We logged it out without it doing a clean shutdown, so it should
+            # now be lost.
+            self._verifyComponentIdGone(id, moods.lost)
 
-        #import pprint
-        #pprint.pprint(mappers.keys())
-        self.assertEqual(len(mappers.keys()), 8)
+            # log out the converter and verify
+            id = '/testflow/converter-ogg-theora'
+            m = mappers[id]
+            avatar = self._components[id]
+            # Pretend this one is a clean, requested shutdown.
+            avatar._shutdown_requested = True
+            self._logoutAvatar(avatar)
 
-        self._verifyComponentIdGone(id)
+            # We requested shutdown, so this should now be sleeping.
+            self._verifyComponentIdGone(id, moods.sleeping)
 
-        # log out the converter and verify
-        id = '/testflow/converter-ogg-theora'
-        m = mappers[id]
-        avatar = self._components[id]
-        self._logoutAvatar(avatar)
+            self._verifyConfigAndNoWorker()
 
-        self._verifyConfigAndNoWorker()
-    testConfigBeforeWorker.skip = "Help, thomas..."
+            # Now log out the worker.
+            self._logoutAvatar(workerAvatar)
+
+        d = self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+        d.addCallback(confLoaded)
+        d.addCallback(gotWorker)
+        d.addCallback(confChecked)
+        return d
+    testConfigBeforeWorker = deferred_result(testConfigBeforeWorker)
 
     def testConfigAfterWorker(self):
         # test a config with three components being loaded after the worker
         # logs in
-        mappers = self.vishnu._componentMappers
+        def loadConfigAndOneWorker(workerAvatar):
+            log.debug('unittest', 'loadConfigAndOneWorker')
+            self.failUnlessEqual(len(self._workers), 1)
+            self.failUnlessEqual(len(self._components), 0)
+            
+            # load configuration
+            d = self.vishnu.loadConfigurationXML(file, manager.LOCAL_IDENTITY)
+            d.addCallback(lambda _: self._verifyConfigAndOneWorker())
+            d.addCallback(lambda _: workerAvatar)
+            return d
+        
+        def logoutComponent(workerAvatar):
+            log.debug('unittest', 'logoutComponent: producer')
+            # log out the producer and verify the mapper
+            id = '/testflow/producer-video-test'
+            avatar = self._components[id]
+            m = mappers[avatar]
 
+            self._logoutAvatar(avatar)
+
+            #import pprint
+            #pprint.pprint(mappers.keys())
+            self.assertEqual(len(mappers.keys()), 8)
+
+            # We logged it out without it doing a clean shutdown, so it should
+            # now be lost.
+            self._verifyComponentIdGone(id, moods.lost)
+            
+            # log out the converter and verify
+            log.debug('unittest', 'logoutComponent: converter')
+            id = '/testflow/converter-ogg-theora'
+            m = mappers[id]
+            avatar = self._components[id]
+            # Pretend this one is a clean, requested shutdown.
+            avatar._shutdown_requested = True
+            self._logoutAvatar(avatar)
+
+            self._verifyComponentIdGone(id, moods.sleeping)
+
+            log.debug('unittest', 'logoutComponent: _verifyConfigAndNoWorker')
+            self._verifyConfigAndNoWorker()
+
+            # Now log out the worker.
+            log.debug('unittest', 'logoutComponent: _logoutAvatar')
+            self._logoutAvatar(workerAvatar)
+
+        def emptyPlanet(_):
+            return self.vishnu.emptyPlanet()
+
+        def removeWorkersAndCheckDAG(result):
+            # make sure the depgraph is empty
+            # remove worker 'streamer' that is set as the worker for 
+            # http-streamer in test.xml
+            # and remove worker 'worker' that was logged in, in this test
+            self.vishnu._depgraph.removeWorker("streamer")
+            self.vishnu._depgraph.removeWorker("worker")
+
+            self.assertEqual(self.vishnu._depgraph._dag._nodes,{})
+        def verifyMappersIsZero(result):
+            self.assertEqual(len(mappers.keys()), 0)
+
+        mappers = self.vishnu._componentMappers
         __thisdir = os.path.dirname(os.path.abspath(__file__))
         file = os.path.join(__thisdir, 'test.xml')
-
-        # log in worker
-        avatar = self._loginWorker('worker')
-        self.failUnlessEqual(len(self._workers), 1)
-        self.failUnlessEqual(len(self._components), 0)
-        
-        # load configuration
-        self.vishnu.loadConfiguration(file)
-
-        self._verifyConfigAndOneWorker()
-        
-        # log out the producer and verify the mapper
-        id = '/testflow/producer-video-test'
-        avatar = self._components[id]
-        m = mappers[avatar]
-
-        self._logoutAvatar(avatar)
-
-        #import pprint
-        #pprint.pprint(mappers.keys())
-        self.assertEqual(len(mappers.keys()), 8)
-
-        self._verifyComponentIdGone(id)
-        
-        # log out the converter and verify
-        id = '/testflow/converter-ogg-theora'
-        m = mappers[id]
-        avatar = self._components[id]
-        self._logoutAvatar(avatar)
-
-        self._verifyConfigAndNoWorker()
-
-        # clear out the complete planet
-        d = self.vishnu.emptyPlanet()
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
-            self.assertEqual(len(mappers.keys()), 0)
-        else:
-            def verifyMappersIsZero(result):
-                self.assertEqual(len(mappers.keys()), 0)
-            d.addCallback(verifyMappersIsZero)
-            return d
+        d = self._loginWorker('worker')
+        d.addCallback(loadConfigAndOneWorker)
+        d.addCallback(logoutComponent)
+        d.addCallback(emptyPlanet)
+        d.addCallback(removeWorkersAndCheckDAG)
+        d.addCallback(verifyMappersIsZero)
+        return d
+    testConfigAfterWorker = deferred_result(testConfigAfterWorker)
 
     def _verifyConfigAndOneWorker(self):
         self.debug('verifying after having loaded config and started worker')
@@ -601,26 +678,18 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         self.failUnless(m.jobState)
 
         state = m.jobState
-        l = MyListener()
-        state.addListener(l)
+        l = MyListener(state)
         d = l.notifyOnSet(state, 'mood', moods.happy.value)
-        if weHaveAnOldTwisted():
-            unittest.deferredResult(d)
+
+        def verifyMoodIsHappy(result):
             self.assertEqual(state.get('mood'), moods.happy.value,
-                "mood of %s is not happy but %r" % (
-                    m.state.get('name'), moods.get(state.get('mood'))))
+                             "mood of %s is not happy but %r" % (
+                m.state.get('name'), moods.get(state.get('mood'))))
             # verify the component avatars
             self.failUnless(avatar.jobState)
             self.failUnless(avatar.componentState)
-        else:
-            def verifyMoodIsHappy(result):
-                self.assertEqual(state.get('mood'), moods.happy.value,
-                    "mood of %s is not happy but %r" % (
-                        m.state.get('name'), moods.get(state.get('mood'))))
-                # verify the component avatars
-                self.failUnless(avatar.jobState)
-                self.failUnless(avatar.componentState)
-            d.addCallback(verifyMoodIsHappy)
+        d.addCallback(verifyMoodIsHappy)
+        return d
         
     def _verifyConfigAndNoWorker(self):
         mappers = self.vishnu._componentMappers
@@ -630,7 +699,8 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         self._verifyComponentIdGone('/testflow/converter-ogg-theora')
         self._verifyComponentIdGone('/testflow/producer-video-test')
 
-    def _verifyComponentIdGone(self, id):
+    def _verifyComponentIdGone(self, id, expectedMood=None,
+            expectedMoodPending=None):
         # verify logged out components
         mappers = self.vishnu._componentMappers
         m = mappers[id]
@@ -643,8 +713,21 @@ class TestVishnu(log.Loggable, unittest.TestCase):
         self.assertEqual(m.avatar, None)
         self.failUnless(m.state)
         state = m.state
-        self.assertEqual(state.get('mood'), moods.sleeping.value,
-            'mood is %s instead of sleeping' % moods.get(state.get('mood')))
+        if expectedMood: # Only check this if we had an expected mood passed
+            self.assertEqual(state.get('mood'), expectedMood.value,
+                '%s: mood is %s instead of %s' % (id,
+                    moods.get(state.get('mood')), expectedMood))
+
+        # always check moodPending
+        moodPendingValue = state.get('moodPending')
+        expectedValue = expectedMoodPending
+        if expectedMoodPending is not None:
+            expectedValue = expectedMoodPending.value
+        self.assertEqual(moodPendingValue, expectedValue,
+            '%s: moodPending is %r instead of %r' % (id,
+                moodPendingValue is not None \
+                    and moods.get(moodPendingValue) or 'None',
+                expectedMoodPending))
 
         # verify avatar state
         self.failIf(avatar.jobState)
