@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -21,6 +21,7 @@
 
 import common
 from twisted.trial import unittest
+import twisted.copyright #T1.3
 
 import crypt
 
@@ -34,18 +35,26 @@ from twisted.cred import portal, error
 from flumotion.twisted import checkers, credentials, pb
 from flumotion.twisted import portal as fportal
 from flumotion.twisted.compat import implements
-from flumotion.common import keycards, log
-from flumotion.component.bouncers import htpasswdcrypt
 
-import twisted.copyright #T1.3
+from flumotion.common import keycards, log, interfaces
+from flumotion.component.bouncers import htpasswdcrypt, saltsha256
+
+htpasswdcryptConf = {
+    'name':  'testbouncer',
+    'plugs': {},
+    'properties': {'data': "user:qi1Lftt0GZC0o"}
+}
+
+saltsha256Conf = {
+    'name':  'testbouncer',
+    'plugs': {},
+    'properties': {'data': "user:iamsalt:2f826124ada2b2cdf11f4fd427c9ca48de0ed49b41476266d8df08d2cf86120e"}
+}
 
 #T1.3
 def weHaveAnOldTwisted():
     return twisted.copyright.version < '2.0.0'
 
-bouncerconf = {'name': 'testbouncer',
-               'properties': {'data': "user:qi1Lftt0GZC0o"}}
-
 ### lots of fake objects to have fun with
 
 class FakePortalWrapperPlaintext:
@@ -54,7 +63,7 @@ class FakePortalWrapperPlaintext:
         self.broker = FakeBroker()
         self.checker = tcheckers.InMemoryUsernamePasswordDatabaseDontUse()
         self.checker.addUser("username", "password")
-        self.portal = portal.Portal(FakeRealm(), (self.checker, ))
+        self.portal = portal.Portal(FakeTRealm(), (self.checker, ))
 
 class FakePortalWrapperCrypt:
     # a fake wrapper with a checker that lets username, crypt(password, iq) in
@@ -62,7 +71,7 @@ class FakePortalWrapperCrypt:
         self.checker = checkers.CryptChecker()
         cryptPassword = crypt.crypt('password', 'iq')
         self.checker.addUser("username", cryptPassword)
-        self.portal = portal.Portal(FakeRealm(), (self.checker, ))
+        self.portal = portal.Portal(FakeTRealm(), (self.checker, ))
 
 # FIXME: using real portal
 class FakeBouncerPortal:
@@ -83,16 +92,20 @@ class FakeAvatar(tpb.Avatar):
     def logout(self):
         self.loggedOut = True
 
-class FakeRealm:
+class FakeTRealm:
     def __init__(self):
         self.avatar = FakeAvatar()
-    def requestAvatar(self, avatarId, mind, *interfaces):
-        interface = interfaces[0]
-        assert interface == tpb.IPerspective
+    def requestAvatar(self, avatarId, mind, *ifaces):
+        interface = ifaces[0]
+        assert interface == tpb.IPerspective, "interface is %r and not IPerspective" % interface
         self.avatar.loggedIn = True
         # we can return a deferred, or return directly
         return defer.succeed((tpb.IPerspective, self.avatar, self.avatar.logout))
 
+class FakeFRealm(FakeTRealm):
+    def requestAvatar(self, avatarId, keycard, mind, *interfaces):
+        return FakeTRealm.requestAvatar(self, avatarId, mind, *interfaces)
+
 class FakeMind(tpb.Referenceable):
     pass
 
@@ -118,8 +131,8 @@ class TestTwisted_PortalAuthChallenger(unittest.TestCase):
     def testWrongPassword(self):
         # client is asked to respond, so generate the response
         response = tpb.respond(self.challenge, 'wrong')
-
         d = self.challenger.remote_respond(response, None)
+        
         def wrongPasswordErrback(wrongpasserror):
             self.assert_(isinstance(wrongpasserror.type(), error.UnauthorizedLogin))
 
@@ -136,15 +149,19 @@ class Test_BouncerWrapper(unittest.TestCase):
         broker = FakeBroker()
 
         self.bouncer = htpasswdcrypt.HTPasswdCrypt()
-        self.bouncer.setup(bouncerconf)
-        self.bouncerPortal = fportal.BouncerPortal(FakeRealm(), self.bouncer)
+        self.bouncer.setup(htpasswdcryptConf)
+        self.bouncerPortal = fportal.BouncerPortal(FakeFRealm(), self.bouncer)
         self.wrapper = pb._BouncerWrapper(self.bouncerPortal, broker)
+
+    def tearDown(self):
+        self.bouncer.stop()
         
     def testUACPPOk(self):
         mind = FakeMind()
         keycard = keycards.KeycardUACPP('user', 'test', '127.0.0.1')
         d = self.wrapper.remote_login(keycard, mind,
             'twisted.spread.pb.IPerspective')
+
         def uacppOkCallback(result):
             self.assert_(isinstance(result, tpb.AsReferenceable))
             return result
@@ -159,6 +176,7 @@ class Test_BouncerWrapper(unittest.TestCase):
         keycard = keycards.KeycardUACPP('user', 'tes', '127.0.0.1')
         d = self.wrapper.remote_login(keycard, "avatarId", 
             'twisted.spread.pb.IPerspective')
+        
         def uacppWrongPasswordErrback(wrongpasserror):
             self.assert_(isinstance(wrongpasserror.type(), error.UnauthorizedLogin))
         
@@ -176,6 +194,7 @@ class Test_BouncerWrapper(unittest.TestCase):
         # send
         d = self.wrapper.remote_login(keycard, None, 
             'twisted.spread.pb.IPerspective')
+        
         def uacpccOkCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
             # respond to challenge
@@ -193,7 +212,7 @@ class Test_BouncerWrapper(unittest.TestCase):
             unittest.deferredResult(d)
         else:
             return d
-    
+            
     def testUACPCCWrongUser(self):
         # create
         keycard = keycards.KeycardUACPCC('wronguser', '127.0.0.1')
@@ -201,6 +220,7 @@ class Test_BouncerWrapper(unittest.TestCase):
         # send
         d = self.wrapper.remote_login(keycard, "avatarId", 
             'twisted.spread.pb.IPerspective')
+
         def uacpccWrongUserCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
             # respond to challenge
@@ -228,6 +248,7 @@ class Test_BouncerWrapper(unittest.TestCase):
         # send
         d = self.wrapper.remote_login(keycard, "avatarId",
             'twisted.spread.pb.IPerspective')
+
         def uacpccWrongPasswordCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
             # respond to challenge
@@ -257,6 +278,7 @@ class Test_BouncerWrapper(unittest.TestCase):
         # submit for auth
         d = self.wrapper.remote_login(keycard, "avatarId",
             'twisted.spread.pb.IPerspective')
+
         def uacpccTamperCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
 
@@ -279,7 +301,7 @@ class Test_BouncerWrapper(unittest.TestCase):
 
 class Test_FPortalRoot(unittest.TestCase):
     def setUp(self):
-        self.bouncerPortal = fportal.BouncerPortal(FakeRealm(), 'bouncer')
+        self.bouncerPortal = fportal.BouncerPortal(FakeFRealm(), 'bouncer')
         self.root = pb._FPortalRoot(self.bouncerPortal)
 
     def testRootObject(self):
@@ -287,20 +309,49 @@ class Test_FPortalRoot(unittest.TestCase):
         self.failUnless(isinstance(root, pb._BouncerWrapper))
         self.assertEquals(root.broker, 'a')
 
+class TestAuthenticator(unittest.TestCase):
+    def testIssueNoInfo(self):
+        # not setting any useful auth info on the authenticator does not
+        # allow us to issue a keycard
+        a = pb.Authenticator(username="tarzan")
+        d = a.issue(["flumotion.common.keycards.KeycardUACPP",])
+        d.addCallback(lambda r: self.failIf(r))
+        return d
+
+    def testIssueUACPP(self):
+        # our authenticator by default only does challenge-based keycards
+        a = pb.Authenticator(username="tarzan", password="jane",
+            address="localhost")
+        d = a.issue(["flumotion.common.keycards.KeycardUACPP",])
+        d.addCallback(lambda r: self.failIf(r))
+
+    def testIssueUACPCC(self):
+        a = pb.Authenticator(username="tarzan", password="jane",
+            address="localhost")
+        d = a.issue(["flumotion.common.keycards.KeycardUACPCC",])
+        d.addCallback(lambda r: self.failUnless(isinstance(r,
+            keycards.KeycardUACPCC)))
+        return d
+
 # time for the big kahuna
+# base class so we can use different bouncers
 class Test_FPBClientFactory(unittest.TestCase):
+
     def setUp(self):
-        self.realm = FakeRealm()
-        self.bouncer = htpasswdcrypt.HTPasswdCrypt()
-        self.bouncer.setup(bouncerconf)
+        self.realm = FakeFRealm()
+        self.bouncer = self.bouncerClass()
+        self.bouncer.setup(self.bouncerConf)
         self.portal = fportal.BouncerPortal(self.realm, self.bouncer)
-        self.factory = tpb.PBServerFactory(self.portal, unsafeTracebacks=0)
-        self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
+        self.serverFactory = tpb.PBServerFactory(self.portal,
+            unsafeTracebacks=0)
+        self.port = reactor.listenTCP(0, self.serverFactory,
+            interface="127.0.0.1")
         self.portno = self.port.getHost().port
         # don't output Twisted tracebacks for PB errors we will trigger
         log._getTheFluLogObserver().ignoreErrors(error.UnauthorizedLogin)
 
     def tearDown(self):
+        self.bouncer.stop()
         tlog.flushErrors(error.UnauthorizedLogin)
         log._getTheFluLogObserver().clearIgnores()
         self.port.stopListening()
@@ -316,59 +367,65 @@ class Test_FPBClientFactory(unittest.TestCase):
         if reference:
             return d
 
-    def testUACPPOk(self):
+# test with htpasswdcrypt bouncer first
+class Test_FPBClientFactoryHTPasswdCrypt(Test_FPBClientFactory):
+    bouncerClass = htpasswdcrypt.HTPasswdCrypt
+    bouncerConf = htpasswdcryptConf
+    def testOk(self):
         factory = pb.FPBClientFactory()
-
-        # create
-        keycard = keycards.KeycardUACPP('user', 'test', '127.0.0.1')
+        a = pb.Authenticator(username="user", password="test",
+            address="127.0.0.1")
 
         # send 
-        d = factory.login(keycard, 'MIND')
-        reactor.connectTCP("127.0.0.1", self.portno, factory)
-
-        # get result
-        def uacppOkCallback(result):
+        d = factory.login(a)
+        c = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        
+        def OkCallback(result):
+            # make sure we really used challenge/response keycard
+            self.failUnless(isinstance(factory.keycard, keycards.KeycardUACPCC))
             self.assert_(isinstance(result, tpb.RemoteReference))
             return self.clientDisconnect(factory, result)
         
-        d.addCallback(uacppOkCallback)
+        d.addCallback(OkCallback)
         if weHaveAnOldTwisted():
             result = unittest.deferredResult(d)
         else:
             return d
 
-    def testUACPPWrongPassword(self):
+    def testWrongPassword(self):
         factory = pb.FPBClientFactory()
-        keycard = keycards.KeycardUACPP('user', 'tes', '127.0.0.1')
-        d = factory.login(keycard, 'MIND')
+        a = pb.Authenticator()
+        a = pb.Authenticator(username="user", password="wrong",
+            address="127.0.0.1")
+        d = factory.login(a)
+
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
 
         log.debug("trial", "wait for result")
-        def uacppWrongPasswordErrback(failure):
+        def WrongPasswordErrback(failure):
+            self.failUnless(isinstance(factory.keycard, keycards.KeycardUACPCC))
             # This is a CopiedFailure
             self.assert_(failure.check(
                 "twisted.cred.error.UnauthorizedLogin"))
             log.debug("trial", "got failure %r" % failure)
             c.disconnect()
             return True
-        if weHaveAnOldTwisted(): #T1.3
+        if weHaveAnOldTwisted():
             unittest.deferredError(d)
         else:
-            d.addErrback(uacppWrongPasswordErrback)
+            d.addErrback(WrongPasswordErrback)
             return d
-
-    def testUACPCCOk(self):
+        
+# FIXME: rewrite such that we can enforce a challenger, possibly
+# by setting a property on the bouncer
+    def notestUACPCCOk(self):
         factory = pb.FPBClientFactory()
 
-        # create
-        keycard = keycards.KeycardUACPCC('user', '127.0.0.1')
-
         # send
-        d = factory.login(keycard, 'MIND')
+        d = factory.login(self.authenticator, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-
-        # get result
-        def uacpccOkCallback(keycard):
+        
+        def OkCallback(keycard):
             # get result
             self.assertEquals(keycard.state, keycards.REQUESTING)
             # respond to challenge
@@ -387,37 +444,37 @@ class Test_FPBClientFactory(unittest.TestCase):
         else:
             return d
 
-    def testUACPCCWrongUser(self):
+    def testWrongUser(self):
         factory = pb.FPBClientFactory()
 
         # create
-        keycard = keycards.KeycardUACPCC('wronguser', '127.0.0.1')
+        a = pb.Authenticator(username="wronguser", password="test",
+            address="127.0.0.1")
 
         # send
-        d = factory.login(keycard, 'MIND')
+        d = factory.login(a)
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-        def uacpccWrongUserCallback(keycard):
-            self.assertEquals(keycard.state, keycards.REQUESTING)
-
-            # respond to challenge
-            keycard.setPassword('test')
-            d = factory.login(keycard, 'MIND')
+        
+        def WrongUserCb(keycard):
+            self.fail("Should have returned UnauthorizedLogin")
             
-            def uacpccWrongUserErrback(failure):
-                # find copied failure
-                self.failUnless(failure.check("twisted.cred.error.UnauthorizedLogin"))
-                from twisted.cred.error import UnauthorizedLogin
-                tlog.flushErrors(UnauthorizedLogin)
-                return self.clientDisconnect(factory, None)
-            d.addErrback(uacpccWrongUserErrback)
-            return d
-        d.addCallback(uacpccWrongUserCallback)
-        if weHaveAnOldTwisted(): #T1.3
+        def WrongUserEb(failure):
+            # find copied failure
+            self.failUnless(failure.check(
+                "twisted.cred.error.UnauthorizedLogin"))
+            from twisted.cred.error import UnauthorizedLogin
+            tlog.flushErrors(UnauthorizedLogin)
+            return self.clientDisconnect(factory, None)
+    
+        d.addCallback(WrongUserCb)
+        d.addErrback(WrongUserEb)
+    
+        if weHaveAnOldTwisted():
             unittest.deferredResult(d)
         else:
             return d
 
-    def testUACPCCWrongPassword(self):
+    def notestUACPCCWrongPassword(self):
         factory = pb.FPBClientFactory()
 
         # create
@@ -426,6 +483,7 @@ class Test_FPBClientFactory(unittest.TestCase):
         # send
         d = factory.login(keycard, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        
         def uacpccWrongPasswordCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
 
@@ -443,13 +501,14 @@ class Test_FPBClientFactory(unittest.TestCase):
             
             d.addErrback(uacpccWrongPasswordErrback)
             return d
+        
         d.addCallback(uacpccWrongPasswordCallback)
-        if weHaveAnOldTwisted(): #T1.3
+        if weHaveAnOldTwisted():
             unittest.deferredResult(d)
         else:
             return d
 
-    def testUACPCCTamperWithChallenge(self):
+    def notestUACPCCTamperWithChallenge(self):
         factory = pb.FPBClientFactory()
 
         # create
@@ -460,7 +519,7 @@ class Test_FPBClientFactory(unittest.TestCase):
         # send
         d = factory.login(keycard, 'MIND')
         c = reactor.connectTCP("127.0.0.1", self.portno, factory)
-
+        
         def uacpccTamperCallback(keycard):
             self.assertEquals(keycard.state, keycards.REQUESTING)
 
@@ -485,5 +544,122 @@ class Test_FPBClientFactory(unittest.TestCase):
         else:
             return d
 
+# test with sha256 bouncer
+class Test_FPBClientFactorySaltSha256(Test_FPBClientFactory):
+    bouncerClass = saltsha256.SaltSha256
+    bouncerConf = saltsha256Conf
+
+    def testOk(self):
+        factory = pb.FPBClientFactory()
+        a = pb.Authenticator(username="user", password="test",
+            address="127.0.0.1")
+        # send 
+        d = factory.login(a)
+        c = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        
+        def OkCallback(result):
+            # make sure we really used an SHA256 challenge/response keycard
+            self.failUnless(isinstance(factory.keycard, keycards.KeycardUASPCC))
+            self.assert_(isinstance(result, tpb.RemoteReference))
+            return self.clientDisconnect(factory, result)
+        
+        d.addCallback(OkCallback)
+        if weHaveAnOldTwisted():
+            result = unittest.deferredResult(d)
+        else:
+            return d
+
+    def testWrongPassword(self):
+        factory = pb.FPBClientFactory()
+        a = pb.Authenticator(username="user", password="wrong",
+            address="127.0.0.1")
+        d = factory.login(a)
+
+        c = reactor.connectTCP("127.0.0.1", self.portno, factory)
+
+        log.debug("trial", "wait for result")
+        def WrongPasswordErrback(failure):
+            # make sure we really used an SHA256 challenge/response keycard
+            self.failUnless(isinstance(factory.keycard, keycards.KeycardUASPCC))
+            # This is a CopiedFailure
+            self.assert_(failure.check(
+                "twisted.cred.error.UnauthorizedLogin"))
+            log.debug("trial", "got failure %r" % failure)
+            c.disconnect()
+            return True
+        if weHaveAnOldTwisted():
+            unittest.deferredError(d)
+        else:
+            d.addErrback(WrongPasswordErrback)
+            return d
+        
+    def testWrongUser(self):
+        factory = pb.FPBClientFactory()
+
+        # create
+        a = pb.Authenticator(username="wronguser", password="test",
+            address="127.0.0.1")
+
+        # send
+        d = factory.login(a)
+        c = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        
+        def WrongUserCb(keycard):
+            self.fail("Should have returned UnauthorizedLogin")
+            
+        def WrongUserEb(failure):
+            # find copied failure
+            self.failUnless(failure.check(
+                "twisted.cred.error.UnauthorizedLogin"))
+            from twisted.cred.error import UnauthorizedLogin
+            tlog.flushErrors(UnauthorizedLogin)
+            return self.clientDisconnect(factory, None)
+    
+        d.addCallback(WrongUserCb)
+        d.addErrback(WrongUserEb)
+    
+        if weHaveAnOldTwisted():
+            unittest.deferredResult(d)
+        else:
+            return d
+
+# FIXME: do this with a fake authenticator that tampers with the challenge
+    def notestUACPCCTamperWithChallenge(self):
+        factory = pb.FPBClientFactory()
+
+        # create
+        keycard = keycards.KeycardUACPCC('user', '127.0.0.1')
+        self.assert_(keycard)
+        self.assertEquals(keycard.state, keycards.REQUESTING)
+
+        # send
+        d = factory.login(keycard, 'MIND')
+        c = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        
+        def uacpccTamperCallback(keycard):
+            self.assertEquals(keycard.state, keycards.REQUESTING)
+
+            # mess with challenge, respond to challenge and resubmit
+            keycard.challenge = "I am a h4x0r"
+            keycard.setPassword('test')
+            d = factory.login(keycard, 'MIND')
+            
+            def uacpccTamperErrback(failure):
+                # find copied failure
+                self.failUnless(failure.check(
+                    "twisted.cred.error.UnauthorizedLogin"))
+                from twisted.cred.error import UnauthorizedLogin
+                tlog.flushErrors(UnauthorizedLogin)
+                return self.clientDisconnect(factory, None)
+
+            d.addErrback(uacpccTamperErrback)
+            return d
+        d.addCallback(uacpccTamperCallback)
+        if weHaveAnOldTwisted():
+            unittest.deferredResult(d)
+        else:
+            return d
+
+
 if __name__ == '__main__':
      unittest.main()
