@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -27,8 +27,7 @@ import warnings
 import tempfile
 warnings.filterwarnings('ignore', category=FutureWarning)
 
-from flumotion.common import registry
-from flumotion.common.registry import _istrue
+from flumotion.common import registry, fxml, common
 
 class TestRegistry(unittest.TestCase):
     def setUp(self):
@@ -41,15 +40,15 @@ class TestRegistry(unittest.TestCase):
         self.failUnless(isinstance(reg, registry.ComponentRegistry))
         
     def testIsTrue(self):
-        self.failUnless(_istrue('True'))
-        self.failUnless(_istrue('true'))
-        self.failUnless(_istrue('1'))
-        self.failUnless(_istrue('yes'))
-        self.failIf(_istrue('False') )
-        self.failIf(_istrue('false') )
-        self.failIf(_istrue('0') )
-        self.failIf(_istrue('no') )
-        self.failIf(_istrue('I am a monkey') )
+        self.failUnless(common.strToBool('True'))
+        self.failUnless(common.strToBool('true'))
+        self.failUnless(common.strToBool('1'))
+        self.failUnless(common.strToBool('yes'))
+        self.failIf(common.strToBool('False') )
+        self.failIf(common.strToBool('false') )
+        self.failIf(common.strToBool('0') )
+        self.failIf(common.strToBool('no') )
+        self.failIf(common.strToBool('I am a monkey') )
 
     def testgetMTime(self):
         mtime = registry._getMTime(__file__)
@@ -68,22 +67,19 @@ class TestRegistry(unittest.TestCase):
         self.reg.addFromString("""
 <registry>
   <components>
-    <component name="foo" type="bar">
+    <component type="bar">
     </component>
-    <component name="foobie" type="baz">
+    <component type="baz">
     </component>
   </components>
 </registry>""")
 
         self.failIf(self.reg.isEmpty())
         
-        self.failIf(self.reg.hasComponent('foo'))
         self.failUnless(self.reg.hasComponent('bar'))
         comp1 = self.reg.getComponent('bar')
         self.failUnless(isinstance(comp1, registry.RegistryEntryComponent))
 
-        self.failIf(self.reg.hasComponent('foobie'))
-
         self.failUnless(self.reg.hasComponent('baz'))
         comp2 = self.reg.getComponent('baz')
         self.failUnless(isinstance(comp2, registry.RegistryEntryComponent))
@@ -99,9 +95,9 @@ class TestRegistry(unittest.TestCase):
         self.reg.addFromString("""
 <registry>
   <components>
-    <component name="foobie" type="component">
+    <component type="component">
       <properties>
-        <property name="source" type="string" required="yes" multiple="yes"/>
+        <property name="source" type="string" required="yes" multiple="yes" description="a source property" />
       </properties>
     </component>
   </components>
@@ -114,6 +110,7 @@ class TestRegistry(unittest.TestCase):
         prop = props[0]
         self.assertEquals(prop.getName(), 'source')
         self.assertEquals(prop.getType(), 'string')
+        self.assertEquals(prop.getDescription(), 'a source property')
         self.failUnless(prop.isRequired())
         self.failUnless(prop.isMultiple())
 
@@ -121,7 +118,7 @@ class TestRegistry(unittest.TestCase):
         template = """
 <registry>
   <components>
-    <component name="foobie" type="component">
+    <component type="component">
       <properties>
         %s
       </properties>
@@ -130,22 +127,22 @@ class TestRegistry(unittest.TestCase):
 </registry>"""
 
         property = "<base-name/>"
-        self.assertRaises(registry.XmlParserError,
+        self.assertRaises(fxml.ParserError,
                           self.reg.addFromString, template % property)
 
         property = '<property without-name=""/>'
-        self.assertRaises(registry.XmlParserError,
+        self.assertRaises(fxml.ParserError,
                           self.reg.addFromString, template % property)
 
         property = '<property name="bar" without-type=""/>'
-        self.assertRaises(registry.XmlParserError,
+        self.assertRaises(fxml.ParserError,
                           self.reg.addFromString, template % property)
 
     def testClean(self):
         xml = """
 <registry>
   <components>
-    <component name="foo" type="bar">
+    <component type="bar">
     </component>
   </components>
 </registry>"""
@@ -159,7 +156,7 @@ class TestRegistry(unittest.TestCase):
         xml = """
 <registry>
   <components>
-    <component name="foo" type="bar"></component>
+    <component type="bar"></component>
   </components>
 </registry>"""
         reg.addFromString(xml) 
@@ -169,12 +166,12 @@ class TestRegistry(unittest.TestCase):
         xml = """
 <registry>
   <components>
-    <component name="unique"></component>
+    <component></component>
   </components>
 </registry>"""
-        self.assertRaises(registry.XmlParserError, reg.addFromString, xml)
+        self.assertRaises(fxml.ParserError, reg.addFromString, xml)
         xml = """<registry><components><foo></foo></components></registry>"""
-        self.assertRaises(registry.XmlParserError, reg.addFromString, xml)
+        self.assertRaises(fxml.ParserError, reg.addFromString, xml)
         
     # addFromString does not parse <directory> toplevel entries since they
     # should not be in partial registry files
@@ -182,12 +179,21 @@ class TestRegistry(unittest.TestCase):
         xml = """
 <registry>
   <components>
-    <component type="bar" base="base/dir">
+    <component type="bar" base="base/dir"
+               description="A bar component.">
       <entries>
         <entry type="test/test" location="loc" function="main"/>
       </entries>
     </component>
   </components>
+  <plugs>
+    <plug type="baz" socket="frogger">
+      <entry location="loc" function="main"/>
+      <properties>
+        <property name="qux" type="string" description="a quxy property"/>
+      </properties>
+    </plug>
+  </plugs>
   <bundles>
     <bundle name="test-bundle">
       <dependencies>
@@ -216,7 +222,8 @@ class TestRegistry(unittest.TestCase):
 
   <components>
 
-    <component type="bar" base="base/dir">
+    <component type="bar" base="base/dir"
+               description="A bar component.">
       <source location="None"/>
       <synchronization required="no" clock-priority="100"/>
       <properties>
@@ -228,8 +235,21 @@ class TestRegistry(unittest.TestCase):
 
   </components>
 
+  <plugs>
+
+    <plug type="baz" socket="frogger">
+      <entry location="loc" function="main"/>
+      <properties>
+        <property name="qux" type="string"
+                  description="a quxy property"
+                  required="False" multiple="False"/>
+      </properties>
+    </plug>
+
+  </plugs>
+
   <bundles>
-    <bundle name="test-bundle" project="flumotion">
+    <bundle name="test-bundle" under="pythondir" project="flumotion">
       <dependencies>
         <dependency name="test-dependency"/>
       </dependencies>
@@ -255,19 +275,20 @@ class TestRegistry(unittest.TestCase):
             i = i + 1
             d = datalines.pop()
             t = targetlines.pop()
-            self.assertEquals(t, d, "line %d: '%s' != '%s'" % (i, d, t))
+            self.assertEquals(t, d, "line %d: '%s' != expected '%s'" % (
+                i, d, t))
             
 class TestComponentEntry(unittest.TestCase):
     def setUp(self):
         self.file = registry.RegistryEntryFile('gui-filename', 'type')
         rec = registry.RegistryEntryComponent
-        self.entry = rec('filename', 'type', 'source', 'base', 
-                         ['prop'], [self.file], {}, [], [], False, 100)
-        self.empty_entry = rec('filename', 'type', 'source', 'base',
-                               ['prop'], [], {}, [], [], True, 130)
-        self.multiple_entry = rec('filename', 'type', 'source', 'base', ['prop'],
+        self.entry = rec('filename', 'type', 'source', 'description', 'base', 
+                         ['prop'], [self.file], {}, [], [], False, 100, [])
+        self.empty_entry = rec('filename', 'type', 'source', 'description', 'base',
+                               ['prop'], [], {}, [], [], True, 130, [])
+        self.multiple_entry = rec('filename', 'type', 'source', 'description', 'base', ['prop'],
                                   [self.file, self.file], {}, [], [],
-                                  False, 100)
+                                  False, 100, [])
 
     def testThings(self):
         self.assertEquals(self.entry.getType(), 'type')
@@ -280,6 +301,7 @@ class TestComponentEntry(unittest.TestCase):
         self.assertEquals(self.empty_entry.getClockPriority(), 130)
         self.assertEquals(self.multiple_entry.getNeedsSynchronization(), False)
         self.assertEquals(self.multiple_entry.getClockPriority(), 100)
+        self.assertEquals(self.multiple_entry.getSockets(), [])
 
 def rmdir(root):
     for file in os.listdir(root):
