@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -43,6 +43,19 @@ class TestConfig(unittest.TestCase):
         s = """<worker><invalid-name/></worker>"""
         self.assertRaises(config.ConfigError, parse, s)
 
+    def testParseWorkerRandomFeederPorts(self):
+        s = """<worker><feederports random="yes" /></worker>"""
+        conf = parse(s)
+        self.assertEquals(conf.feederports, [])
+        self.assertEquals(conf.randomFeederports, True)
+
+    def testParseWorkerFeederPorts(self):
+        s = """<worker><feederports random="no">1000-1002</feederports></worker>"""
+        conf = parse(s)
+        self.assertEquals(conf.feederports, [1000, 1001, 1002])
+        self.assertEquals(conf.randomFeederports, False)
+
+
     def testParseManager(self):
         conf = parse("""<worker><manager>
         <host>hostname</host>
