@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -37,27 +37,6 @@ def filterWarnings(namespace, category):
     c = getattr(namespace, category)
     warnings.filterwarnings('ignore', category=c)
 
-def install_reactor(gtk=False):
-    if version[0] >= '2':
-        from twisted.internet import gtk2reactor as Reactor
-    else:
-        from flumotion.twisted import gtk2reactor as Reactor
-
-    Reactor.install(useGtk=gtk)
-
-    # this monkeypatched var exists to let reconnecting factories know
-    # when they should warn about a connection being closed, and when
-    # they shouldn't because the system is shutting down.
-    # 
-    # there is no race condition here -- the reactor doesn't handle
-    # signals until it is run().
-    from twisted.internet import reactor
-    reactor.killed = False
-    def setkilled(killed):
-        reactor.killed = killed
-    reactor.addSystemEventTrigger('before', 'startup', setkilled, False)
-    reactor.addSystemEventTrigger('before', 'shutdown', setkilled, True)
-
 def implementsInterface(object, interface):
     if version[0] < '2':
         from twisted.python import components
@@ -65,6 +44,19 @@ def implementsInterface(object, interface):
     else:
         return interface.providedBy(object)
 
+def implementedBy(object):
+    if version[0] < '2':
+        return getattr(object, '__implements__', ())
+    else:
+        from zope.interface import implementedBy
+        return implementedBy(object)
+
+def isInterface(object):
+    if version[0] < '2':
+        raise NotImplementedError()
+    from zope.interface.interface import InterfaceClass
+    return isinstance(object, InterfaceClass)
+
 if version[0] < '2':
     from twisted.python.components import Interface as OurLovelyInterface
     import sys
