@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -31,7 +31,6 @@ from flumotion.configure import configure
 from flumotion.common import log, planet
 from flumotion.twisted import flavors
 from flumotion.twisted.compat import implements
-
 from flumotion.common.planet import moods
 
 class FluTrayIcon(log.Loggable):
@@ -97,11 +96,11 @@ class FluTrayIcon(log.Loggable):
         
         # get a dictionary of components
         self._components = components
-        names = components.keys()
-
-        for name in names:
-            component = components[name]
-            component.addListener(self)
+        for component in components.values():
+            try:
+                component.addListener(self, self.stateSet)
+            except KeyError:
+                pass
 
         self._update_mood()
 
@@ -164,12 +163,6 @@ class FluTrayIcon(log.Loggable):
             if self._tray_container:
                 self._tray_container.send_message(1000, value)
 
-    # FIXME: we might want to show messages
-    def stateAppend(self, state, key, value):
-        pass
-    def stateRemove(self, state, key, value):
-        pass
-
     def _show_popup_menu(self):
         self.popupMenu = gtk.Menu()
         self.popupMenuQuititem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
