@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -42,10 +42,10 @@ VideoDevice = enum.EnumClass('VideoDevice',
                               'Firewire'
                              ),
                         component_type=(
-                                        'videotest',
-                                        'web-camera',
-                                        'tv-card',
-                                        'firewire'
+                                        'videotest-producer',
+                                        'webcam-producer',
+                                        'tvcard-producer',
+                                        'firewire-producer'
                                         ),
                         element_names=(
                                        ('videotestsrc',),
@@ -66,9 +66,9 @@ AudioDevice = enum.EnumClass('AudioDevice',
                               'Unused'
                              ),
                         component_type=(
-                                        'audiotest',
-                                        'soundcard',
-                                        'firewire'
+                                        'audiotest-producer',
+                                        'soundcard-producer',
+                                        'firewire-producer'
                                         ))
 # TVCard
 TVCardDevice = enum.EnumClass('TVCardDevice', ('/dev/video0',
@@ -106,8 +106,11 @@ SoundcardInput = enum.EnumClass('SoundcardInput',
                            ('Line in', 'Microphone', 'CD'))
 SoundcardChannels = enum.EnumClass('SoundcardChannels', ('Stereo', 'Mono'),
                               intvalue=(2, 1))
-SoundcardSamplerate = enum.EnumClass('SoundcardSamplerate', ('44100',
+SoundcardSamplerate = enum.EnumClass('SoundcardSamplerate', ('48000',
+                                                        '44100',
+                                                        '32000',
                                                         '22050',
+                                                        '16000',
                                                         '11025',
                                                         '8000'))
 SoundcardBitdepth = enum.EnumClass('SoundcardBitdepth', ('16', '8'),
