@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -81,7 +81,11 @@ class WizardStep(GladeWidget, log.Loggable):
         for w in self.iterate_widgets():
             if hasattr(w, 'get_state') and w != self:
                 # only fgtk widgets implement get_state
-                key = w.get_name().split('_', 1)[1]
+                # every widget that implements get_state automatically becomes
+                # a property
+                # spinbutton_some_property -> some-property 
+                name = '-'.join(w.get_name().split('_'))
+                key = name.split('-', 1)[1]
                 state_dict[key] = w.get_state()
 
         return state_dict
@@ -117,6 +121,15 @@ class WizardStep(GladeWidget, log.Loggable):
         def callback(result):
             self.debug('workerRun callbacked a result')
             self.clear_msg(function)
+
+            if not isinstance(result, messages.Result):
+                msg = messages.Error(T_(
+                    N_("Internal error: could not run check code on worker.")),
+                    debug=('function %r returned a non-Result %r'
+                           % (function, result)))
+                self.add_msg(msg)
+                raise errors.RemoteRunError(function, 'Internal error.')
+
             for m in result.messages:
                 self.debug('showing msg %r' % m)
                 self.add_msg(m)
