@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -28,9 +28,7 @@ from flumotion.twisted import flavors
 from flumotion.twisted.compat import implements
 
 class WorkerListStore(gtk.ListStore):
-
     implements(flavors.IStateListener)
-
     gsignal('changed')
 
     def __init__(self, whs):
@@ -38,7 +36,8 @@ class WorkerListStore(gtk.ListStore):
         for x in whs.get('names'):
             i = self.append()
             self.set_value(i, 0, x)
-        whs.addListener(self)
+        whs.addListener(self, append=self.stateAppend,
+                        remove=self.stateRemove)
 
     def stateAppend(self, state, key, val):
         if key == 'names':
