@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -23,10 +23,7 @@ from flumotion.worker.checks import check
 
 import gst
 
-if gst.gst_version[1] == 8:
-    from video08 import *
-else:
-    from video010 import *
+from video010 import *
 
 def checkTVCard(device, id='check-tvcard'):
     """
@@ -75,13 +72,8 @@ def checkWebcam(device, id):
         caps = element.get_pad("src").get_negotiated_caps()
         log.debug('check', 'negotiated caps: %s' % caps.to_string())
         s = caps[0]
-        # before 0.10 framerate was a double
-        if gst.gst_version[0] == 0 and gst.gst_version[1] < 9:
-            num = int(s['framerate'] * 16)
-            denom = 16
-        else:
-            num = s['framerate'].num
-            denom = s['framerate'].denom
+        num = s['framerate'].num
+        denom = s['framerate'].denom
 
         d = {
             'mime': s.get_name(),
@@ -106,7 +98,8 @@ def checkWebcam(device, id):
 
     pipeline = 'v4lsrc name=source device=%s %s ! fakesink' % (device,
         autoprobe)
-    d = do_element_check(pipeline, 'source', get_device_name, state=gst.STATE_PAUSED)
+    d = do_element_check(pipeline, 'source', get_device_name, 
+        state=gst.STATE_PAUSED, set_state_deferred=True)
 
     d.addCallback(check.callbackResult, result)
     d.addErrback(check.errbackNotFoundResult, result, id, device)
