@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -24,9 +24,9 @@ import os
 import gobject
 import gst
 import gst.interfaces
+from twisted.internet.threads import deferToThread
 
-from twisted.internet import defer, reactor
-
+from twisted.internet import defer
 from flumotion.common import gstreamer, errors, log, messages
 from flumotion.twisted import defer as fdefer
 
@@ -48,7 +48,8 @@ class BusResolution(fdefer.Resolution):
             self.pipeline.set_state(gst.STATE_NULL)
             self.pipeline = None
 
-def do_element_check(pipeline_str, element_name, check_proc, state=None):
+def do_element_check(pipeline_str, element_name, check_proc, state=None,
+    set_state_deferred=False):
     """
     Parse the given pipeline and set it to the given state.
     When the bin reaches that state, perform the given check function on the
@@ -57,7 +58,11 @@ def do_element_check(pipeline_str, element_name, check_proc, state=None):
     @param pipeline_str: description of the pipeline used to test
     @param element_name: name of the element being checked
     @param check_proc: a function to call with the GstElement as argument.
-
+    @param state: an unused keyword parameter that will be removed when
+    support for GStreamer 0.8 is dropped.
+    @param set_state_deferred: a flag to say whether the set_state is run in
+    a deferToThread
+    @type set_state_deferred: bool
     @returns: a deferred that will fire with the result of check_proc, or
               fail.
     @rtype: L{twisted.internet.defer.Deferred}
@@ -68,12 +73,12 @@ def do_element_check(pipeline_str, element_name, check_proc, state=None):
             retval = check_proc(element)
             resolution.callback(retval)
         except check.CheckProcError, e:
-            log.debug('CheckProcError when running %r: %r' % (check_proc,
-                e.data))
+            log.debug('check', 'CheckProcError when running %r: %r',
+                      check_proc, e.data)
             resolution.errback(errors.RemoteRunError(e.data))
         except Exception, e:
-            log.debug('Unhandled exception while running %r: %r' % (check_proc,
-                e))
+            log.debug('check', 'Unhandled exception while running %r: %r',
+                      check_proc, e)
             resolution.errback(errors.RemoteRunError(
                 log.getExceptionMessage(e)))
         # set pipeline state to NULL so worker does not consume
@@ -112,6 +117,7 @@ def do_element_check(pipeline_str, element_name, check_proc, state=None):
     log.debug('check', 'parsing pipeline %s' % pipeline_str)
     try:
         pipeline = gst.parse_launch(pipeline_str)
+        log.debug('check', 'parsed pipeline %s' % pipeline_str)
     except gobject.GError, e:
         resolution.errback(errors.GStreamerError(e.message))
         return resolution.d
@@ -122,10 +128,16 @@ def do_element_check(pipeline_str, element_name, check_proc, state=None):
 
     resolution.watch_id = watch_id
     resolution.pipeline = pipeline
-    
-    pipeline.set_state(gst.STATE_PLAYING)
-
-    return resolution.d
+    log.debug('check', 'setting state to playing')
+    if set_state_deferred:
+        d = deferToThread(pipeline.set_state, gst.STATE_PLAYING)
+        def stateChanged(res):
+            return resolution.d
+        d.addCallback(stateChanged)
+        return d
+    else:
+        pipeline.set_state(gst.STATE_PLAYING)
+        return resolution.d
 
 def check1394(id):
     """
@@ -155,7 +167,7 @@ def check1394(id):
         h = s['height']
         par = s['pixel-aspect-ratio']
         result = dict(width=w, height=h, par=(par.num, par.denom))
-        log.debug('returning dict %r' % result)
+        log.debug('check', 'returning dict %r' % result)
         return result
         
     # first check if the obvious device node exists
@@ -180,7 +192,7 @@ def check1394(id):
                     # dv1394src was fixed after gst-plugins-good 0.10.2
                     # to distinguish NOT_FOUND and OPEN_READ
                     version = gstreamer.get_plugin_version('1394')
-                    if version >= (0, 10, 0) and version <= (0, 10, 2):
+                    if version >= (0,10,0,0) and version <= (0,10,2,0):
                         m = messages.Error(T_(
                             N_("Could not find or open the Firewire device. "
                                "Check the device node and its permissions.")))
@@ -206,4 +218,3 @@ def check1394(id):
     d.addErrback(errbackResult)
 
     return d
-
