@@ -2,7 +2,7 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# Copyright (C) 2004,2005,2006,2007 Fluendo, S.L. (www.fluendo.com).
 # All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
@@ -27,7 +27,7 @@ import os
 from xml.dom import minidom, Node
 from xml.parsers import expat
 
-from flumotion.common import log
+from flumotion.common import log, config
 
 class ConfigError(Exception):
     pass
@@ -54,6 +54,7 @@ class WorkerConfigXML(log.Loggable):
         self.authentication = None
         self.feederports = None
         self.fludebug = None
+        self.randomFeederports = False
 
         try:
             if filename != None:
@@ -96,7 +97,8 @@ class WorkerConfigXML(log.Loggable):
             elif node.nodeName == 'authentication':
                 self.authentication = self.parseAuthentication(node)
             elif node.nodeName == 'feederports':
-                self.feederports = self.parseFeederports(node)
+                self.feederports, self.randomFeederports = \
+                    self.parseFeederports(node)
             elif node.nodeName == 'debug':
                 self.fludebug = str(node.firstChild.nodeValue)
             else:
@@ -118,7 +120,10 @@ class WorkerConfigXML(log.Loggable):
                 continue
 
             if child.nodeName == "host":
-                host = str(child.firstChild.nodeValue)
+                if child.firstChild:
+                    host = str(child.firstChild.nodeValue)
+                else:
+                    host = 'localhost'
             elif child.nodeName == "port":
                 try:
                     port = int(child.firstChild.nodeValue)
@@ -157,12 +162,33 @@ class WorkerConfigXML(log.Loggable):
         return ConfigEntryAuthentication(username, password)
         
     def parseFeederports(self, node):
-        # returns a list of allowed port numbers
-        # <feederports>[lower]-[upper]</feederports>
+        """
+        Returns a list of feeder ports to use (possibly empty),
+        and whether or not to use random feeder ports.
 
-        value = str(node.firstChild.nodeValue)
-        if not '-' in value:
-            raise ConfigError("feederports '%s' does not contain '-'" % value)
-        (lower, upper) = value.split('-')
-        ports = range(int(lower), int(upper) + 1)
-        return ports
+        @rtype: (list, bool)
+        """
+        # returns a list of allowed port numbers
+        # port := int
+        # port-range := port "-" port
+        # port-term := port | port-range
+        # port-list := "" | port-term | port-term "," port-list
+        # <feederports>port-list</feederports>
+        random = False
+        if node.hasAttribute('random'):
+            random = node.getAttribute('random') in config.BOOL_TRUE_VALUES
+        ports = []
+        if not node.firstChild:
+            return (ports, random)
+        terms = str(node.firstChild.nodeValue).split(',')
+        for term in terms:
+            if '-' in term:
+                (lower, upper) = [int(x) for x in term.split('-')]
+                for port in range(lower, upper+1):
+                    if port not in ports:
+                        ports.append(port)
+            else:
+                port = int(term)
+                if port not in ports:
+                    ports.append(port)
+        return (ports, random)
